Encoding: UTF-8
Type: Package
Package: susieR
Title: Sum of Single Effects Linear Regression
Description: Implements methods for variable selection in linear
    regression based on the "Sum of Single Effects" (SuSiE) model, as
    described in Wang et al (2020) <DOI:10.1101/501114> and Zou et al
    (2021) <DOI:10.1101/2021.11.03.467167>. These methods provide
    simple summaries, called "Credible Sets", for accurately
    quantifying uncertainty in which variables should be selected.
    The methods are motivated by genetic fine-mapping applications,
    and are particularly well-suited to settings where variables are
    highly correlated and detectable effects are sparse. The fitting
    algorithm, a Bayesian analogue of stepwise selection methods
    called "Iterative Bayesian Stepwise Selection" (IBSS), is simple
    and fast, allowing the SuSiE model be fit to large data sets
    (thousands of samples and hundreds of thousands of variables).
Date: 2025-06-04
Version: 0.14.2
Authors@R: c(person("Gao","Wang",role="aut",email="wang.gao@columbia.edu"),
             person("Yuxin","Zou",role="aut"),
             person("Kaiqian","Zhang",role="aut"),
             person("Peter","Carbonetto",role=c("aut","cre"),
                    email="peter.carbonetto@gmail.com"),
	     person("Matthew","Stephens",role="aut"))
URL: https://github.com/stephenslab/susieR
BugReports: https://github.com/stephenslab/susieR/issues
License: BSD_3_clause + file LICENSE
Depends: R (>= 3.0.0)
Imports: methods, graphics, grDevices, stats, Matrix, matrixStats,
        mixsqp, reshape, crayon, ggplot2
Suggests: curl, testthat, microbenchmark, knitr, rmarkdown, Rfast,
        cowplot
LazyData: yes
LazyDataCompression: xz
NeedsCompilation: no
RoxygenNote: 7.2.3
VignetteBuilder: knitr
Packaged: 2025-06-04 13:51:52 UTC; pcarbo
Author: Gao Wang [aut],
  Yuxin Zou [aut],
  Kaiqian Zhang [aut],
  Peter Carbonetto [aut, cre],
  Matthew Stephens [aut]
Maintainer: Peter Carbonetto <peter.carbonetto@gmail.com>
Repository: CRAN
Date/Publication: 2025-06-05 08:00:02 UTC
Built: R 4.6.0; ; 2026-04-28 01:14:24 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(coef,susie)
S3method(predict,susie)
S3method(print,summary.susie)
S3method(summary,susie)
export(coef.susie)
export(compute_ss)
export(compute_suff_stat)
export(estimate_s_rss)
export(get_cs_correlation)
export(kriging_rss)
export(predict.susie)
export(print.summary.susie)
export(summary.susie)
export(susie)
export(susie_auto)
export(susie_get_cs)
export(susie_get_lfsr)
export(susie_get_niter)
export(susie_get_objective)
export(susie_get_pip)
export(susie_get_posterior_mean)
export(susie_get_posterior_samples)
export(susie_get_posterior_sd)
export(susie_get_prior_variance)
export(susie_get_residual_variance)
export(susie_init_coef)
export(susie_plot)
export(susie_plot_changepoint)
export(susie_plot_iteration)
export(susie_rss)
export(susie_suff_stat)
export(susie_trendfilter)
export(univariate_regression)
importFrom(Matrix,colMeans)
importFrom(Matrix,colSums)
importFrom(Matrix,crossprod)
importFrom(Matrix,diag)
importFrom(Matrix,rowSums)
importFrom(Matrix,sparseMatrix)
importFrom(Matrix,summary)
importFrom(Matrix,t)
importFrom(Matrix,tcrossprod)
importFrom(crayon,combine_styles)
importFrom(ggplot2,aes_string)
importFrom(ggplot2,annotate)
importFrom(ggplot2,geom_abline)
importFrom(ggplot2,geom_col)
importFrom(ggplot2,geom_line)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,ggtitle)
importFrom(ggplot2,labs)
importFrom(ggplot2,theme_bw)
importFrom(ggplot2,theme_classic)
importFrom(grDevices,dev.off)
importFrom(grDevices,pdf)
importFrom(graphics,legend)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,points)
importFrom(graphics,segments)
importFrom(matrixStats,colSds)
importFrom(methods,as)
importFrom(mixsqp,mixsqp)
importFrom(reshape,melt)
importFrom(stats,.lm.fit)
importFrom(stats,coef)
importFrom(stats,cor)
importFrom(stats,cov2cor)
importFrom(stats,dnorm)
importFrom(stats,lm)
importFrom(stats,median)
importFrom(stats,optim)
importFrom(stats,optimize)
importFrom(stats,pnorm)
importFrom(stats,rmultinom)
importFrom(stats,rnorm)
importFrom(stats,sd)
importFrom(stats,summary.lm)
importFrom(stats,uniroot)
importFrom(stats,var)
importFrom(utils,head)
importFrom(utils,modifyList)

# susieR

[![CRAN status badge](https://www.r-pkg.org/badges/version/susieR)](https://cran.r-project.org/package=susieR)
[![Travis Build Status](https://travis-ci.org/stephenslab/susieR.svg?branch=master)](https://app.travis-ci.com/stephenslab/susieR)
[![Appveyor Build status](https://ci.appveyor.com/api/projects/status/tcgeqxd8q8krija6?svg=true)](https://ci.appveyor.com/project/pcarbo/susier)
[![CircleCI](https://dl.circleci.com/status-badge/img/gh/stephenslab/susieR/tree/master.svg?style=svg)](https://app.circleci.com/pipelines/github/stephenslab/susieR?branch=master)
[![codecov](https://codecov.io/gh/stephenslab/susieR/branch/master/graph/badge.svg)](https://app.codecov.io/gh/stephenslab/susieR)

The `susieR` package implements a simple new way to perform variable
selection in multiple regression ($y=Xb+e$). The methods implemented
here are particularly well-suited to settings where some of the X
variables are highly correlated, and the true effects are highly
sparse (e.g. <20 non-zero effects in the vector $b$).  One example of
this is genetic fine-mapping applications, and this application was a
major motivation for developing these methods. However, the methods
should also be useful more generally.

The methods are based on a new model for sparse multiple regression,
which we call the "Sum of Single Effects" (SuSiE) model.  This model,
which will be described in a manuscript in preparation (Wang et al),
lends itself to a particularly simple and intuitive fitting procedure
-- effectively a Bayesian modification of simple forward selection,
which we call "Iterative Bayesian Step-wise Selection".

The output of the fitting procedure is a number of "Credible Sets"
(CSs), which are each designed to have high probability to contain a
variable with non-zero effect, while at the same time being as small
as possible. You can think of the CSs as being a set of "highly
correlated" variables that are each associated with the response: you
can be confident that one of the variables has a non-zero coefficient,
but they are too correlated to be sure which one.

The package is developed by Gao Wang, Peter Carbonetto, Yuxin Zou,
Kaiqian Zhang, and Matthew Stephens from the
[Stephens Lab](https://stephenslab.uchicago.edu) at the University of
Chicago.

Please
[post issues](https://github.com/stephenslab/susieR/issues) to ask
questions, get our support or provide us feedback; please
[send pull requests](https://github.com/stephenslab/susieR/pulls) if
you have helped fixing bugs or making improvements to the source code.

## Quick Start

Install susieR from [CRAN](https://cran.r-project.org/package=susieR):

```R
install.packages("susieR")
```

Alternatively, install the latest development version of `susieR`
from GitHub:

```R
# install.packages("remotes")
remotes::install_github("stephenslab/susieR")
```

See [here](https://stephenslab.github.io/susieR/articles/mwe.html) for
a brief illustration of `susieR`. For more documentation and examples
please visit https://stephenslab.github.io/susieR

## Citing this work

If you find the `susieR` package or any of the source code in this
repository useful for your work, please cite:

> G. Wang, G., Sarkar, A., Carbonetto, P. & Stephens, M. (2020). A
> simple new approach to variable selection in regression, with
> application to genetic fine mapping. *Journal of the Royal
> Statistical Society, Series B* **82**, 1273–1300.
> https://doi.org/10.1111/rssb.12388

If you use any of the summary data methods such as `susie_suff_stat`
or `susie_rss`, please also cite:

> Zou, Y., Carbonetto, P., Wang, G. & Stephens, M. (2022). Fine-mapping
> from summary data with the "Sum of Single Effects" model. *PLoS
> Genetics* **18**, e1010299. https://doi.org/10.1371/journal.pgen.1010299

## Developer notes

+ When any changes are made to `roxygen2` markup, run
`devtools::document()` to update package `NAMESPACE` and documentation
files.

+ Run `pkgdown::build_site()` to build the website. Getting `pkgdown`
to work properly can be frustrating due to numerous & fragile dependencies.
If `pkgdown` does not work for you out of the box you can use this `docker`
command to run all vignettes and build the site:

```bash
docker run --rm --security-opt label:disable -t -P -w $PWD -v $PWD:$PWD \
  -u $UID:${GROUPS[0]} -e HOME=/home/$USER -e USER=$USER gaow/susie \
  R --slave -e "pkgdown::build_site(lazy=TRUE, examples=FALSE)"
```

[susie-preprint]: https://doi.org/10.1101/501114

#!/usr/bin/env Rscript
library(dplyr)
library(readr)
library(magrittr)

#' CAVIAR I/O
write_caviar_sumstats <- function(z, prefix) {
  cfg = list(z=paste0(prefix,".z"),
             set=paste0(prefix,"_set"),
             post=paste0(prefix,"_post"),
             log=paste0(prefix,".log"))
  write.table(z,cfg$z,quote=F,col.names=F)
  return(cfg)
}

#' Run CAVIAR
#' https://github.com/fhormoz/caviar

run_caviar <- function(z, LD_file, args = "", prefix="data")
{
  cfg = write_caviar_sumstats(z, prefix)
  cmd = paste("CAVIAR", "-z", cfg$z, "-l", LD_file, "-o", prefix, args)
  dscrutils::run_cmd(cmd)
  if(!all(file.exists(cfg$post, cfg$set, cfg$log))) {
      stop("Cannot find one of the post, set, and log files")
  }
  
  log <- readLines(cfg$log)

  # read output tables
  snp <- read.delim(cfg$post)  
  stopifnot(ncol(snp) == 3)
  names(snp) <- c("snp", "snp_prob_set", "snp_prob")
  snp$snp <- as.character(snp$snp)
  snp <- rank_snp(snp)

  # `set` of snps
  set <- readLines(cfg$set)
  set_ordered <- left_join(data_frame(snp = set), snp, by = "snp") %>% 
    arrange(rank) %$% snp
  return(list(snp=snp, set=set_ordered))
}

rank_snp <- function(snp) {
  snp <- arrange(snp, -snp_prob) %>%
    mutate(
        rank = seq(1, n()),
        snp_prob_cumsum = cumsum(snp_prob) / sum(snp_prob)) %>%
    select(rank, snp, snp_prob, snp_prob_cumsum, snp_prob_set)
  return(snp)    
}

finemap_mcaviar <- function(zscore, LD_file, args, prefix) {
  if (is.null(dim(zscore))) {
      zscore = matrix(ncol=1,zscore)
  }
  return(parallel::mclapply(1:ncol(zscore), function(r)
          run_caviar(zscore[,r], LD_file, args, 
                     paste0(prefix, '_condition_', r)), 
                            mc.cores = min(8, ncol(zscore))))
}

eval(parse(text=commandArgs(T)))
dat = readRDS(input)
sumstats = dat$sumstats
ld = tempfile(fileext = ".ld")
write.table(cor(dat$data$X),ld,quote=F,col.names=F,row.names=F)
posterior = finemap_mcaviar(sumstats[1,,] / sumstats[2,,],
                            ld,
                            args, prefix=tempfile(fileext = ".caviar"))
saveRDS(posterior, paste0(output, '.rds'))

#!/usr/bin/env Rscript
library(dplyr)
library(readr)
library(magrittr)

#' FINEMAP I/O
write_finemap_sumstats <- function(z, LD_file, n, k, prefix) {
  cfg = list(z=paste0(prefix,".z"),
             ld=LD_file,
             snp=paste0(prefix,".snp"),
             config=paste0(prefix,".config"),
             k=paste0(prefix,".k"),
             log=paste0(prefix,".log"),
             meta=paste0(prefix,".master"))
  write.table(z,cfg$z,quote=F,col.names=F)
  if (!is.null(k)) {
      write.table(t(k),cfg$k,quote=F,col.names=F,row.names=F)
      write("z;ld;snp;config;k;log;n-ind",file=cfg$meta)
      write(paste(cfg$z, cfg$ld, cfg$snp, cfg$config, cfg$k, cfg$log, n, sep=";"),
        file=cfg$meta,append=TRUE)
  } else {
      write("z;ld;snp;config;log;n-ind",file=cfg$meta)
      write(paste(cfg$z, cfg$ld, cfg$snp, cfg$config, cfg$log, n, sep=";"),
            file=cfg$meta,append=TRUE)
  }
  return(cfg)
}

#' Run FINEMAP version 1.1
#' http://www.christianbenner.com
## FIXME: read the finemapr implementation for data sanity check.
## Can be useful as a general data sanity checker (in previous modules)

run_finemap <- function(z, LD_file, n, k, args = "", prefix="data")
{
  cfg = write_finemap_sumstats(z, LD_file, n, k, prefix)
  cmd = paste("finemap --sss --log", "--in-files", cfg$meta, args)
  dscrutils::run_cmd(cmd)

  # read output tables
  snp = read.table(cfg$snp,header=TRUE,sep=" ")
  snp$snp = as.character(snp$snp)

  snp = rank_snp(snp)
  config = read.table(cfg$config,header=TRUE,sep=" ")

  # Only keep configurations with cumulative 95% probability
  # config = within(config, config_prob_cumsum <- cumsum(config_prob))
  # config = config[config$config_prob_cumsum <= 0.95,]

  # extract number of causal
  ncausal = finemap_extract_ncausal(cfg$log)
  return(list(snp=snp, set=config, ncausal=ncausal))
}

rank_snp <- function(snp) {
  snp <- arrange(snp, -snp_prob) %>%
    mutate(
        rank = seq(1, n()),
        snp_prob_cumsum = cumsum(snp_prob) / sum(snp_prob)) %>%
    select(rank, snp, snp_prob, snp_prob_cumsum, snp_log10bf)
  return(snp)
}

finemap_extract_ncausal <- function(logfile)
{
  lines <- grep("->", readLines(logfile), value = TRUE)
  lines <- gsub("\\(|\\)|>", "", lines)
  splits <- strsplit(lines, "\\s+")
  tab <- data.frame(
    ncausal_num = sapply(splits, function(x) as.integer(x[2])),
    ncausal_prob = sapply(splits, function(x) as.double(x[4])))
  tab <- mutate(tab, type = ifelse(duplicated(ncausal_num), "post", "prior"))
  return(tab)
}

finemap_mvar <- function(zscore, LD_file, n, k, args, prefix, parallel = FALSE) {
  if (is.null(dim(zscore))) {
      zscore = matrix(ncol=1,zscore)
  }

  single_core = function(r) 
      run_finemap(zscore[,r], LD_file, n, k, args, 
                  paste0(prefix, '_condition_', r))
  if (parallel)
      return(parallel::mclapply(1:ncol(zscore), function(r) single_core(r),
                                mc.cores = min(8, ncol(zscore))))
  else
      return(lapply(1:ncol(zscore), function(r) single_core(r)))
}

eval(parse(text=commandArgs(T)))
dat = readRDS(input)
sumstats = dat$sumstats
N = nrow(dat$data$X)
ld = tempfile(fileext = ".ld")
write.table(cor(dat$data$X),ld,quote=F,col.names=F,row.names=F)
posterior = finemap_mvar(sumstats[1,,] / sumstats[2,,],
                                        ld, N, k=NULL,
                                        args, prefix=tempfile(fileext = ".finemap"))
saveRDS(posterior, paste0(output, '.rds'))

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(collapse = TRUE,comment = "#",fig.width = 5,
                      fig.height = 3,fig.align = "center",
                      dpi = 120)

## ----load-pkgs----------------------------------------------------------------
library(susieR)

## ----simdata------------------------------------------------------------------
set.seed(1)
n <- 200
p <- 1000
beta <- rep(0,p)
beta[1:4] <- 1
X <- matrix(rnorm(n*p),nrow = n,ncol = p)
X <- scale(X,center = TRUE,scale = FALSE)
y <- drop(X %*% beta + rnorm(n))

## ----sumstats-no-standardize--------------------------------------------------
ss  <- univariate_regression(X,y)
dat <- compute_suff_stat(X,y,standardize = FALSE)
R   <- cov2cor(dat$XtX)

## ----first-comparison, fig.height=3.5, fig.width=3----------------------------
res1 <- susie(X,y,L = 10)
res2 <- susie_rss(bhat = ss$betahat,shat = ss$sebetahat,R = R,n = n,
                  var_y = var(y),L = 10,estimate_residual_variance = TRUE)
plot(coef(res1),coef(res2),pch = 20,xlab = "susie",ylab = "susie_rss")
abline(a = 0,b = 1,col = "skyblue",lty = "dashed")

## ----sumstats-standardize-1---------------------------------------------------
ss  <- univariate_regression(scale(X),scale(y))
dat <- compute_suff_stat(X,y,standardize = TRUE)
R   <- cov2cor(dat$XtX)

## ----sumstats-standardize-2---------------------------------------------------
zhat <- ss$betahat/ss$sebetahat

## ----second-comparison, fig.height=3.5, fig.width=6, message=FALSE------------
res1 <- susie(scale(X),scale(y),L = 10)
res2 <- susie_rss(bhat = ss$betahat,shat = ss$sebetahat,R = R,n = n,
                  L = 10,estimate_residual_variance = TRUE)
res3 <- susie_rss(zhat,R,n = n,L = 10,estimate_residual_variance = TRUE)
layout(matrix(1:2,1,2))
plot(coef(res1),coef(res2),pch = 20,xlab = "susie",
          ylab = "susie_rss(bhat,shat)")
abline(a = 0,b = 1,col = "skyblue",lty = "dashed")
plot(coef(res1),coef(res3),pch = 20,xlab = "susie",ylab = "susie_rss(z)")
abline(a = 0,b = 1,col = "skyblue",lty = "dashed")

## ----third-comparison, fig.height=3.5, fig.width=3, message=FALSE-------------
res4 <- susie_rss(zhat,R,n = n,L = 10)
plot(coef(res1),coef(res4),pch = 20,xlab = "susie",ylab = "susie_rss")
abline(a = 0,b = 1,col = "skyblue",lty = "dashed")

## ----fourth-comparison, fig.height=3.5, fig.width=3, message=FALSE, warning=FALSE----
res5 <- susie_rss(zhat,R,L = 10)
plot(coef(res1),coef(res5)/sqrt(n),pch = 20,xlab = "susie",
ylab = "susie_rss/sqrt(n)")
abline(a = 0,b = 1,col = "skyblue",lty = "dashed")

#!/usr/bin/env Rscript
library(dplyr)
library(readr)
library(magrittr)

#' FINEMAP I/O
write_finemap_sumstats <- function(beta, se, LD_file, n, k, prefix) {
  cfg = list(z=paste0(prefix,".z"),
             ld=LD_file,
             snp=paste0(prefix,".snp"),
             config=paste0(prefix,".config"),
             cred=paste0(prefix, ".cred"),
             k=paste0(prefix,".k"),
             log=paste0(prefix,".log"),
             meta=paste0(prefix,".master"))
  se = replace(se, se == 0, 'nan')
  z = data.frame(chromosome="chr", position=seq(1, length(beta)), allele1='nan', allele2='nan', maf='nan', beta, se)
  z = cbind(rsid=z$position, z)
  write.table(z,cfg$z,quote=F,col.names=T,row.names=F)
  if (!is.null(k)) {
      write.table(t(k),cfg$k,quote=F,col.names=F,row.names=F)
      write("z;ld;snp;config;cred;n_samples;k;log",file=cfg$meta)
      write(paste(cfg$z, cfg$ld, cfg$snp, cfg$config, cfg$cred, n, cfg$k, cfg$log, sep=";"),
        file=cfg$meta,append=TRUE)
  } else {
      write("z;ld;snp;config;cred;n_samples;log",file=cfg$meta)
      write(paste(cfg$z, cfg$ld, cfg$snp, cfg$config, cfg$cred, n, cfg$log, sep=";"),
            file=cfg$meta,append=TRUE)
  }
  return(cfg)
}

#' Run FINEMAP version 1.4
#' http://www.christianbenner.com
## FIXME: read the finemapr implementation for data sanity check.
## Can be useful as a general data sanity checker (in previous modules)

run_finemap <- function(beta, se, LD_file, n, k, args = "", prefix="data")
{
  cfg = write_finemap_sumstats(beta, se, LD_file, n, k, prefix)
  cmd = paste("finemap --sss --log", "--in-files", cfg$meta, args)
  dscrutils::run_cmd(cmd)
  cfg$log = paste0(cfg$log, "_sss")

  # read output tables
  snp = read.table(cfg$snp,header=TRUE,sep=" ")
  snp$snp = as.character(snp$rsid)

  snp = rank_snp(snp)
  # we add snp-prob for backwards-compatability with code that used this script with FINEMAP v1.1
  snp$prob = snp$snp_prob
  config = read.table(cfg$config,header=TRUE,sep=" ")

  # Only keep configurations with cumulative 95% probability
  # config = within(config, config_prob_cumsum <- cumsum(config_prob))
  # config = config[config$config_prob_cumsum <= 0.95,]

  # extract number of causal
  ncausal = finemap_extract_ncausal(cfg$log)
  return(list(snp=snp, set=config, ncausal=ncausal))
}

rank_snp <- function(snp) {
  snp <- arrange(snp, -prob) %>%
    mutate(
        rank = seq(1, n()),
        prob_cumsum = cumsum(prob) / sum(prob)) %>%
    select(rank, snp, prob, prob_cumsum, log10bf)
  return(snp)
}

finemap_extract_ncausal <- function(logfile)
{
  lines <- grep("->", readLines(logfile), value = TRUE)
  lines <- gsub("\\(|\\)|>", "", lines)
  splits <- strsplit(lines, "\\s+")
  tab <- data.frame(
    ncausal_num = sapply(splits, function(x) as.integer(x[2])),
    ncausal_prob = sapply(splits, function(x) as.double(x[4])))
  tab <- mutate(tab, type = ifelse(duplicated(ncausal_num), "post", "prior"))
  return(tab)
}

finemap_mvar <- function(beta, se, LD_file, n, k, args, prefix, parallel = FALSE) {
  if (is.null(dim(beta))) {
      beta = matrix(ncol=1,beta)
  }
  if (is.null(dim(se))) {
      se = matrix(ncol=1,se)
  }

  single_core = function(r) 
      run_finemap(beta[,r], se[,r], LD_file, n, k, args, 
                  prefix=paste0(prefix, '_condition_', r))
  if (parallel)
      return(parallel::mclapply(1:ncol(beta), function(r) single_core(r),
                                mc.cores = min(8, ncol(beta))))
  else
      return(lapply(1:ncol(beta), function(r) single_core(r)))
}

eval(parse(text=commandArgs(T)))
dat = readRDS(input)
sumstats = dat$sumstats
N = nrow(dat$data$X)
ld = tempfile(fileext = ".ld")
ld_mat = cor(dat$data$X)
ld_mat[is.na(ld_mat)] = 'nan'
write.table(ld_mat,ld,quote=F,col.names=F,row.names=F)
posterior = finemap_mvar(sumstats[1,,], sumstats[2,,],
                                        ld, N, k=NULL,
                                        args, prefix=tempfile(fileext = ".finemap"))
saveRDS(posterior, paste0(output, '.rds'))

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(collapse = TRUE,comment = "#",fig.width = 5,
                      fig.height = 3,fig.align = "center",
                      fig.cap = "&nbsp;",dpi = 120)

## -----------------------------------------------------------------------------
library(susieR)
library(curl)
data_file <- tempfile(fileext = ".RData")
data_url <- paste0("https://raw.githubusercontent.com/stephenslab/susieR/",
                   "master/inst/datafiles/FinemappingConvergence1k.RData")
curl_download(data_url,data_file)
load(data_file)
b <- FinemappingConvergence$true_coef
susie_plot(FinemappingConvergence$z, y = "z", b=b)

## -----------------------------------------------------------------------------
fitted <- with(FinemappingConvergence,
               susie_suff_stat(XtX = XtX, Xty = Xty, yty = yty, n = n))
susie_plot(fitted, y="PIP", b=b, main=paste0("ELBO = ", round(susie_get_objective(fitted),2)))

## -----------------------------------------------------------------------------
fitted_refine <- with(FinemappingConvergence,
                      susie_suff_stat(XtX = XtX, Xty = Xty, yty = yty,
					                  n = n, refine=TRUE))
susie_plot(fitted_refine, y="PIP", b=b, main=paste0("ELBO = ", round(susie_get_objective(fitted_refine),2)))

## -----------------------------------------------------------------------------
sessionInfo()

# export MEM_CHECK_INTERVAL=0.01
# python3 monitor_memory.py Rscript compute_ss_memory.R
#
# NOTES:
#
# - Without any improvements:
#   Size of X: 0.3 GB
#   max rss_memory: 1.65 GB
#
# - The original centering and scaling steps require about 1 GB.
#
# - With the improvements:
#   Size of X: 0.3 GB
#   max rss_memory: 0.66 GB
#
# set.seed(1)
# p <- 2000
# n <- 20000
# X <- matrix(rnorm(n*p),n,p)
# y <- rnorm(n)
# save(list = c("X","y"),file = "compute_ss_data.RData")
# library(susieR)
devtools::load_all()
set.seed(1)
load("compute_ss_data.RData")
cat("Size of X:\n")
print(object.size(X),unit = "GB")
cat("Running compute_ss.\n")
out <- compute_ss(X,y,standardize = TRUE)

# export MEM_CHECK_INTERVAL=0.01
# python3 monitor_memory.py Rscript susie_rss_memory.R
#
# NOTES:
#
# - Without any improvements:
#   Size of X: 0.5 GB
#   max rss_memory: 4.15 GB
#
# - With some improvements, the initial steps right before the main
#   loop use 0.86 GB.
#
# - susie_rss right before the CS and PIP calculations uses 1.6 GB.
#
# library(susieR)
devtools::load_all()
# set.seed(1)
# p  <- 8000
# n  <- 1000
# X  <- matrix(rnorm(n*p),n,p)
# y  <- rnorm(n)
# ss <- susieR:::univariate_regression(X,y)
# z  <- ss$betahat/ss$sebetahat
# R  <- cor(X)
# save(list = c("X","y","z","R"),file = "susie_rss_data.RData")
load("susie_rss_data.RData")
cat("Size of R:\n")
print(object.size(R),unit = "GB")
cat("Running susie_rss.\n")
set.seed(1)
out <- susie_rss(z,R,estimate_prior_variance = FALSE,min_abs_corr = 0,
                 check_input = FALSE,refine = FALSE,verbose = TRUE)
print(sapply(out$sets$cs,length))

# export MEM_CHECK_INTERVAL=0.01
# python3 monitor_memory.py Rscript susie_memory.R
#
# NOTES:
#
# - Without any improvements:
#   Size of X: 1 GB
#   max rss_memory: 4.70 GB
#
# - With the improvements:
#   Size of X: 1 GB
#   max rss_memory: 3.00 GB
#
# set.seed(1)
# p <- 16000
# n <- 8000
# X <- matrix(rnorm(n*p),n,p)
# X <- scale(X,center = TRUE,scale = TRUE)
# y <- rnorm(n)
# save(list = c("X","y"),file = "susie_data.RData")
# library(susieR)
devtools::load_all()
load("susie_data.RData")
cat("Size of X:\n")
print(object.size(X),unit = "GB")
cat("Running susie.\n")
set.seed(1)
out <- susie(X,y,estimate_prior_variance = FALSE,min_abs_corr = 0,
             verbose = TRUE)
print(sapply(out$sets$cs,length))

## results from original susie
devtools::install_github("stephenslab/susieR")
library(susieR)

create_sparsity_mat = function(sparsity, n, p){
  nonzero = round(n*p*(1-sparsity))
  nonzero.idx = sample(n*p, nonzero)
  mat = numeric(n*p)
  mat[nonzero.idx] = 1
  mat = matrix(mat, nrow=n, ncol=p)
  return(mat)
}

set.seed(1)
n = 100
p = 200
beta = rep(0,p)
beta[1]    = 10
beta[2]  = 10
beta[3]  = 10
beta[4] = 10
X.dense = create_sparsity_mat(0.99,n,p)
y = c(X.dense %*% beta + rnorm(n))
L = 10
residual_variance = 0.8
scaled_prior_variance = 0.2
s = list(alpha=matrix(1/p,nrow=L,ncol=p),
         mu=matrix(2,nrow=L,ncol=p),
         mu2=matrix(3,nrow=L,ncol=p),
         Xr=rep(5,n), KL=rep(1.2,L),
         sigma2=residual_variance, V=scaled_prior_variance * as.numeric(var(y)))
X = susieR:::set_X_attributes(X.dense)
Eb = rep(1, p)
Eb2 = rep(1, p)
s2 = residual_variance
V = scaled_prior_variance


objective.original.res = susieR::susie_get_objective(s)
saveRDS(objective.original.res, 'objective_original_res.rds')

Eloglik.original.res = susieR:::Eloglik(X,y,s)
saveRDS(Eloglik.original.res, 'Eloglik_original_res.rds')

ER2.original.res = susieR:::get_ER2(X,y,s)
saveRDS(ER2.original.res, 'ER2_original_res.rds')

SER.original.res = susieR:::SER_posterior_e_loglik(X,y,s2,Eb,Eb2)
saveRDS(SER.original.res, 'SER_original_res.rds')

singleReg.original.res = susieR:::single_effect_regression(y,X,V)
saveRDS(singleReg.original.res, 'singleReg_original_res.rds')

vbupdate.original.res = susieR:::update_each_effect(X, y, s)
saveRDS(vbupdate.original.res, 'vbupdate_original_res.rds')

susiefit.original.res = susie(X.dense,y)
saveRDS(susiefit.original.res, 'susiefit_original_res.rds')

susiefit.original.res2 = susie(X.dense, y, standardize = TRUE, intercept = FALSE)
susiefit.original.res3 = susie(X.dense, y, standardize = FALSE, intercept = TRUE)
susiefit.original.res4 = susie(X.dense, y, standardize = FALSE, intercept = FALSE)
saveRDS(susiefit.original.res2, 'susiefit_original_res2.rds')
saveRDS(susiefit.original.res3, 'susiefit_original_res3.rds')
saveRDS(susiefit.original.res4, 'susiefit_original_res4.rds')

# @title Compute unscaled X %*% b using the special structure of trend
#   filtering
# @param order is the order of trend filtering
# @param b an n=p vector
# @return an n vector
compute_tf_Xb = function(order,b) {
  for (i in 1:(order+1))
    b = rev(-1*cumsum(rev(b)))
  return(b)
}

# @title Compute unscaled t(X) %*% y using the special structure of
#   trend filtering
# @param order is the order of trend filtering
# @param y an n vector
# @return an n vector
compute_tf_Xty = function(order, y) {
  for (i in 1:(order+1))
    y = -1*cumsum(y)
  return(y)
}

# @title Compute colSums(X*X) for X under four scenarios
# @param order is the order of trend filtering
# @param n the length of y
# @param cm column means of X
# @param csd column standard deviations of X
# @param intercept a boolean denotes whether mean centering X
# @param standardize a boolean denotes whether scaling X by standard deviation
# @return an n vector
compute_tf_d = function (order, n, cm, csd, standardize = FALSE,
                         intercept = FALSE) {
  if (intercept) {
      
    # When standardize = TRUE, intercept = TRUE: by special
    # observation d = [n-1, n-1, ...]
    d = rep(n-1,n)
    if (order == 0)
      d[n] = 0

    # When standardize = FALSE, intercept = TRUE:
    # d = [n-1, n-1, ...] * (csd^2)
    if (!standardize)
      d = d*csd^2
    return(d)
  } else {
      
    # When standardize = FALSE, intercept = FALSE: d = colSums(X^2)
    base = rep(-1,n)
    if (order == 0)
      d = cumsum(base^2)
    else {
      for (i in 1:order)
        base = cumsum(base)
      d = cumsum(base^2) 
    }

    # When standardize = TRUE, intercept = TRUE:
    # d = colSums(X^2) / (csd^2)
    if (standardize)
      d = d/csd^2
    return(d)
  }
}

# @title Compute column mean of the trend filtering matrix X.
# @param order is the order of trend filtering
# @param n the length of y
# @return an n vector
compute_tf_cm = function (order, n) {
  base = rep(1,n)
  for (i in 1:(order+1))
    base = -cumsum(base)
  return(base/n)
}

# @title Compute column standard deviation of the trend filtering
#   matrix X
# @param order is the order of trend filtering
# @param n is the length of y
# @return an n vector
compute_tf_csd = function (order, n) {
  cm = compute_tf_cm(order,n)
  csd = sqrt((compute_tf_d(order,n)/n - cm^2)*n/(n-1))
  csd[which(csd == 0)] = 1
  return(csd)
}

# @title A fast way to compute colSums(X*X), where X is a
#   mean-centered and standardized trend filtering matrix.
# @param order order of trend filtering
# @param n the length of y
# @return an n vector
compute_tf_std_d = function (order, n) {
  res = rep(n-1,n)
  if (order == 0)
    res[n] = 0
  return(res)
}

# @title sets three attributes for matrix X
# @param X an n by p data matrix that can be either a trend filtering
#   matrix or a regular dense/sparse matrix
# @param center boolean indicating centered by column means or not
# @param scale boolean indicating scaled by column standard deviations or not
# @return X with three attributes e.g. `attr(X, 'scaled:center') is a
# p vector of column means of X if center=TRUE, a p vector of zeros
# otherwise. 'attr(X, 'scaled:scale') is a p vector of columan standard
# deviations of X if scale=TRUE, a p vector of 1s otherwise. 'attr(X,
# 'd') is a p vector of column sums of X.standardized^2,' where
# X.standardized is the matrix X centered by attr(X, 'scaled:center')
# and scaled by attr(X, 'scaled:scale').
#
#' @importFrom Matrix rowSums
#' @importFrom Matrix colMeans
set_X_attributes = function (X, center = TRUE, scale = TRUE) {
    
  # if X is a trend filtering matrix
  if (!is.null(attr(X,"matrix.type"))) {
    order = attr(X,"order")
    n = ncol(X)
    
    # Set three attributes for X.
    attr(X,"scaled:center") = compute_tf_cm(order,n)
    attr(X,"scaled:scale") = compute_tf_csd(order,n)
    attr(X,"d") = compute_tf_d(order,n,attr(X,"scaled:center"),
                               attr(X,"scaled:scale"),scale,center)
    if (!center)
      attr(X,"scaled:center") = rep(0,n)
    if (!scale)
      attr(X,"scaled:scale") = rep(1,n)
  } else {
      
    # If X is either a dense or sparse ordinary matrix.
    # Get column means.
    cm = colMeans(X,na.rm = TRUE)
    
    # Get column standard deviations.
    csd = compute_colSds(X)
    
    # Set sd = 1 when the column has variance 0.
    csd[csd == 0] = 1
    if (!center)
      cm = rep(0,length = length(cm))
    if (!scale) 
      csd = rep(1,length = length(cm))

    # Ah, this code is very inefficient because the matrix becomes
    # dense!
    # 
    #   X.std = as.matrix(X)
    #   X.std = (t(X.std) - cm)/csd
    #   attr(X,"d") = rowSums(X.std * X.std)
    #
    # Set three attributes for X.
    n = nrow(X)
    d = n*colMeans(X)^2 + (n-1)*compute_colSds(X)^2
    d = (d - n*cm^2)/csd^2
    attr(X,"d") = d
    attr(X,"scaled:center") = cm
    attr(X,"scaled:scale") = csd
  }
  return(X)
}

create_sparsity_mat = function(sparsity, n, p){
  nonzero = round(n*p*(1-sparsity))
  nonzero.idx = sample(n*p, nonzero)
  mat = numeric(n*p)
  mat[nonzero.idx] = 1
  mat = matrix(mat, nrow=n, ncol=p)
  return(mat)
}

simulate = function(n=100, p=200, sparse=F) {
  suppressWarnings(RNGversion("3.5.0"))
  set.seed(1)
  beta = rep(0,p)
  beta[1:4] = 10
  if (sparse) {
    X = create_sparsity_mat(0.99,n,p)
    X.sparse = as(X,"CsparseMatrix")
  } else {
    X = matrix(rnorm(n*p,3,4),n,p)
    X.sparse = NA
  }
  y = c(X %*% beta + rnorm(n))
  L = 10
  residual_variance = 0.8
  scaled_prior_variance = 0.2
  s = list(alpha=matrix(1/p,nrow=L,ncol=p),
           mu=matrix(2,nrow=L,ncol=p),
           mu2=matrix(3,nrow=L,ncol=p),
           Xr=rep(5,n), KL=rep(1.2,L),
           sigma2=residual_variance,
      V=scaled_prior_variance * as.numeric(var(y)),
      lbf_variable = matrix(0,L,p))
  return(list(X=X, X.sparse=X.sparse, s=s, y=y, n=n, p=p, b=beta))
}

simulate_tf = function(order){
  suppressWarnings(RNGversion("3.5.0"))
  set.seed(2)
  n = 50
  D = diag(-1, n)
  for (i in 1:(n-1)){
    D[i, i+1] = 1
  }
  if (order==0) {
    beta = c(rep(0,5),rep(1,5),rep(3,5),rep(-2,5),rep(0,30))
    y = beta + rnorm(n)
    X = solve(D)
  } else if (order==1) {
    beta = numeric(n)
    for (i in 1:n){
      if (i <= 5){
        beta[i] = 0.001*i + 2
      } else if (i <= 15){
        beta[i] = 5*0.001*i + 1.6
      } else{
        beta[i] = 6.1 - 10*0.001*i
      }
    }
    y = beta + rnorm(n)
    X = solve(D%*%D)
  } else if (order==2) {
    beta = numeric(n)
    for (i in 1:n){
      if (i <= 5){
        beta[i] = (0.001*i)^2
      } else if (i <= 35){
        beta[i] = -5*(0.001*i)^2 + 0.06
      } else{
        beta[i] = 3*(0.001*i)^2 - 3.86
      }
    }
    y = beta + rnorm(n)
    X = solve(D%*%D%*%D)
  }
  return(list(X=X, y=y))
}

expect_equal_susie_update = function(new.res, original.res, tolerance = .Machine$double.eps^0.5){
  expect_equal(new.res$alpha, original.res$alpha, scale = 1, tolerance = tolerance)
  expect_equal(new.res$mu, original.res$mu, scale = 1, tolerance = tolerance)
  expect_equal(new.res$mu2, original.res$mu2, scale = 1, tolerance = tolerance)
  expect_equal(new.res$Xr, original.res$Xr, scale = 1, tolerance = tolerance)
  expect_equal(new.res$KL, original.res$KL, scale = 1, tolerance = tolerance)
  expect_equal(new.res$sigma2, original.res$sigma2, scale = 1, tolerance = tolerance)
  expect_equal(new.res$V, original.res$V, scale = 1, tolerance = tolerance)
}

expect_equal_susie_suff_stat_update = function(new.res, original.res, tolerance = .Machine$double.eps^0.5){
  expect_equal(new.res$alpha, original.res$alpha, scale = 1, tolerance = tolerance)
  expect_equal(new.res$mu, original.res$mu, scale = 1, tolerance = tolerance)
  expect_equal(new.res$mu2, original.res$mu2, scale = 1, tolerance = tolerance)
  expect_equal(new.res$XtXr, original.res$XtXr, scale = 1, tolerance = tolerance)
  expect_equal(new.res$KL, original.res$KL, scale = 1, tolerance = tolerance)
  expect_equal(new.res$sigma2, original.res$sigma2, scale = 1, tolerance = tolerance)
  expect_equal(new.res$V, original.res$V, scale = 1, tolerance = tolerance)
}

expect_equal_susie_rss_update = function(new.res, original.res, tolerance = .Machine$double.eps^0.5){
  expect_equal(new.res$alpha, original.res$alpha, scale = 1, tolerance = tolerance)
  expect_equal(new.res$mu, original.res$mu, scale = 1, tolerance = tolerance)
  expect_equal(new.res$mu2, original.res$mu2, scale = 1, tolerance = tolerance)
  expect_equal(new.res$Rz, original.res$Rz, scale = 1, tolerance = tolerance)
  expect_equal(new.res$KL, original.res$KL, scale = 1, tolerance = tolerance)
  expect_equal(new.res$sigma2, original.res$sigma2, scale = 1, tolerance = tolerance)
  expect_equal(new.res$V, original.res$V, scale = 1, tolerance = tolerance)
}

expect_equal_SER = function(new.res, original.res){
  expect_equal(new.res$alpha, original.res$alpha)
  expect_equal(new.res$mu, original.res$mu)
  expect_equal(new.res$mu2, original.res$mu2)
  expect_equal(new.res$lbf, original.res$lbf)
  expect_equal(new.res$V, original.res$V)
  expect_equal(new.res$loglik, original.res$loglik)
}

expect_equal_SER_suff_stat = function(new.res, original.res, tolerance = .Machine$double.eps^0.5){
  expect_equal(new.res$alpha, original.res$alpha, scale = 1, tolerance = tolerance)
  expect_equal(new.res$mu, original.res$mu, scale = 1, tolerance = tolerance)
  expect_equal(new.res$mu2, original.res$mu2, scale = 1, tolerance = tolerance)
  expect_equal(new.res$lbf, original.res$lbf, scale = 1, tolerance = tolerance)
  expect_equal(new.res$V, original.res$V, scale = 1, tolerance = tolerance)
  expect_equal(new.res$lbf_model, original.res$lbf_model, scale = 1, tolerance = tolerance)
}

expect_equal_susie = function(new.res, original.res, tolerance = .Machine$double.eps^0.5){
  expect_equal_susie_update(new.res, original.res, tolerance = tolerance)
  expect_equal(new.res$elbo, original.res$elbo, scale = 1, tolerance = tolerance)
  expect_equal(new.res$niter, original.res$niter, scale = 1, tolerance = tolerance)
  expect_equal(new.res$intercept, original.res$intercept, scale = 1, tolerance = tolerance)
  expect_equal(new.res$fitted, original.res$fitted, scale = 1, tolerance = tolerance)
  expect_equal(new.res$X_column_scale_factors, original.res$X_column_scale_factors, scale = 1, tolerance = tolerance)
}

expect_equal_susie_suff_stat = function(new.res, original.res, tolerance = .Machine$double.eps^0.5){
  expect_equal_susie_suff_stat_update(new.res, original.res, tolerance = tolerance)
  expect_equal(new.res$elbo, original.res$elbo, scale = 1, tolerance = tolerance)
  expect_equal(new.res$niter, original.res$niter, scale = 1, tolerance = tolerance)
  expect_equal(new.res$intercept, original.res$intercept, scale = 1, tolerance = tolerance)
  expect_equal(new.res$Xtfitted, original.res$Xtfitted, scale = 1, tolerance = tolerance)
}

expect_equal_susie_rss = function(new.res, original.res, tolerance = .Machine$double.eps^0.5){
  expect_equal_susie_rss_update(new.res, original.res, scale = 1, tolerance = tolerance)
  expect_equal(new.res$elbo, original.res$elbo, scale = 1, tolerance = tolerance)
  expect_equal(new.res$niter, original.res$niter, scale = 1, tolerance = tolerance)
  expect_equal(new.res$intercept, original.res$intercept, scale = 1, tolerance = tolerance)
  expect_equal(new.res$Rz, original.res$Rz, scale = 1, tolerance = tolerance)
}

context("test_compute_tf.R")

expect_equal_compute_tf = function(order){
  with(simulate_tf(order), {
    X = set_X_attributes(X)

    suppressWarnings(RNGversion("3.5.0"))
    set.seed(1)
    b = rnorm(length(y))
    Xb = compute_tf_Xb(order, b=b)
    expect_equal(as.matrix(Xb), X%*%b)

    Xty = compute_tf_Xty(order,y=y)
    expect_equal(as.matrix(Xty), t(X)%*%y)

    d = compute_tf_d(order, n=length(y))
    expect_equal(d, colSums(X*X))

    cm = compute_tf_cm(order, n=length(y))
    expect_equal(cm, attr(X, 'scaled:center'))

    csd = compute_tf_csd(order, n=length(y))
    expect_equal(csd, attr(X, 'scaled:scale'))

    std_d = compute_tf_std_d(order, n=length(y))
    expect_equal(std_d, attr(X, 'd'))
  })
}

test_that("computation about trend filtering",{
  expect_equal_compute_tf(order=0)
  expect_equal_compute_tf(order=1)
  expect_equal_compute_tf(order=2)
})

#' @rdname susie
#'
#' @param XtX A p by p matrix \eqn{X'X} in which the columns of X
#'   are centered to have mean zero.
#'
#' @param Xty A p-vector \eqn{X'y} in which y and the columns of X are
#'   centered to have mean zero.
#'
#' @param yty A scalar \eqn{y'y} in which y is centered to have mean
#'   zero.
#'
#' @param n The sample size.
#'
#' @param X_colmeans A p-vector of column means of \code{X}. If both
#'   \code{X_colmeans} and \code{y_mean} are provided, the intercept
#'   is estimated; otherwise, the intercept is NA.
#'
#' @param y_mean A scalar containing the mean of \code{y}. If both
#'   \code{X_colmeans} and \code{y_mean} are provided, the intercept
#'   is estimated; otherwise, the intercept is NA.
#'
#' @param maf Minor allele frequency; to be used along with
#'   \code{maf_thresh} to filter input summary statistics.
#'
#' @param maf_thresh Variants having a minor allele frequency smaller
#'   than this threshold are not used.
#'
#' @param r_tol Tolerance level for eigenvalue check of positive
#'   semidefinite matrix of R.
#'
#' @param check_input If \code{check_input = TRUE},
#'   \code{susie_suff_stat} performs additional checks on \code{XtX} and
#'   \code{Xty}. The checks are: (1) check that \code{XtX} is positive
#'   semidefinite; (2) check that \code{Xty} is in the space spanned by
#'   the non-zero eigenvectors of \code{XtX}.
#'
#' @param check_prior If \code{check_prior = TRUE}, it checks if the
#'   estimated prior variance becomes unreasonably large (comparing with
#'   10 * max(abs(z))^2).
#'
#' @param n_purity Passed as argument \code{n_purity} to
#'   \code{\link{susie_get_cs}}.
#'
#' @param ... Additional arguments to provide backward compatibility
#'   with earlier versions of \code{susie_suff_stat}.
#'
#' @export
#'
susie_suff_stat = function (XtX, Xty, yty, n,
                            X_colmeans = NA, y_mean = NA,
                            maf = NULL, maf_thresh = 0, L = 10,
                            scaled_prior_variance = 0.2,
                            residual_variance = NULL,
                            estimate_residual_variance = TRUE,
                            estimate_prior_variance = TRUE,
                            estimate_prior_method = c("optim","EM","simple"),
                            check_null_threshold = 0, prior_tol = 1e-9,
                            r_tol = 1e-08, prior_weights = NULL,
                            null_weight = 0, standardize = TRUE,
                            max_iter = 100, s_init = NULL, coverage = 0.95,
                            min_abs_corr = 0.5, tol = 1e-3,
                            verbose = FALSE, track_fit = FALSE,
                            check_input = FALSE, refine = FALSE,
                            check_prior = FALSE, n_purity = 100, ...) {

  # Check for use of arguments that are now deprecated.
  args <- list(...)
  if (any(is.element(names(args),c("bhat","shat","R","var_y"))))
    stop("susie_suff_stat no longer accepts inputs bhat, shat, R or var_y; ",
         "these inputs are now accepted by susie_rss instead")
    
  # Process input estimate_prior_method.
  estimate_prior_method = match.arg(estimate_prior_method)

  if (missing(n))
    stop("n must be provided")
  if (n <= 1)
    stop("n must be greater than 1")

  # Check sufficient statistics.
  missing_XtX = c(missing(XtX), missing(Xty), missing(yty))

  if (all(missing_XtX))
    stop("Please provide all of XtX, Xty, yty, n")

  if (ncol(XtX) > 1000 & !requireNamespace("Rfast",quietly = TRUE))
    warning_message("For large R or large XtX, consider installing the ",
                    "Rfast package for better performance.", style="hint")

  # Check input XtX.
  if (ncol(XtX) != length(Xty))
    stop(paste0("The dimension of XtX (",nrow(XtX)," by ",ncol(XtX),
                ") does not agree with expected (",length(Xty)," by ",
                length(Xty),")"))
  if (!is_symmetric_matrix(XtX)) {
    warning_message("XtX is not symmetric; forcing XtX to be symmetric by ",
            "replacing XtX with (XtX + t(XtX))/2")
    XtX = XtX + t(XtX)
    XtX = XtX/2
  }
  
  # MAF filter.
  if (!is.null(maf)) {
    if (length(maf) != length(Xty))
      stop(paste("The length of maf does not agree with expected",length(Xty)))
    id = which(maf > maf_thresh)
    XtX = XtX[id,id]
    Xty = Xty[id]
  }

  if (any(is.infinite(Xty)))
    stop("Input Xty contains infinite values")
  if (!(is.double(XtX) & is.matrix(XtX)) & !inherits(XtX,"CsparseMatrix"))
    stop("Input XtX must be a double-precision matrix, or a sparse matrix")
  if (anyNA(XtX))
    stop("Input XtX matrix contains NAs")

  # Replace NAs in Xty with zeros.
  if (anyNA(Xty)) {
    warning_message("NA values in Xty are replaced with 0")
    Xty[is.na(Xty)] = 0
  }

  if (check_input) {

    # Check whether XtX is positive semidefinite.
    semi_pd = check_semi_pd(XtX,r_tol)
    if (!semi_pd$status)
      stop("XtX is not a positive semidefinite matrix")

    # Check whether Xty in space spanned by the non-zero eigenvectors of XtX
    proj = check_projection(semi_pd$matrix,Xty)
    if (!proj$status)
      warning_message("Xty does not lie in the space of the non-zero eigenvectors ",
              "of XtX")
  }

  if (is.numeric(null_weight) && null_weight == 0)
    null_weight = NULL
  if (!is.null(null_weight)) {
    if (!is.numeric(null_weight))
      stop("Null weight must be numeric")
    if (null_weight < 0 || null_weight >= 1)
      stop("Null weight must be between 0 and 1")
    if (is.null(prior_weights))
      prior_weights = c(rep(1/ncol(XtX)*(1-null_weight),ncol(XtX)),null_weight)
    else
      prior_weights = c(prior_weights*(1 - null_weight),null_weight)
    XtX = cbind(rbind(XtX,0),0)
    Xty = c(Xty,0)
  }

  p = ncol(XtX)

  if (standardize) {
    dXtX = diag(XtX)
    csd = sqrt(dXtX/(n-1))
    csd[csd == 0] = 1
    XtX = t((1/csd) * XtX) / csd
    Xty = Xty / csd
  } else
    csd = rep(1,length = p)
  attr(XtX,"d") = diag(XtX)
  attr(XtX,"scaled:scale") = csd

  # Check that X_colmeans has length 1 or p.
  if (length(X_colmeans) == 1)
    X_colmeans = rep(X_colmeans,p)
  if (length(X_colmeans) != p)
    stop("The length of X_colmeans does not agree with number of variables")

  # Initialize susie fit.
  s = init_setup(0,p,L,scaled_prior_variance,residual_variance,prior_weights,
                 null_weight,yty/(n-1),standardize)
  s$Xr = NULL
  s$XtXr = rep(0,p)

  if (!missing(s_init)&& !is.null(s_init)) {
    if (!inherits(s_init,"susie"))
      stop("s_init should be a susie object")
    if (max(s_init$alpha) > 1 || min(s_init$alpha) < 0)
      stop("s_init$alpha has invalid values outside range [0,1]; please ",
           "check your input")

    # First, remove effects with s_init$V = 0
    s_init = susie_prune_single_effects(s_init)
    num_effects = nrow(s_init$alpha)
    if(missing(L)){ # if L is not specified, we set it as in s_init.
      L = num_effects
    }else if(min(p, L) < num_effects){
      warning_message(paste("Specified number of effects L =",min(p, L),
                    "is smaller than the number of effects",num_effects,
                    "in input SuSiE model. The initialized SuSiE model will have",
                    num_effects,"effects."))
      L = num_effects
    }
    # expand s_init if L > num_effects.
    s_init = susie_prune_single_effects(s_init, min(p, L), s$V)
    s = modifyList(s,s_init)
    s = init_finalize(s,X = XtX)
    s$XtXr = s$Xr
    s$Xr = NULL
  } else
    s = init_finalize(s)

  # Initialize elbo to NA.
  elbo = rep(as.numeric(NA),max_iter + 1)
  elbo[1] = -Inf;
  tracking = list()

  bhat = (1/attr(XtX,"d")) * Xty
  shat = sqrt(s$sigma2/attr(XtX,"d"))
  z = bhat/shat
  zm = max(abs(z[!is.nan(z)]))

  for (i in 1:max_iter) {
    if (track_fit)
      tracking[[i]] = susie_slim(s)
    s = update_each_effect_ss(XtX,Xty,s,estimate_prior_variance,
                              estimate_prior_method,check_null_threshold)
    if(check_prior){
      if(any(s$V > 100*(zm^2))){
        stop('The estimated prior variance is unreasonably large.
	     This is usually caused by mismatch between the summary statistics and the LD matrix.
             Please check the input.')
      }
    }

    if (verbose)
      print(paste0("objective:",get_objective_ss(XtX,Xty,s,yty,n)))

    # Compute objective before updating residual variance because part
    # of the objective s$kl has already been computed under the
    # residual variance before the update.
    elbo[i+1] = get_objective_ss(XtX,Xty,s,yty,n)
    if(is.infinite(elbo[i+1])){
      stop('The objective becomes infinite. Please check the input.')
    }

    if ((elbo[i+1] - elbo[i]) < tol) {
      s$converged = TRUE
      break
    }
    if (estimate_residual_variance) {
      est_sigma2 = estimate_residual_variance_ss(XtX,Xty,s,yty,n)
      if (est_sigma2 < 0)
        stop("Estimating residual variance failed: the estimated value ",
             "is negative")
      s$sigma2 = est_sigma2
      if (verbose)
        print(paste0("objective:",get_objective_ss(XtX,Xty,s,yty,n)))
    }
  }
  elbo = elbo[2:(i+1)] # Remove first (infinite) entry, and trailing NAs.
  s$elbo = elbo
  s$niter = i

  if (is.null(s$converged)) {
    warning(paste("IBSS algorithm did not converge in",max_iter,"iterations!
                  Please check consistency between summary statistics and LD matrix.
                  See https://stephenslab.github.io/susieR/articles/susierss_diagnostic.html"))
    s$converged = FALSE
  }

  s$X_column_scale_factors = attr(XtX,"scaled:scale")

  # Compute intercept.
  s$intercept = y_mean -
    sum(X_colmeans * (colSums(s$alpha * s$mu)/s$X_column_scale_factors))


  if (track_fit)
    s$trace = tracking

  # SuSiE CS and PIP.
  if (!is.null(coverage) && !is.null(min_abs_corr)) {
    if(any(!(diag(XtX) %in% c(0,1)))){
      s$sets = susie_get_cs(s,coverage = coverage,Xcorr = muffled_cov2cor(XtX),
                            min_abs_corr = min_abs_corr,
                            check_symmetric = FALSE,
                            n_purity = n_purity)
    }else
      s$sets = susie_get_cs(s,coverage = coverage,Xcorr = XtX,
                            min_abs_corr = min_abs_corr,
                            check_symmetric = FALSE,
                            n_purity = n_purity)
    s$pip = susie_get_pip(s,prune_by_cs = FALSE,prior_tol = prior_tol)
  }

  if (!is.null(colnames(XtX))) {
    variable_names = colnames(XtX)
    if (!is.null(null_weight)) {
      variable_names[length(variable_names)] = "null"
      names(s$pip) = variable_names[-p]
    } else
      names(s$pip) = variable_names
    colnames(s$alpha) = variable_names
    colnames(s$mu)    = variable_names
    colnames(s$mu2)   = variable_names
    colnames(s$lbf_variable) = variable_names
  }

  if (refine) {
    if (!missing(s_init) && !is.null(s_init))
      warning("The given s_init is not used in refinement")
    if (!is.null(null_weight) && null_weight != 0) {
      ## if null_weight is specified
      ## we remove the extra 0 column
      XtX = XtX[1:(p-1), 1:(p-1)]
      Xty = Xty[1:(p-1)]
      pw_s = s$pi[-s$null_index]/(1 - null_weight)
    } else
      pw_s = s$pi
    conti = TRUE
    while (conti & length(s$sets$cs)>0) {
      m = list()
      for(cs in 1:length(s$sets$cs)) {
        pw_cs = pw_s
        pw_cs[s$sets$cs[[cs]]] = 0
        if(all(pw_cs == 0)){
          break
        }
        s2 = susie_suff_stat(XtX = XtX, Xty = Xty, yty = yty, n = n, L = L,
            X_colmeans = X_colmeans, y_mean = y_mean,
            prior_weights = pw_cs, s_init = NULL,
            scaled_prior_variance = scaled_prior_variance,
            residual_variance = residual_variance,
            estimate_residual_variance = estimate_residual_variance,
            estimate_prior_variance = estimate_prior_variance,
            estimate_prior_method = estimate_prior_method,
            check_null_threshold = check_null_threshold, prior_tol = prior_tol,
            r_tol = r_tol, max_iter = max_iter,
            null_weight = null_weight, standardize = standardize,
            coverage = coverage, min_abs_corr = min_abs_corr, tol = tol,
            verbose = FALSE, track_fit = FALSE, check_input = FALSE,
            refine = FALSE)
        sinit2 = s2[c("alpha","mu","mu2")]
        class(sinit2) = "susie"
        s3 = susie_suff_stat(XtX = XtX, Xty = Xty, yty = yty, n = n, L = L,
            X_colmeans = X_colmeans, y_mean = y_mean,
            prior_weights = pw_s, s_init = sinit2,
            scaled_prior_variance = scaled_prior_variance,
            residual_variance = residual_variance,
            estimate_residual_variance = estimate_residual_variance,
            estimate_prior_variance = estimate_prior_variance,
            estimate_prior_method = estimate_prior_method,
            check_null_threshold = check_null_threshold, prior_tol = prior_tol,
            r_tol = r_tol, max_iter = max_iter, null_weight = null_weight,
            standardize = standardize, coverage = coverage,
            min_abs_corr = min_abs_corr, tol = tol, verbose = FALSE,
            track_fit = FALSE, check_input = FALSE, refine = FALSE)
        m = c(m,list(s3))
      }
      if(length(m) == 0){
        conti = FALSE
      }else{
        elbo = sapply(m, function(x) susie_get_objective(x))
        if ((max(elbo) - susie_get_objective(s)) <= 0)
          conti = FALSE
        else
          s = m[[which.max(elbo)]]
      }
    }
  }

  return(s)
}

context("test_susie_XtX_Xty.R")

test_that(paste("compute_ss XtX calculations are correct with",
                "standardize = FALSE and with standardize = TRUE"),{
  set.seed(1)
  n <- 10
  p <- 5
  X <- matrix(rnorm(n*p),n,p)
  y <- rnorm(n)
  Y1 <- scale(X,center = TRUE,scale = FALSE)
  Y2 <- scale(X,center = TRUE,scale = TRUE)
  out1 <- compute_suff_stat(X,y,standardize = FALSE)
  out2 <- compute_suff_stat(X,y,standardize = TRUE)
  dimnames(out1$XtX) <- NULL
  dimnames(out2$XtX) <- NULL
  expect_equal(out1$XtX,crossprod(Y1),scale = 1,tolerance = 1e-14)
  expect_equal(out2$XtX,crossprod(Y2),scale = 1,tolerance = 1e-14)

  # Run the same checks again, but with X now being a sparse matrix.
  X    <- as(X,"CsparseMatrix")
  out1 <- compute_suff_stat(X,y,standardize = FALSE)
  out2 <- compute_suff_stat(X,y,standardize = TRUE)
  dimnames(out1$XtX) <- NULL
  dimnames(out2$XtX) <- NULL
  expect_equal(out1$XtX,crossprod(Y1),scale = 1,tolerance = 1e-14)
  expect_equal(out2$XtX,crossprod(Y2),scale = 1,tolerance = 1e-14)
})

test_that("Results from sufficient stat vs original data", with(simulate(200,1000), {
  ss = compute_ss(X, y, standardize = FALSE)

  expect_warning(res <- susie(X, y, intercept = TRUE, standardize = TRUE, max_iter = 2,
                              estimate_residual_variance=FALSE, estimate_prior_variance = FALSE))

  expect_warning(res2 <- susie_suff_stat(XtX = ss$XtX, Xty = ss$Xty, yty = ss$yty,
                                  n = ss$n, standardize = TRUE, max_iter = 2,
                                  estimate_prior_variance =FALSE, estimate_residual_variance = FALSE))

  expect_equal(res$alpha, res2$alpha)
  expect_equal(res$mu, res2$mu)
  expect_equal(res$mu2, res2$mu2)
  expect_equal(res$V, res2$V)
  expect_equal(res$elbo, res2$elbo)
  expect_equal(coef(res)[-1], coef(res2)[-1])

  cm  = colMeans(X, na.rm = TRUE)
  csd = compute_colSds(X)
  csd[csd==0] = 1
  X.cs = t( (t(X) - cm) / csd )
  expect_equal(crossprod(X.cs, res$Xr), res2$XtXr)
}))

test_that("Results from sufficient stat vs original data: estimate residual variance", with(simulate(200,1000), {
  ss = compute_ss(X, y, standardize = FALSE)

  expect_warning(res <- susie(X, y, intercept = TRUE, standardize = TRUE, max_iter = 2,
              estimate_residual_variance=TRUE, estimate_prior_variance = FALSE))

  expect_warning(res2 <- susie_suff_stat(XtX = ss$XtX, Xty = ss$Xty, yty = ss$yty,
                  n = ss$n, standardize = TRUE, max_iter = 2,
                  estimate_prior_variance = FALSE, estimate_residual_variance = TRUE))

  expect_equal(res$alpha, res2$alpha)
  expect_equal(res$mu, res2$mu)
  expect_equal(res$mu2, res2$mu2)
  expect_equal(res$V, res2$V)
  expect_equal(res$elbo, res2$elbo)
  expect_equal(res$sigma2, res2$sigma2)
  expect_equal(coef(res)[-1], coef(res2)[-1])

  cm = colMeans(X, na.rm = TRUE)
  csd = compute_colSds(X)
  csd[csd==0] = 1
  X.cs = t( (t(X) - cm) / csd )
  expect_equal(crossprod(X.cs, res$Xr), res2$XtXr)
}))

test_that("MAF filter works", with(simulate(200,1000), {
  X.maf = cbind(X, 0)
  ss = compute_ss(X, y, standardize = FALSE)
  ss.maf = compute_ss(X.maf, y, standardize = FALSE)
  maf = c(rep(1,1000),0.01)
  res1 <- susie_suff_stat(XtX = ss$XtX, Xty = ss$Xty, yty = ss$yty,
                   n = ss$n, standardize = TRUE,
                   estimate_prior_variance =FALSE, estimate_residual_variance = FALSE)

  res2 <- susie_suff_stat(XtX = ss.maf$XtX, Xty = ss.maf$Xty, yty = ss$yty,
                   n = ss$n, maf_thresh = 0.05, maf = maf, standardize = TRUE,
                   estimate_prior_variance =FALSE, estimate_residual_variance = FALSE)

  expect_equal(res1$alpha, res2$alpha)
  expect_equal(res1$mu, res2$mu)
  expect_equal(res1$mu2, res2$mu2)
  expect_equal(res1$V, res2$V)
  expect_equal(res1$elbo, res2$elbo)
  expect_equal(coef(res1), coef(res2))
  expect_equal(res1$XtXr, res2$XtXr)

}))

context("test_null_weight.R")

test_that("null weight specification agrees with default", with(simulate(200,1000), {
  res1 = susie(cbind(X,0), y, estimate_prior_variance = TRUE)
  res2 = susie(X, y, estimate_prior_variance = TRUE, null_weight = 1/(ncol(X)+1))
  expect_equal_susie(res2,res1,tolerance = 1e-6)
}))

test_that("Sufficient stat (Xty): null weight specification agrees with default", with(simulate(200,1000), {
  ss = compute_ss(X,y)

  res1 = susie_suff_stat(XtX = cbind(rbind(ss$XtX,0),0), Xty = c(ss$Xty, 0), yty = ss$yty, n = ss$n,
                  estimate_prior_variance = TRUE, estimate_residual_variance = TRUE)

  res2 = susie_suff_stat(XtX = ss$XtX, Xty = ss$Xty, yty = ss$yty, n = ss$n,
                  estimate_prior_variance = TRUE, estimate_residual_variance = TRUE,
                  null_weight = 1/(ncol(ss$XtX)+1))
  expect_equal_susie_suff_stat(res2,res1,tolerance = 1e-6)
}))

test_that("RSS: null weight specification agrees with default", with(simulate(200,500), {
  ss = univariate_regression(X, y)
  R = cor(X)
  z = ss$betahat/ss$sebetahat
  res1 = susie_rss(z = c(z, 0), R = cbind(rbind(R,0),0), n=n,
                   estimate_prior_variance = TRUE)

  res2 = susie_rss(z = z, R = R,n=n,
                   estimate_prior_variance = TRUE,
                   null_weight = 1/(ncol(R)+1))
  expect_equal_susie_suff_stat(res2,res1,tolerance = 1e-04)
}))

context("test_susie_beta_se.R")

test_that("Results from sufficient stat vs original data: no standardize", with(simulate(200,1000), {
  ss = univariate_regression(X, y)
  suff_stat = compute_suff_stat(X, y)

  R = cor(X)

  orig <- susie(X, y, intercept = TRUE, standardize = FALSE,
                estimate_residual_variance=FALSE, estimate_prior_variance = FALSE)

  fit_suff <- susie_suff_stat(XtX = suff_stat$XtX,
                              Xty = suff_stat$Xty,
                              yty = suff_stat$yty,
                              n = suff_stat$n,
                              standardize = FALSE,
                              estimate_prior_variance = FALSE,
                              estimate_residual_variance = FALSE)

  fit_rss <- susie_rss(bhat = ss$betahat,
                   shat = ss$sebetahat, R = R,
                   var_y = var(y), n = n, standardize = FALSE,
                   estimate_prior_variance = FALSE,
                   estimate_residual_variance = FALSE)

  expect_equal(fit_suff$alpha, orig$alpha)
  expect_equal(fit_suff$mu, orig$mu)
  expect_equal(fit_suff$mu2, orig$mu2)
  expect_equal(fit_suff$V, orig$V)
  X.c = set_X_attributes(X, center = TRUE, scale = FALSE)
  X.c = t((t(X.c) - attr(X.c, 'scaled:center'))/attr(X.c, 'scaled:scale'))
  expect_equal(crossprod(X.c, orig$Xr), fit_suff$XtXr)

  expect_equal(fit_rss$alpha, orig$alpha)
  expect_equal(fit_rss$mu, orig$mu)
  expect_equal(fit_rss$mu2, orig$mu2)
  expect_equal(fit_rss$V, orig$V)
  expect_equal(crossprod(X.c, orig$Xr), fit_rss$XtXr)
}))

test_that("Results from sufficient stat interface vs original data: standardize", with(simulate(200,1000), {
  ss = univariate_regression(X, y)
  suff_stat = compute_suff_stat(X, y)
  R = cor(X)

  X.cs = set_X_attributes(X, center = TRUE, scale = TRUE)
  X.cs = t((t(X.cs) - attr(X.cs, 'scaled:center'))/attr(X.cs, 'scaled:scale'))

  orig <- susie(X, y, intercept = TRUE, standardize = TRUE,
                estimate_residual_variance=FALSE, estimate_prior_variance = FALSE)

  fit_suff <- susie_suff_stat(XtX = suff_stat$XtX,
                              Xty = suff_stat$Xty,
                              yty = suff_stat$yty,
                              n = suff_stat$n,
                              standardize = TRUE,
                              estimate_prior_variance = FALSE,
                              estimate_residual_variance = FALSE)

  fit_rss <- susie_rss(bhat = ss$betahat, shat = ss$sebetahat, R = R,
                       n = n, var_y = var(y), standardize = TRUE,
                       estimate_prior_variance = FALSE,
                       estimate_residual_variance = FALSE)

  expect_equal(fit_suff$alpha, orig$alpha)
  expect_equal(fit_suff$mu, orig$mu)
  expect_equal(fit_suff$mu2, orig$mu2)
  expect_equal(fit_suff$V, orig$V)
  expect_equal(crossprod(X.cs, orig$Xr), fit_suff$XtXr)

  expect_equal(fit_rss$alpha, orig$alpha)
  expect_equal(fit_rss$mu, orig$mu)
  expect_equal(fit_rss$mu2, orig$mu2)
  expect_equal(fit_rss$V, orig$V)
  expect_equal(crossprod(X.cs, orig$Xr), fit_rss$XtXr)
}))

test_that("Results from sufficient stat interface vs original data on standaridzed input", with(simulate(200,1000), {
  ss = univariate_regression(X, y)
  y = y - mean(y)
  y = y/sd(y)
  suff_stat = compute_suff_stat(X, y, standardize = TRUE)
  R = cor(X)

  mu  = colMeans(X)
  s   = compute_colSds(X)
  X.cs = t((t(X) - mu)/s)

  orig <- susie(X.cs, y, intercept = FALSE, standardize = FALSE,
               estimate_residual_variance=FALSE, estimate_prior_variance = FALSE)

  fit_suff <- susie_suff_stat(XtX = suff_stat$XtX,
                              Xty = suff_stat$Xty,
                              yty = suff_stat$yty,
                              n = suff_stat$n,
                              standardize = FALSE,
                              estimate_prior_variance = FALSE,
                              estimate_residual_variance = FALSE)

  fit_rss <- susie_rss(bhat = ss$betahat, shat = ss$sebetahat, R = R,
                       n = n, standardize = TRUE,
                       estimate_prior_variance = FALSE,
                       estimate_residual_variance = FALSE)

  expect_equal(fit_suff$alpha, orig$alpha)
  expect_equal(fit_suff$mu, orig$mu)
  expect_equal(fit_suff$mu2, orig$mu2)
  expect_equal(fit_suff$V, orig$V)
  expect_equal(crossprod(X.cs, orig$Xr), fit_suff$XtXr)

  expect_equal(fit_rss$alpha, orig$alpha)
  expect_equal(fit_rss$mu, orig$mu)
  expect_equal(fit_rss$mu2, orig$mu2)
  expect_equal(fit_rss$V, orig$V)
  expect_equal(crossprod(X.cs, orig$Xr), fit_rss$XtXr)
}))

context("test_prior_weights.R")

test_that("prior weights specification agrees with default", with(simulate(200,1000), {
  res1 = susie(X, y, estimate_prior_variance = TRUE)
  res2 = susie(X, y, estimate_prior_variance = TRUE,
               prior_weights = rep(1/ncol(X), ncol(X)))
  expect_equal_susie(res1,res2,tolerance = 0.1)
}))

test_that("Sufficient stat (Xty): prior weights specification agrees with default", with(simulate(200,1000), {
  ss = compute_ss(X, y)
  res1 = susie_suff_stat(XtX = ss$XtX, Xty = ss$Xty, yty = ss$yty, n = n, estimate_prior_variance = TRUE)
  res2 = susie_suff_stat(XtX = ss$XtX, Xty = ss$Xty, yty = ss$yty, n = n, estimate_prior_variance = TRUE,
                  prior_weights = rep(1/ncol(ss$XtX), ncol(ss$XtX)))
  expect_equal_susie_suff_stat(res1,res2,tolerance = 1e-6)
}))

test_that("Sufficient stat (beta): prior weights specification agrees with default", with(simulate(200,1000), {
  ss = univariate_regression(X, y)
  R = cor(X)
  res1 = susie_rss(bhat = ss$betahat,shat = ss$sebetahat,R = R,
                   n = n, var_y = var(y), estimate_prior_variance = TRUE)
  res2 = susie_rss(bhat = ss$betahat, shat = ss$sebetahat, R = R, n = n, var_y = var(y),
                   estimate_prior_variance = TRUE,
                   prior_weights = rep(1/ncol(R), ncol(R)))
  expect_equal_susie_suff_stat(res1,res2,tolerance = 0.001)
}))

test_that("RSS: prior weights specification agrees with default", with(simulate(200,500), {
  ss = univariate_regression(X, y)
  R = cor(X)
  set.seed(1) # otherwise it might fail because the two results will be slighly different
  res1 = susie_rss(z = ss$betahat/ss$sebetahat, R = R, n=n, estimate_prior_variance = TRUE)
  res2 = susie_rss(z = ss$betahat/ss$sebetahat, R = R, n=n, estimate_prior_variance = TRUE,
                   prior_weights = rep(1/ncol(R), ncol(R)))
  expect_equal_susie_suff_stat(res1,res2,tolerance = 1e-05)
}))

library(curl)
data_file <- tempfile(fileext = ".RData")
data_url <- paste0("https://raw.githubusercontent.com/stephenslab/susieR/",
                   "master/inst/datafiles/SummaryConsistency1k.RData")
curl_download(data_url,data_file)
load(data_file)
zflip = SummaryConsistency$z
ld = SummaryConsistency$ldref
b = numeric(length(zflip))
b[SummaryConsistency$signal_id] = zflip[SummaryConsistency$signal_id]

fit = susie_rss(zflip, ld)
susie_plot(fit, y='PIP', b=b)
points(SummaryConsistency$flip_id, fit$pip[SummaryConsistency$flip_id],
       col=7, pch=16)

condz = kriging_rss(zflip, ld)
condz$plot

library(ggplot2)
library(cowplot)
flipped <- rep(FALSE,1002)
flipped[SummaryConsistency$flip_id] <- TRUE
pdat <- subset(condz$conditional_dist,is.finite((logLR)))
ggplot(pdat,aes(x = z,y = -logLR,color = flipped)) +
  geom_point() +
  scale_color_manual(values = c("darkblue","darkorange")) +
  theme_cowplot()

# @title Get objective function from data and susie fit object.
# @param data A flash data object.
# @param f A flash fit object.
# @keywords internal
get_objective = function (X, Y, s) {
  return(Eloglik(X,Y,s) - sum(s$KL))
}

# Expected loglikelihood for a susie fit.
Eloglik = function (X, Y, s) {
  n = nrow(X)
  return(-(n/2) * log(2*pi*s$sigma2) - (1/(2*s$sigma2)) * get_ER2(X,Y,s))
}

# Expected squared residuals.
# s$Xr is column sum of Xr_L
get_ER2 = function (X, Y, s) {
  Xr_L = compute_MXt(s$alpha * s$mu,X) # L by N matrix
  postb2 = s$alpha * s$mu2 # Posterior second moment.
  return(sum((Y - s$Xr)^2) - sum(Xr_L^2) + sum(attr(X,"d") * t(postb2)))
}

# @title posterior expected loglikelihood for a single effect regression
# @param X an n by p matrix of covariates
# @param Y an n vector of regression outcome
# @param s2 the residual variance
# @param Eb the posterior mean of b (p vector) (alpha * mu)
# @param Eb2 the posterior second moment of b (p vector) (alpha * mu2)
SER_posterior_e_loglik = function (X, Y, s2, Eb, Eb2) {
  n = nrow(X)
  return(-0.5*n*log(2*pi*s2) - 0.5/s2*(sum(Y*Y)
                                       - 2*sum(Y*compute_Xb(X,Eb))
                                       + sum(attr(X,"d") * Eb2)))
}

context("test_get_objective.R")

test_that("get_objective agrees with version 0.3", with(simulate(sparse = TRUE), {
  original.res = readRDS('objective_original_res.rds')

  scaledX = set_X_attributes(X)
  scaledX.sparse = set_X_attributes(X.sparse)
  s$Xr = colSums(compute_MXt(s$alpha*s$mu, scaledX))

  dense.res = get_objective(scaledX, y, s)
  sparse.res = get_objective(scaledX.sparse, y, s)

  expect_equal(dense.res, original.res)
  expect_equal(sparse.res, original.res)

}))

context("test_SER_posterior_e_loglik.R")

test_that("SER_posterior_e_loglik agrees with version 0.3", with(simulate(sparse=T), {
  original.res = readRDS('SER_original_res.rds')
  Eb = rep(1, p)
  Eb2 = rep(1, p)
  s2 = s$sigma2
  
  scaledX = set_X_attributes(X)
  scaledX.sparse = set_X_attributes(X.sparse)
  
  dense.res = SER_posterior_e_loglik(scaledX,y,s2,Eb,Eb2)
  sparse.res = SER_posterior_e_loglik(scaledX.sparse,y,s2,Eb,Eb2)
  
  expect_equal(dense.res, original.res)
  expect_equal(sparse.res, original.res)
  
}))

context("test_Eloglik.R")

test_that("Eloglik agrees with version 0.3", with(simulate(sparse = TRUE), {
  original.res = readRDS('Eloglik_original_res.rds')

  scaledX = set_X_attributes(X)
  scaledX.sparse = set_X_attributes(X.sparse)
  s$Xr = colSums(compute_MXt(s$alpha*s$mu, scaledX))

  dense.res  = Eloglik(scaledX, y, s)
  sparse.res = Eloglik(scaledX.sparse, y, s)
  expect_equal(dense.res, original.res)
  expect_equal(sparse.res, original.res)
}))

context("test_get_objective_rss.R")

test_that("get_objective (lambda = 0) agrees with previous version", with(simulate(200, 500), {
  original.res = readRDS('objective_rss_lambda0_res.rds')$s

  ss = univariate_regression(X, y)
  R = cor(X)
  z = ss$betahat/ss$sebetahat
  R = set_R_attributes(R, 1e-08)
  attr(R, 'lambda') = 0

  res = get_objective_rss(R, z, s)

  expect_equal(res, original.res, tolerance=1e-4)
}))

test_that("get_objective (lambda = 1) agrees with previous version", with(simulate(200, 500), {
  original.res = readRDS('objective_rss_lambda1_res.rds')$s

  ss = univariate_regression(X, y)
  R = cor(X)
  z = ss$betahat/ss$sebetahat
  R = set_R_attributes(R, 1e-08)
  attr(R, 'lambda') = 1

  res = get_objective_rss(R, z, s)

  expect_equal(res, original.res, tolerance=1e-4)
}))

#' @title Attempt at Automating SuSiE for Hard Problems
#'
#' @description \code{susie_auto} is an attempt to automate reliable
#'   running of susie even on hard problems. It implements a three-stage
#'   strategy for each L: first, fit susie with very small residual
#'   error; next, estimate residual error; finally, estimate the prior
#'   variance. If the last step estimates some prior variances to be
#'   zero, stop. Otherwise, double L, and repeat. Initial runs are
#'   performed with relaxed tolerance; the final run is performed using
#'   the default susie tolerance.
#'
#' @param X An n by p matrix of covariates.
#'
#' @param y The observed responses, a vector of length n.
#'
#' @param L_init The initial value of L.
#'
#' @param L_max The largest value of L to consider.
#'
#' @param verbose If \code{verbose = TRUE}, the algorithm's progress,
#'   and a summary of the optimization settings, are printed to the
#'   console.
#'
#' @param init_tol The tolerance to passed to \code{susie} during
#'   early runs (set large to shorten the initial runs).
#'
#' @param standardize If \code{standardize = TRUE}, standardize the
#'   columns of X to unit variance prior to fitting. Note that
#'   \code{scaled_prior_variance} specifies the prior on the
#'   coefficients of X \emph{after} standardization (if it is
#'   performed). If you do not standardize, you may need to think more
#'   carefully about specifying \code{scaled_prior_variance}. Whatever
#'   your choice, the coefficients returned by \code{coef} are given for
#'   \code{X} on the original input scale. Any column of \code{X} that
#'   has zero variance is not standardized.
#'
#' @param intercept If \code{intercept = TRUE}, the intercept is
#'   fitted; it \code{intercept = FALSE}, the intercept is set to
#'   zero. Setting \code{intercept = FALSE} is generally not
#'   recommended.
#'
#' @param max_iter Maximum number of IBSS iterations to perform.
#'
#' @param tol A small, non-negative number specifying the convergence
#'   tolerance for the IBSS fitting procedure. The fitting procedure
#'   will halt when the difference in the variational lower bound, or
#'   \dQuote{ELBO} (the objective function to be maximized), is
#'   less than \code{tol}.
#'
#' @param \dots Additional arguments passed to \code{\link{susie}}.
#'
#' @return See \code{\link{susie}} for a description of return values.
#'
#' @seealso \code{\link{susie}}
#'
#' @examples
#' set.seed(1)
#' n = 1000
#' p = 1000
#' beta = rep(0,p)
#' beta[1:4] = 1
#' X = matrix(rnorm(n*p),nrow = n,ncol = p)
#' X = scale(X,center = TRUE,scale = TRUE)
#' y = drop(X %*% beta + rnorm(n))
#' res = susie_auto(X,y)
#' plot(beta,coef(res)[-1])
#' abline(a = 0,b = 1,col = "skyblue",lty = "dashed")
#' plot(y,predict(res))
#' abline(a = 0,b = 1,col = "skyblue",lty = "dashed")
#'
#' @importFrom stats sd
#'
#' @export
#'
susie_auto = function (X, y, L_init = 1, L_max = 512, verbose = FALSE,
                       init_tol = 1, standardize = TRUE, intercept = TRUE,
                       max_iter = 100,tol = 1e-2, ...) {
  L = L_init
  if (verbose)
    message(paste0("Trying L=",L))
  s.0 = susie(X,y,L = L,residual_variance = 0.01*sd(y)^2,tol = init_tol,
              scaled_prior_variance = 1,estimate_residual_variance = FALSE,
              estimate_prior_variance = FALSE,standardize = standardize,
              intercept = intercept,max_iter = max_iter,...)
  s.1 = susie(X,y,s_init = s.0,tol = init_tol,
              estimate_residual_variance = TRUE,
              estimate_prior_variance = FALSE,
              standardize = standardize,intercept = intercept,
              max_iter = max_iter,...)
  s.2 = susie(X,y,s_init = s.1,tol = init_tol,
              estimate_residual_variance = TRUE,
              estimate_prior_variance = TRUE,
              standardize = standardize,intercept = intercept,
              max_iter = max_iter,...)

  # We call it converged---i.e., L is "big enough"---if there are any
  # prior variances set to zero.
  converged = !all(s.2$V > 0)
  while (!converged & (L <= L_max)) {
    for (i in 1:L) {
      s.2 = add_null_effect(s.2,1) # Add in L more effects.
      s.2$sigma2 = 0.01*sd(y)^2    # Set residual variance to be small
                                   # again for next iteration.
    }
    L = 2*L
    if (verbose)
      message(paste0("Trying L=",L))
    s.0 = susie(X,y,s_init = s.2,tol = init_tol,
                estimate_residual_variance = FALSE,
                estimate_prior_variance = FALSE,
                standardize = standardize,intercept = intercept,
                max_iter = max_iter,...)
    s.1 = susie(X,y,s_init = s.0,tol = init_tol,
                estimate_residual_variance = TRUE,
                estimate_prior_variance = FALSE,
                standardize = standardize,intercept = intercept,
                max_iter = max_iter,...)
    s.2 = susie(X,y,s_init = s.1,tol = init_tol,
                estimate_residual_variance = TRUE,
                estimate_prior_variance = TRUE,
                standardize = standardize,intercept = intercept,
                max_iter = max_iter,...)

    # We call it converged---i.e., L is "big enough"---if there are
    # any prior variances set to zero.
    converged = !all(s.2$V > 0)
  }

  # Final run at default tolerance to improve fit.
  s.2 = susie(X,y,s_init = s.2,estimate_residual_variance = TRUE,
              estimate_prior_variance = TRUE,tol = tol,
              standardize = standardize,intercept = intercept,
              max_iter = max_iter,...)
  return(s.2)
}

# Set default susie initialization.
init_setup_rss = function(p, L, prior_variance, residual_variance,
                          prior_weights, null_weight) {
  if (!is.numeric(prior_variance) || prior_variance < 0)
    stop("Prior variance should be positive number.")
  if(!is.null(residual_variance) &&
     (residual_variance > 1 | residual_variance < 0))
    stop("Residual variance should be a scaler between 0 and 1")
  if (is.null(residual_variance))
    residual_variance = 1
  if (is.null(prior_weights)){
    prior_weights = rep(1/p,p)
  }else{
   if(all(prior_weights == 0)){
     stop("Prior weight should greater than 0 for at least one variable.")
   }
   prior_weights = prior_weights / sum(prior_weights)
  }
  if(length(prior_weights) != p)
    stop("Prior weights must have length p")
  if (p < L)
    L = p
  s = list(alpha  = matrix(1/p,nrow = L,ncol = p),
           mu     = matrix(0,nrow = L,ncol = p),
           mu2    = matrix(0,nrow = L,ncol = p),
           Rz     = rep(0,p),
           KL     = rep(as.numeric(NA),L),
           lbf    = rep(as.numeric(NA),L),
           lbf_variable = matrix(as.numeric(NA),L,p),
           sigma2 = residual_variance,
           V      = prior_variance,
           pi     = prior_weights)
  if (is.null(null_weight))
    s$null_index = 0
  else
    s$null_index = p
  class(s) = "susie"
  return(s)
}

# Update a susie fit object in order to initialize susie model.
init_finalize_rss = function (s, R = NULL, Rz = NULL) {
  if(length(s$V) == 1)
    s$V = rep(s$V,nrow(s$alpha))

  # Check sigma2.
  if (!is.numeric(s$sigma2))
    stop("Input residual variance sigma2 must be numeric")

  # Avoid problems with dimension if input is a 1 x 1 matrix.
  s$sigma2 = as.numeric(s$sigma2)
  if (length(s$sigma2) != 1)
    stop("Input residual variance sigma2 must be a scalar")
  if (s$sigma2 <= 0)
    stop("residual variance sigma2 must be positive (is your var(y) zero?)")

  # Check prior variance.
  if (!is.numeric(s$V))
    stop("Input prior variance must be numeric")
  if (!all(s$V >= 0))
    stop("prior variance must be non-negative")
  if (!all(dim(s$mu) == dim(s$mu2)))
    stop("dimension of mu and mu2 in input object do not match")
  if (!all(dim(s$mu) == dim(s$alpha)))
    stop("dimension of mu and alpha in input object do not match")
  if (nrow(s$alpha) != length(s$V))
    stop("Input prior variance V must have length of nrow of alpha in ",
         "input object")

  # Update Rz.
  if (!missing(Rz))
    s$Rz = Rz
  if (!missing(R))
    s$Rz = compute_Xb(R,colSums(s$mu * s$alpha))

  # Reset KL and lbf.
  s$KL = rep(as.numeric(NA),nrow(s$alpha))
  s$lbf = rep(as.numeric(NA),nrow(s$alpha))
  class(s) = "susie"
  return(s)
}

# @title Get objective function from data and susie fit object.
# @param XtX a p by p matrix, X'X
# @param Xty a p vector, X'y,
# @param s a susie fit object
# @param yty a scaler, y'y, where y is centered to have mean 0
# @param n sample size
get_objective_ss = function (XtX, Xty, s, yty, n)
  Eloglik_ss(XtX,Xty,s,yty,n) - sum(s$KL)

# Expected loglikelihood for a susie fit.
Eloglik_ss = function (XtX, Xty, s, yty, n)
  -n/2*log(2*pi*s$sigma2) - 1/(2*s$sigma2) * get_ER2_ss(XtX,Xty,s,yty)

# Expected squared residuals.
get_ER2_ss = function (XtX, Xty, s, yty) {
  B = s$alpha * s$mu
  XB2 = sum((B %*% XtX) * B)
  betabar = colSums(B)
  d = attr(XtX,"d")
  postb2 = s$alpha * s$mu2 # Posterior second moment.
  return(yty - 2*sum(betabar * Xty) + sum(betabar * (XtX %*% betabar)) -
         XB2 + sum(d * t(postb2)))
}

# @title posterior expected loglikelihood for a single effect regression
# @param dXtX a p vector of diagonal elements of XtX
# @param Xty a p vector
# @param s2 the residual variance
# @param Eb the posterior mean of b (p vector) (alpha * mu)
# @param Eb2 the posterior second moment of b (p vector) (alpha * mu2)
SER_posterior_e_loglik_ss = function (dXtX, Xty, s2, Eb, Eb2)
  -0.5/s2 * (-2*sum(Eb*Xty) + sum(dXtX * as.vector(Eb2)))

#' @title Sum of Single Effects (SuSiE) Regression using Summary Statistics
#'
#' @description \code{susie_rss} performs variable selection under a
#'   sparse Bayesian multiple linear regression of \eqn{Y} on \eqn{X}
#'   using the z-scores from standard univariate regression
#'   of \eqn{Y} on each column of \eqn{X}, an estimate, \eqn{R}, of
#'   the correlation matrix for the columns of \eqn{X}, and optionally,
#'   \emph{but strongly recommended}, the sample size n. See
#'   \dQuote{Details} for other ways to call \code{susie_rss}
#'
#' @details In some applications, particularly genetic applications,
#' it is desired to fit a regression model (\eqn{Y = Xb + E} say,
#' which we refer to as "the original regression model" or ORM)
#' without access to the actual values of \eqn{Y} and \eqn{X}, but
#' given only some summary statistics. \code{susie_rss} assumes
#' availability of z-scores from standard univariate regression of
#' \eqn{Y} on each column of \eqn{X}, and an estimate, \eqn{R}, of the
#' correlation matrix for the columns of \eqn{X} (in genetic
#' applications \eqn{R} is sometimes called the \dQuote{LD matrix}).
#'
#' With the inputs \code{z}, \code{R} and sample size \code{n},
#' \code{susie_rss} computes PVE-adjusted z-scores \code{z_tilde}, and
#' calls \code{susie_suff_stat} with \code{XtX = (n-1)R}, \code{Xty =
#' } \eqn{\sqrt{n-1} z_tilde}, \code{yty = n-1}, \code{n = n}. The
#' output effect estimates are on the scale of \eqn{b} in the ORM with
#' \emph{standardized} \eqn{X} and \eqn{y}. When the LD matrix
#' \code{R} and the z-scores \code{z} are computed using the same
#' matrix \eqn{X}, the results from \code{susie_rss} are same as, or
#' very similar to, \code{susie} with \emph{standardized} \eqn{X} and
#' \eqn{y}.
#'
#' Alternatively, if the user provides \code{n}, \code{bhat} (the
#' univariate OLS estimates from regressing \eqn{y} on each column of
#' \eqn{X}), \code{shat} (the standard errors from these OLS
#' regressions), the in-sample correlation matrix \eqn{R =
#' cov2cor(crossprod(X))}, and the variance of \eqn{y}, the results
#' from \code{susie_rss} are same as \code{susie} with \eqn{X} and
#' \eqn{y}. The effect estimates are on the same scale as the
#' coefficients \eqn{b} in the ORM with \eqn{X} and \eqn{y}.
#'
#' In rare cases in which the sample size, \eqn{n}, is unknown,
#' \code{susie_rss} calls \code{susie_suff_stat} with \code{XtX = R}
#' and \code{Xty = z}, and with \code{residual_variance = 1}. The
#' underlying assumption of performing the analysis in this way is
#' that the sample size is large (\emph{i.e.}, infinity), and/or the
#' effects are small. More formally, this combines the log-likelihood
#' for the noncentrality parameters, \eqn{\tilde{b} = \sqrt{n} b},
#' \deqn{L(\tilde{b}; z, R) = -(\tilde{b}'R\tilde{b} -
#' 2z'\tilde{b})/2,} with the \dQuote{susie prior} on
#' \eqn{\tilde{b}}; see \code{\link{susie}} and Wang \emph{et al}
#' (2020) for details. In this case, the effect estimates returned by
#' \code{susie_rss} are on the noncentrality parameter scale.
#'
#' The \code{estimate_residual_variance} setting is \code{FALSE} by
#' default, which is recommended when the LD matrix is estimated from
#' a reference panel. When the LD matrix \code{R} and the summary
#' statistics \code{z} (or \code{bhat}, \code{shat}) are computed
#' using the same matrix \eqn{X}, we recommend setting
#' \code{estimate_residual_variance = TRUE}.
#'
#' @param z p-vector of z-scores.
#'
#' @param R p x p correlation matrix.
#'
#' @param n The sample size.
#'
#' @param bhat Alternative summary data giving the estimated effects
#'   (a vector of length p). This, together with \code{shat}, may be
#'   provided instead of \code{z}.
#'
#' @param shat Alternative summary data giving the standard errors of
#'   the estimated effects (a vector of length p). This, together with
#'   \code{bhat}, may be provided instead of \code{z}.
#'
#' @param var_y The sample variance of y, defined as \eqn{y'y/(n-1)}.
#'   When the sample variance is not provided, the coefficients
#'   (returned from \code{coef}) are computed on the
#'   \dQuote{standardized} X, y scale.
#'
#' @param z_ld_weight This parameter is included for backwards
#'   compatibility with previous versions of the function, but it is no
#'   longer recommended to set this to a non-zero value. When
#'   \code{z_ld_weight > 0}, the matrix \code{R} is adjusted to be
#'   \code{cov2cor((1-w)*R + w*tcrossprod(z))}, where \code{w =
#'   z_ld_weight}.
#'
#' @param prior_variance The prior variance(s) for the non-zero
#'   noncentrality parameterss \eqn{\tilde{b}_l}. It is either a scalar,
#'   or a vector of length L. When the \code{susie_suff_stat} option
#'   \code{estimate_prior_variance} is set to \code{TRUE} (which is
#'   highly recommended) this simply provides an initial value for the
#'   prior variance. The default value of 50 is simply intended to be a
#'   large initial value. Note this setting is only relevant when
#'   \code{n} is unknown. If \code{n} is known, the relevant option is
#'   \code{scaled_prior_variance} in \code{\link{susie_suff_stat}}.
#'
#' @param estimate_residual_variance The default is FALSE, the
#'   residual variance is fixed to 1 or variance of y. If the in-sample
#'   LD matrix is provided, we recommend setting
#'   \code{estimate_residual_variance = TRUE}.
#'
#' @param check_prior When \code{check_prior = TRUE}, it checks if the
#'   estimated prior variance becomes unreasonably large (comparing with
#'   100 * max(abs(z))^2).
#'
#' @param \dots Other parameters to be passed to
#' \code{\link{susie_suff_stat}}.
#'
#' @return A \code{"susie"} object with the following
#'   elements:
#'
#' \item{alpha}{An L by p matrix of posterior inclusion probabilites.}
#'
#' \item{mu}{An L by p matrix of posterior means, conditional on
#'   inclusion.}
#'
#' \item{mu2}{An L by p matrix of posterior second moments,
#'   conditional on inclusion.}
#'
#' \item{lbf}{log-Bayes Factor for each single effect.}
#'
#' \item{lbf_variable}{log-Bayes Factor for each variable and single effect.}
#'
#' \item{V}{Prior variance of the non-zero elements of b.}
#'
#' \item{elbo}{The value of the variational lower bound, or
#'   \dQuote{ELBO} (objective function to be maximized), achieved at
#'   each iteration of the IBSS fitting procedure.}
#'
#' \item{sets}{Credible sets estimated from model fit; see
#'   \code{\link{susie_get_cs}} for details.}
#'
#' \item{pip}{A vector of length p giving the (marginal) posterior
#'   inclusion probabilities for all p covariates.}
#'
#' \item{niter}{Number of IBSS iterations that were performed.}
#'
#' \item{converged}{\code{TRUE} or \code{FALSE} indicating whether
#'   the IBSS converged to a solution within the chosen tolerance
#'   level.}
#'
#' @references
#' G. Wang, A. Sarkar, P. Carbonetto and M. Stephens (2020). A simple
#'   new approach to variable selection in regression, with application
#'   to genetic fine-mapping. \emph{Journal of the Royal Statistical
#'   Society, Series B} \bold{82}, 1273-1300 \doi{10.1101/501114}.
#'
#'   Y. Zou, P. Carbonetto, G. Wang and M. Stephens (2021).
#'   Fine-mapping from summary data with the \dQuote{Sum of Single Effects}
#'   model. \emph{bioRxiv} \doi{10.1101/2021.11.03.467167}.
#'
#' @examples
#' set.seed(1)
#' n = 1000
#' p = 1000
#' beta = rep(0,p)
#' beta[1:4] = 1
#' X = matrix(rnorm(n*p),nrow = n,ncol = p)
#' X = scale(X,center = TRUE,scale = TRUE)
#' y = drop(X %*% beta + rnorm(n))
#'
#' input_ss = compute_suff_stat(X,y,standardize = TRUE)
#' ss   = univariate_regression(X,y)
#' R    = with(input_ss,cov2cor(XtX))
#' zhat = with(ss,betahat/sebetahat)
#' res  = susie_rss(zhat,R, n=n)
#'
#' # Toy example illustrating behaviour susie_rss when the z-scores
#' # are mostly consistent with a non-invertible correlation matrix.
#' # Here the CS should contain both variables, and two PIPs should
#' # be nearly the same.
#' z = c(6,6.01)
#' R = matrix(1,2,2)
#' fit = susie_rss(z,R)
#' print(fit$sets$cs)
#' print(fit$pip)
#'
#' # In this second toy example, the only difference is that one
#' # z-score is much larger than the other. Here we expect that the
#' # second PIP will be much larger than the first.
#' z = c(6,7)
#' R = matrix(1,2,2)
#' fit = susie_rss(z,R)
#' print(fit$sets$cs)
#' print(fit$pip)
#'
#' @export
#'
susie_rss = function (z, R, n, bhat, shat, var_y,
                      z_ld_weight = 0,
                      estimate_residual_variance = FALSE,
                      prior_variance = 50,
                      check_prior = TRUE, ...) {

  if (estimate_residual_variance)
    warning_message("For estimate_residual_variance = TRUE, please check ",
                    "that R is the \"in-sample\" LD matrix; that is, the ",
                    "correlation matrix obtained using the exact same data ",
                    "matrix X that was used for the other summary ",
                    "statistics. Also note, when covariates are included in ",
                    "the univariate regressions that produced the summary ",
                    "statistics, also consider removing these effects from ",
                    "X before computing R.",style = "hint")

  # Check input R.
  if (missing(z))
    p <- length(bhat)
  else
    p <- length(z)
  if (nrow(R) != p)
      stop(paste0("The dimension of R (",nrow(R)," x ",ncol(R),") does not ",
                  "agree with expected (",p," x ",p,")"))

  # Check input n.
  if (!missing(n))
    if (n <= 1)
      stop("n must be greater than 1")

  # Check inputs z, bhat and shat. Note that bhat is no longer used
  # after this step.
  if (sum(c(missing(z),missing(bhat) || missing(shat))) != 1)
    stop("Please provide either z or (bhat, shat), but not both")
  if (missing(z)) {
    if (length(shat) == 1)
      shat = rep(shat,length(bhat))
    if (length(bhat) != length(shat))
      stop("The lengths of bhat and shat do not agree")
    if (anyNA(bhat) || anyNA(shat))
      stop("bhat, shat cannot have missing values")
    if (any(shat <= 0))
      stop("shat cannot have zero or negative elements")
    z = bhat/shat
  }
  z[is.na(z)] = 0

  # When n is provided, compute the PVE-adjusted z-scores.
  if (!missing(n)) {
    adj = (n-1)/(z^2 + n - 2)
    z   = sqrt(adj) * z
  }

  # Modify R by z_ld_weight; this modification was designed to ensure
  # the column space of R contained z, but susie_suff_stat does not
  # require this, and is no longer recommended.
  if (z_ld_weight > 0) {
    warning_message("As of version 0.11.0, use of non-zero z_ld_weight is no longer ",
            "recommended")
    R = muffled_cov2cor((1-z_ld_weight)*R + z_ld_weight*tcrossprod(z))
    R = (R + t(R))/2
  }

  # Call susie_suff_stat. We call susie_suff_stat in two different
  # ways depending on whether n is provided.
  if (missing(n)) {

    # The sample size (n) is not provided, so use unadjusted z-scores.
    # The choice of n=2, yty=1 is mostly arbitrary except in that it
    # ensures var(y) = yty/(n-1) = 1, and because of this
    # scaled_prior_variance = prior_variance.
    warning_message("Providing the sample size (n), or even a rough estimate of n, ",
            "is highly recommended. Without n, the implicit assumption is ",
            "n is large (Inf) and the effect sizes are small (close to zero).")
    s = susie_suff_stat(XtX = R,Xty = z,n = 2,yty = 1,
                        scaled_prior_variance = prior_variance,
                        estimate_residual_variance = estimate_residual_variance,
                        standardize = FALSE,check_prior = check_prior,...)
  } else {

    # The sample size (n) is provided, so use PVE-adjusted z-scores.
    if (!missing(shat) & !missing(var_y)) {

      # var_y, shat (and bhat) are provided, so the effects are on the
      # *original scale*.
      XtXdiag = var_y * adj/(shat^2)
      XtX = t(R * sqrt(XtXdiag)) * sqrt(XtXdiag)
      XtX = (XtX + t(XtX))/2
      Xty = z * sqrt(adj) * var_y / shat
    } else {

      # The effects are on the *standardized* X, y scale.
      XtX = (n-1)*R
      Xty = sqrt(n-1)*z
      var_y = 1
    }
    s = susie_suff_stat(XtX = XtX,Xty = Xty,n = n,yty = (n-1)*var_y,
                        estimate_residual_variance = estimate_residual_variance,
                        check_prior = check_prior,...)
  }
  return(s)
}

context("test_susie_rss.R")

test_that("susie_rss (lambda = 0) agrees with previous version",
          with(simulate(200, 500),{
  skip("susie_rss_lambda is not currently being used")
  original.res  = readRDS('susierssfit_lambda0_res.rds')$s
  
  ss = univariate_regression(X, y)
  R = cor(X)
  z = ss$betahat/ss$sebetahat

  res = susie_rss_lambda(z, R, lambda = 0, tol=1E-2, check_z = FALSE)

  skip('susie_rss_lambda is updated.')
  expect_equal(res$converged, TRUE)
  expect_equal_susie_rss(res, original.res, tolerance=1e-4)
}))

test_that("susie_rss (lambda = 1) agrees with previous version",
          with(simulate(200, 500),{
  skip("susie_rss_lambda is not currently being used")
  original.res  = readRDS('susierssfit_lambda1_res.rds')$s

  ss = univariate_regression(X, y)
  R = cor(X)
  z = ss$betahat/ss$sebetahat

  res = susie_rss_lambda(z, R, lambda = 1, tol=1E-2, check_z = FALSE)

  skip('susie_rss_lambda is updated.')
  expect_equal(res$converged, TRUE)
  expect_equal_susie_rss(res, original.res, tolerance=1e-4)
}))

#' @rdname susie
#'
#' @title Sum of Single Effects (SuSiE) Regression
#'
#' @description Performs a sparse Bayesian multiple linear regression
#'   of y on X, using the "Sum of Single Effects" model from Wang et al
#'   (2020). In brief, this function fits the regression model \eqn{y =
#'   \mu + X b + e}, where elements of \eqn{e} are \emph{i.i.d.} normal
#'   with zero mean and variance \code{residual_variance}, \eqn{\mu} is
#'   an intercept term and \eqn{b} is a vector of length p representing
#'   the effects to be estimated. The \dQuote{susie assumption} is that
#'   \eqn{b = \sum_{l=1}^L b_l} where each \eqn{b_l} is a vector of
#'   length p with exactly one non-zero element. The prior on the
#'   non-zero element is normal with zero mean and variance \code{var(y)
#'   * scaled_prior_variance}. The value of \code{L} is fixed, and
#'   should be chosen to provide a reasonable upper bound on the number
#'   of non-zero effects to be detected. Typically, the hyperparameters
#'   \code{residual_variance} and \code{scaled_prior_variance} will be
#'   estimated during model fitting, although they can also be fixed as
#'   specified by the user. See functions \code{\link{susie_get_cs}} and
#'   other functions of form \code{susie_get_*} to extract the most
#'   commonly-used results from a susie fit.
#'
#' @details The function \code{susie} implements the IBSS algorithm
#' from Wang et al (2020). The option \code{refine = TRUE} implements
#' an additional step to help reduce problems caused by convergence of
#' the IBSS algorithm to poor local optima (which is rare in our
#' experience, but can provide misleading results when it occurs). The
#' refinement step incurs additional computational expense that
#' increases with the number of CSs found in the initial run.
#'
#' The function \code{susie_suff_stat} implements essentially the same
#' algorithms, but using sufficient statistics. (The statistics are
#' sufficient for the regression coefficients \eqn{b}, but not for the
#' intercept \eqn{\mu}; see below for how the intercept is treated.)
#' If the sufficient statistics are computed correctly then the
#' results from \code{susie_suff_stat} should be the same as (or very
#' similar to) \code{susie}, although runtimes will differ as
#' discussed below. The sufficient statistics are the sample
#' size \code{n}, and then the p by p matrix \eqn{X'X}, the p-vector
#' \eqn{X'y}, and the sum of squared y values \eqn{y'y}, all computed
#' after centering the columns of \eqn{X} and the vector \eqn{y} to
#' have mean 0; these can be computed using \code{compute_suff_stat}.
#'
#' The handling of the intercept term in \code{susie_suff_stat} needs
#' some additional explanation. Computing the summary data after
#' centering \code{X} and \code{y} effectively ensures that the
#' resulting posterior quantities for \eqn{b} allow for an intercept
#' in the model; however, the actual value of the intercept cannot be
#' estimated from these centered data. To estimate the intercept term
#' the user must also provide the column means of \eqn{X} and the mean
#' of \eqn{y} (\code{X_colmeans} and \code{y_mean}). If these are not
#' provided, they are treated as \code{NA}, which results in the
#' intercept being \code{NA}. If for some reason you prefer to have
#' the intercept be 0 instead of \code{NA} then set
#' \code{X_colmeans = 0,y_mean = 0}.
#'
#' For completeness, we note that if \code{susie_suff_stat} is run on
#' \eqn{X'X, X'y, y'y} computed \emph{without} centering \eqn{X} and
#' \eqn{y}, and with \code{X_colmeans = 0,y_mean = 0}, this is
#' equivalent to \code{susie} applied to \eqn{X, y} with
#' \code{intercept = FALSE} (although results may differ due to
#' different initializations of \code{residual_variance} and
#' \code{scaled_prior_variance}). However, this usage is not
#' recommended for for most situations.
#'
#' The computational complexity of \code{susie} is \eqn{O(npL)} per
#' iteration, whereas \code{susie_suff_stat} is \eqn{O(p^2L)} per
#' iteration (not including the cost of computing the sufficient
#' statistics, which is dominated by the \eqn{O(np^2)} cost of
#' computing \eqn{X'X}). Because of the cost of computing \eqn{X'X},
#' \code{susie} will usually be faster. However, if \eqn{n >> p},
#' and/or if \eqn{X'X} is already computed, then
#' \code{susie_suff_stat} may be faster.
#'
#' @param X An n by p matrix of covariates.
#'
#' @param y The observed responses, a vector of length n.
#'
#' @param L Maximum number of non-zero effects in the susie
#'   regression model. If L is larger than the number of covariates, p,
#'   L is set to p.
#'
#' @param scaled_prior_variance The prior variance, divided by
#'   \code{var(y)} (or by \code{(1/(n-1))yty} for
#'   \code{susie_suff_stat}); that is, the prior variance of each
#'   non-zero element of b is \code{var(y) * scaled_prior_variance}. The
#'   value provided should be either a scalar or a vector of length
#'   \code{L}. If \code{estimate_prior_variance = TRUE}, this provides
#'   initial estimates of the prior variances.
#'
#' @param residual_variance Variance of the residual. If
#'   \code{estimate_residual_variance = TRUE}, this value provides the
#'   initial estimate of the residual variance. By default, it is set to
#'   \code{var(y)} in \code{susie} and \code{(1/(n-1))yty} in
#'   \code{susie_suff_stat}.
#'
#' @param prior_weights A vector of length p, in which each entry
#'   gives the prior probability that corresponding column of X has a
#'   nonzero effect on the outcome, y.
#'
#' @param null_weight Prior probability of no effect (a number between
#'   0 and 1, and cannot be exactly 1).
#'
#' @param standardize If \code{standardize = TRUE}, standardize the
#'   columns of X to unit variance prior to fitting (or equivalently
#'   standardize XtX and Xty to have the same effect). Note that
#'   \code{scaled_prior_variance} specifies the prior on the
#'   coefficients of X \emph{after} standardization (if it is
#'   performed). If you do not standardize, you may need to think more
#'   carefully about specifying \code{scaled_prior_variance}. Whatever
#'   your choice, the coefficients returned by \code{coef} are given for
#'   \code{X} on the original input scale. Any column of \code{X} that
#'   has zero variance is not standardized.
#'
#' @param intercept If \code{intercept = TRUE}, the intercept is
#'   fitted; it \code{intercept = FALSE}, the intercept is set to
#'   zero. Setting \code{intercept = FALSE} is generally not
#'   recommended.
#'
#' @param estimate_residual_variance If
#'   \code{estimate_residual_variance = TRUE}, the residual variance is
#'   estimated, using \code{residual_variance} as an initial value. If
#'   \code{estimate_residual_variance = FALSE}, the residual variance is
#'   fixed to the value supplied by \code{residual_variance}.
#'
#' @param estimate_prior_variance If \code{estimate_prior_variance =
#'   TRUE}, the prior variance is estimated (this is a separate
#'   parameter for each of the L effects). If provided,
#'   \code{scaled_prior_variance} is then used as an initial value for
#'   the optimization. When \code{estimate_prior_variance = FALSE}, the
#'   prior variance for each of the L effects is determined by the
#'   value supplied to \code{scaled_prior_variance}.
#'
#' @param estimate_prior_method The method used for estimating prior
#'   variance. When \code{estimate_prior_method = "simple"} is used, the
#'   likelihood at the specified prior variance is compared to the
#'   likelihood at a variance of zero, and the setting with the larger
#'   likelihood is retained.
#'
#' @param check_null_threshold When the prior variance is estimated,
#'   compare the estimate with the null, and set the prior variance to
#'   zero unless the log-likelihood using the estimate is larger by this
#'   threshold amount. For example, if you set
#'   \code{check_null_threshold = 0.1}, this will "nudge" the estimate
#'   towards zero when the difference in log-likelihoods is small. A
#'   note of caution that setting this to a value greater than zero may
#'   lead the IBSS fitting procedure to occasionally decrease the ELBO.
#'
#' @param prior_tol When the prior variance is estimated, compare the
#'   estimated value to \code{prior_tol} at the end of the computation,
#'   and exclude a single effect from PIP computation if the estimated
#'   prior variance is smaller than this tolerance value.
#'
#' @param residual_variance_upperbound Upper limit on the estimated
#'   residual variance. It is only relevant when
#'   \code{estimate_residual_variance = TRUE}.
#'
#' @param s_init A previous susie fit with which to initialize.
#'
#' @param coverage A number between 0 and 1 specifying the
#'   \dQuote{coverage} of the estimated confidence sets.
#'
#' @param min_abs_corr Minimum absolute correlation allowed in a
#'   credible set. The default, 0.5, corresponds to a squared
#'   correlation of 0.25, which is a commonly used threshold for
#'   genotype data in genetic studies.
#'
#' @param compute_univariate_zscore If \code{compute_univariate_zscore
#'   = TRUE}, the univariate regression z-scores are outputted for each
#'   variable.
#'
#' @param na.rm Drop any missing values in y from both X and y.
#'
#' @param max_iter Maximum number of IBSS iterations to perform.
#'
#' @param tol A small, non-negative number specifying the convergence
#'   tolerance for the IBSS fitting procedure. The fitting procedure
#'   will halt when the difference in the variational lower bound, or
#'   \dQuote{ELBO} (the objective function to be maximized), is
#'   less than \code{tol}.
#'
#' @param verbose If \code{verbose = TRUE}, the algorithm's progress,
#'   and a summary of the optimization settings, are printed to the
#'   console.
#'
#' @param track_fit If \code{track_fit = TRUE}, \code{trace}
#'   is also returned containing detailed information about the
#'   estimates at each iteration of the IBSS fitting procedure.
#'
#' @param residual_variance_lowerbound Lower limit on the estimated
#'   residual variance. It is only relevant when
#'   \code{estimate_residual_variance = TRUE}.
#'
#' @param refine If \code{refine = TRUE}, then an additional
#'  iterative refinement procedure is used, after the IBSS algorithm,
#'  to check and escape from local optima (see details).
#'
#' @param n_purity Passed as argument \code{n_purity} to
#'   \code{\link{susie_get_cs}}.
#'
#' @return A \code{"susie"} object with some or all of the following
#'   elements:
#'
#' \item{alpha}{An L by p matrix of posterior inclusion probabilites.}
#'
#' \item{mu}{An L by p matrix of posterior means, conditional on
#'   inclusion.}
#'
#' \item{mu2}{An L by p matrix of posterior second moments,
#'   conditional on inclusion.}
#'
#' \item{Xr}{A vector of length n, equal to \code{X \%*\% colSums(alpha
#'   * mu)}.}
#'
#' \item{lbf}{log-Bayes Factor for each single effect.}
#'
#' \item{lbf_variable}{log-Bayes Factor for each variable and single effect.}
#'
#' \item{intercept}{Intercept (fixed or estimated).}
#'
#' \item{sigma2}{Residual variance (fixed or estimated).}
#'
#' \item{V}{Prior variance of the non-zero elements of b, equal to
#'   \code{scaled_prior_variance * var(y)}.}
#'
#' \item{elbo}{The value of the variational lower bound, or
#'   \dQuote{ELBO} (objective function to be maximized), achieved at
#'   each iteration of the IBSS fitting procedure.}
#'
#' \item{fitted}{Vector of length n containing the fitted values of
#'   the outcome.}
#'
#' \item{sets}{Credible sets estimated from model fit; see
#'   \code{\link{susie_get_cs}} for details.}
#'
#' \item{pip}{A vector of length p giving the (marginal) posterior
#'   inclusion probabilities for all p covariates.}
#'
#' \item{z}{A vector of univariate z-scores.}
#'
#' \item{niter}{Number of IBSS iterations that were performed.}
#'
#' \item{converged}{\code{TRUE} or \code{FALSE} indicating whether
#'   the IBSS converged to a solution within the chosen tolerance
#'   level.}
#'
#' \code{susie_suff_stat} returns also outputs:
#'
#' \item{XtXr}{A p-vector of \code{t(X)} times the fitted values,
#'   \code{X \%*\% colSums(alpha*mu)}.}
#'
#' @references
#' G. Wang, A. Sarkar, P. Carbonetto and M. Stephens (2020). A simple
#'   new approach to variable selection in regression, with application
#'   to genetic fine-mapping. \emph{Journal of the Royal Statistical
#'   Society, Series B} \bold{82}, 1273-1300 \doi{10.1101/501114}.
#'
#'   Y. Zou, P. Carbonetto, G. Wang and M. Stephens (2021).
#'   Fine-mapping from summary data with the \dQuote{Sum of Single Effects}
#'   model. \emph{bioRxiv} \doi{10.1101/2021.11.03.467167}.
#'
#' @seealso \code{\link{susie_get_cs}} and other \code{susie_get_*}
#'   functions for extracting results; \code{\link{susie_trendfilter}} for
#'   applying the SuSiE model to non-parametric regression, particularly
#'   changepoint problems, and \code{\link{susie_rss}} for applying the
#'   SuSiE model when one only has access to limited summary statistics
#'   related to \eqn{X} and \eqn{y} (typically in genetic applications).
#'
#' @examples
#' # susie example
#' set.seed(1)
#' n = 1000
#' p = 1000
#' beta = rep(0,p)
#' beta[1:4] = 1
#' X = matrix(rnorm(n*p),nrow = n,ncol = p)
#' X = scale(X,center = TRUE,scale = TRUE)
#' y = drop(X %*% beta + rnorm(n))
#' res1 = susie(X,y,L = 10)
#' susie_get_cs(res1) # extract credible sets from fit
#' plot(beta,coef(res1)[-1])
#' abline(a = 0,b = 1,col = "skyblue",lty = "dashed")
#' plot(y,predict(res1))
#' abline(a = 0,b = 1,col = "skyblue",lty = "dashed")
#'
#' # susie_suff_stat example
#' input_ss = compute_suff_stat(X,y)
#' res2 = with(input_ss,
#'             susie_suff_stat(XtX = XtX,Xty = Xty,yty = yty,n = n,
#'                             X_colmeans = X_colmeans,y_mean = y_mean,L = 10))
#' plot(coef(res1),coef(res2))
#' abline(a = 0,b = 1,col = "skyblue",lty = "dashed")
#'
#' @importFrom stats var
#' @importFrom utils modifyList
#'
#' @export
#'
susie = function (X,y,L = min(10,ncol(X)),
                   scaled_prior_variance = 0.2,
                   residual_variance = NULL,
                   prior_weights = NULL,
                   null_weight = 0,
                   standardize = TRUE,
                   intercept = TRUE,
                   estimate_residual_variance = TRUE,
                   estimate_prior_variance = TRUE,
                   estimate_prior_method = c("optim", "EM", "simple"),
                   check_null_threshold = 0,
                   prior_tol = 1e-9,
                   residual_variance_upperbound = Inf,
                   s_init = NULL,
                   coverage = 0.95,
                   min_abs_corr = 0.5,
                   compute_univariate_zscore = FALSE,
                   na.rm = FALSE,
                   max_iter = 100,
                   tol = 1e-3,
                   verbose = FALSE,
                   track_fit = FALSE,
                   residual_variance_lowerbound = var(drop(y))/1e4,
                   refine = FALSE,
                   n_purity = 100) {

  # Process input estimate_prior_method.
  estimate_prior_method = match.arg(estimate_prior_method)

  # Check input X.
  if (!(is.double(X) & is.matrix(X)) &
      !inherits(X,"CsparseMatrix") &
      is.null(attr(X,"matrix.type")))
    stop("Input X must be a double-precision matrix, or a sparse matrix, or ",
         "a trend filtering matrix")
  if (is.numeric(null_weight) && null_weight == 0)
    null_weight = NULL
  if (!is.null(null_weight) && is.null(attr(X,"matrix.type"))) {
    if (!is.numeric(null_weight))
      stop("Null weight must be numeric")
    if (null_weight < 0 || null_weight >= 1)
      stop("Null weight must be between 0 and 1")
    if (missing(prior_weights))
      prior_weights = c(rep(1/ncol(X) * (1 - null_weight),ncol(X)),null_weight)
    else
      prior_weights = c(prior_weights * (1-null_weight),null_weight)
    X = cbind(X,0)
  }
  if (anyNA(X))
    stop("Input X must not contain missing values")
  if (anyNA(y)) {
    if (na.rm) {
      samples_kept = which(!is.na(y))
      y = y[samples_kept]
      X = X[samples_kept,]
    } else
      stop("Input y must not contain missing values")
  }
  p = ncol(X)
  if (p > 1000 & !requireNamespace("Rfast",quietly = TRUE))
    warning_message("For an X with many columns, please consider installing",
                    "the Rfast package for more efficient credible set (CS)",
                    "calculations.", style='hint')

  # Check input y.
  n = nrow(X)
  mean_y = mean(y)

  # Center and scale input.
  if (intercept)
    y = y - mean_y

  # Set three attributes for matrix X: attr(X,'scaled:center') is a
  # p-vector of column means of X if center=TRUE, a p vector of zeros
  # otherwise; 'attr(X,'scaled:scale') is a p-vector of column
  # standard deviations of X if scale=TRUE, a p vector of ones
  # otherwise; 'attr(X,'d') is a p-vector of column sums of
  # X.standardized^2,' where X.standardized is the matrix X centered
  # by attr(X,'scaled:center') and scaled by attr(X,'scaled:scale').
  out = compute_colstats(X,center = intercept,scale = standardize)
  attr(X,"scaled:center") = out$cm
  attr(X,"scaled:scale") = out$csd
  attr(X,"d") = out$d

  # Initialize susie fit.
  s = init_setup(n,p,L,scaled_prior_variance,residual_variance,prior_weights,
                 null_weight,as.numeric(var(y)),standardize)
  if (!missing(s_init) && !is.null(s_init)) {
    if (!inherits(s_init,"susie"))
      stop("s_init should be a susie object")
    if (max(s_init$alpha) > 1 || min(s_init$alpha) < 0)
      stop("s_init$alpha has invalid values outside range [0,1]; please ",
           "check your input")

    # First, remove effects with s_init$V = 0
    s_init = susie_prune_single_effects(s_init)
    num_effects = nrow(s_init$alpha)
    if(missing(L)){
      L = num_effects
    }else if(min(p,L) < num_effects){
      warning_message(paste("Specified number of effects L =",min(p,L),
                    "is smaller than the number of effects",num_effects,
                    "in input SuSiE model. The SuSiE model will have",
                    num_effects,"effects."))
      L = num_effects
    }
    # expand s_init if L > num_effects.
    s_init = susie_prune_single_effects(s_init, min(p, L), s$V)
    s = modifyList(s,s_init)
    s = init_finalize(s,X = X)
  } else {
    s = init_finalize(s)
  }

  # Initialize elbo to NA.
  elbo = rep(as.numeric(NA),max_iter + 1)
  elbo[1] = -Inf;
  tracking = list()

  for (i in 1:max_iter) {
    if (track_fit)
      tracking[[i]] = susie_slim(s)
    s = update_each_effect(X,y,s,estimate_prior_variance,estimate_prior_method,
                           check_null_threshold)
    if (verbose)
      print(paste0("objective:",get_objective(X,y,s)))

    # Compute objective before updating residual variance because part
    # of the objective s$kl has already been computed under the
    # residual variance before the update.
    elbo[i+1] = get_objective(X,y,s)
    if ((elbo[i+1] - elbo[i]) < tol) {
      s$converged = TRUE
      break
    }
    if (estimate_residual_variance) {
      s$sigma2 = pmax(residual_variance_lowerbound,
                      estimate_residual_variance(X,y,s))
      if (s$sigma2 > residual_variance_upperbound)
        s$sigma2 = residual_variance_upperbound
      if (verbose)
        print(paste0("objective:",get_objective(X,y,s)))
    }
  }

  # Remove first (infinite) entry, and trailing NAs.
  elbo = elbo[2:(i+1)]
  s$elbo = elbo
  s$niter = i

  if (is.null(s$converged)) {
    warning(paste("IBSS algorithm did not converge in",max_iter,"iterations!"))
    s$converged = FALSE
  }

  if (intercept) {

    # Estimate unshrunk intercept.
    s$intercept = mean_y - sum(attr(X,"scaled:center") *
      (colSums(s$alpha * s$mu)/attr(X,"scaled:scale")))
    s$fitted = s$Xr + mean_y
  } else {
    s$intercept = 0
    s$fitted = s$Xr
  }
  s$fitted = drop(s$fitted)
  names(s$fitted) = `if`(is.null(names(y)),rownames(X),names(y))

  if (track_fit)
    s$trace = tracking

  # SuSiE CS and PIP.
  if (!is.null(coverage) && !is.null(min_abs_corr)) {
    s$sets = susie_get_cs(s,coverage = coverage,X = X,
                          min_abs_corr = min_abs_corr,
                          n_purity = n_purity)
    s$pip = susie_get_pip(s,prune_by_cs = FALSE,prior_tol = prior_tol)
  }

  if (!is.null(colnames(X))) {
    variable_names = colnames(X)
    if (!is.null(null_weight)) {
      variable_names[length(variable_names)] = "null"
      names(s$pip) = variable_names[-p]
    } else
      names(s$pip)    = variable_names
    colnames(s$alpha) = variable_names
    colnames(s$mu)    = variable_names
    colnames(s$mu2)   = variable_names
    colnames(s$lbf_variable) = variable_names
  }
  # report z-scores from univariate regression.
  if (compute_univariate_zscore) {
    if (!is.matrix(X))
      warning("Calculation of univariate regression z-scores is not ",
              "implemented specifically for sparse or trend filtering ",
              "matrices, so this step may be slow if the matrix is large; ",
              "to skip this step set compute_univariate_zscore = FALSE")
    if (!is.null(null_weight) && null_weight != 0)
      X = X[,1:(ncol(X) - 1)]
    s$z = calc_z(X,y,center = intercept,scale = standardize)
  }

  # For prediction.
  s$X_column_scale_factors = attr(X,"scaled:scale")

  if (refine) {
    if (!missing(s_init) && !is.null(s_init))
      warning("The given s_init is not used in refinement")
    if (!is.null(null_weight) && null_weight != 0) {

      ## if null_weight is specified, we compute the original prior_weight
      pw_s = s$pi[-s$null_index]/(1-null_weight)
      if (!compute_univariate_zscore)

        ## if null_weight is specified, and the extra 0 column is not
        ## removed from compute_univariate_zscore, we remove it here
        X = X[,1:(ncol(X) - 1)]
    } else
      pw_s = s$pi
    conti = TRUE
    while (conti & length(s$sets$cs)>0) {
      m = list()
      for(cs in 1:length(s$sets$cs)){
        pw_cs = pw_s
        pw_cs[s$sets$cs[[cs]]] = 0
        if(all(pw_cs == 0)){
          break
        }
        s2 = susie(X,y,L = L,scaled_prior_variance = scaled_prior_variance,
            residual_variance = residual_variance,
            prior_weights = pw_cs, s_init = NULL,null_weight = null_weight,
            standardize = standardize,intercept = intercept,
            estimate_residual_variance = estimate_residual_variance,
            estimate_prior_variance = estimate_prior_variance,
            estimate_prior_method = estimate_prior_method,
            check_null_threshold = check_null_threshold,
            prior_tol = prior_tol,coverage = coverage,
            residual_variance_upperbound = residual_variance_upperbound,
            min_abs_corr = min_abs_corr,compute_univariate_zscore = FALSE,
            na.rm = na.rm,max_iter = max_iter,tol = tol,verbose = FALSE,
            track_fit = FALSE,residual_variance_lowerbound = var(drop(y))/1e4,
            refine = FALSE)
        sinit2 = s2[c("alpha","mu","mu2")]
        class(sinit2) = "susie"
        s3 = susie(X,y,L = L,scaled_prior_variance = scaled_prior_variance,
            residual_variance = residual_variance,prior_weights = pw_s,
            s_init = sinit2,null_weight = null_weight,
            standardize = standardize,intercept = intercept,
            estimate_residual_variance = estimate_residual_variance,
            estimate_prior_variance = estimate_prior_variance,
            estimate_prior_method = estimate_prior_method,
            check_null_threshold = check_null_threshold,
            prior_tol = prior_tol,coverage = coverage,
            residual_variance_upperbound = residual_variance_upperbound,
            min_abs_corr = min_abs_corr,compute_univariate_zscore = FALSE,
            na.rm = na.rm,max_iter = max_iter,tol = tol,verbose = FALSE,
            track_fit = FALSE,residual_variance_lowerbound = var(drop(y))/1e4,
            refine = FALSE)
        m = c(m,list(s3))
      }
      if(length(m) == 0){
        conti = FALSE
      }else{
        elbo = sapply(m,function(x) susie_get_objective(x))
        if ((max(elbo) - susie_get_objective(s)) <= 0)
          conti = FALSE
        else
          s = m[[which.max(elbo)]]
      }
    }
  }
  return(s)
}

context("test_intercept_standardize.R")

test_that("scaling and intercept works as expected", with(simulate(200,1000), {
  s1 = susie(X,y,intercept= TRUE, standardize=TRUE)
  s2 = susie(X,y,intercept = FALSE, standardize = FALSE)
  s3 = susie(X,y,intercept =TRUE, standardize = FALSE)
  s4 = susie(X,y,intercept = FALSE,standardize = TRUE)
  
  expect_equal(predict(s2),predict(s2,X))
  expect_equal(predict(s4),predict(s4,X))
  expect_equal(predict(s1),predict(s1,X))
  expect_equal(predict(s3),predict(s3,X))
  
  expect_equal(s2$intercept, 0)
  expect_equal(s4$intercept, 0)
}))

context("test_susie_get_cs.R")

test_that("susie_get_cs purity calculations are correct",{

  # Simulate a data set with correlated variables.
  dat <- simulate(40,100)
  X   <- dat$X
  y   <- dat$y
  X   <- cbind(X,
               X + matrix(rnorm(4000),40,100)/20,
               X + matrix(rnorm(4000),40,100)/20)

  # Fit a susie model.
  fit <- susie(X,y,estimate_prior_variance = FALSE)

  # The purity calculations should be the same whether or not the
  # Rfast package functions are used, and all the purity statistics
  # should be positive.
  set.seed(1)
  purity1 <- susie_get_cs(fit,X,min_abs_corr = 0,use_rfast = FALSE)$purity
  expect_gt(min(purity1),0)
  skip_if_not_installed("Rfast")
  set.seed(1)
  purity2 <- susie_get_cs(fit,X,min_abs_corr = 0,use_rfast = TRUE)$purity
  expect_equal(purity1,purity2,scale = 1,tolerance = 1e-15)
})

context("test_susie_trendfilter.R")

#check if the susie_trendfilter produces the same results as the original version
test_that("susie for trend filtering",{
  #order 0 trend filtering
  with(simulate_tf(0), {
    original.s0 = susie(X,y,estimate_prior_variance = FALSE)
    s0 = susie_trendfilter(y,0,estimate_prior_variance = FALSE, standardize = TRUE, use_mad=FALSE)
    original.s0.ns.ni = susie(X,y,standardize=FALSE,intercept=FALSE,estimate_prior_variance = FALSE)
    s0.ns.ni = susie_trendfilter(y,0,standardize=FALSE,intercept=FALSE,estimate_prior_variance = FALSE, use_mad=FALSE)
    original.s0.ns = susie(X,y,standardize=FALSE,intercept=TRUE,estimate_prior_variance = FALSE)
    s0.ns = susie_trendfilter(y,0,standardize=FALSE,intercept=TRUE,estimate_prior_variance = FALSE, use_mad=FALSE)
    original.s0.ni = susie(X,y,standardize=TRUE,intercept=FALSE,estimate_prior_variance = FALSE)
    s0.ni = susie_trendfilter(y,0,standardize=TRUE,intercept=FALSE,estimate_prior_variance = FALSE, use_mad=FALSE)

    expect_equal_susie(s0, original.s0)
    expect_equal_susie(s0.ns.ni, original.s0.ns.ni)
    expect_equal_susie(s0.ns, original.s0.ns)
    expect_equal_susie(s0.ni, original.s0.ni)
  })
  #order 1 trend filtering
  with(simulate_tf(1), {
    suppressWarnings({
      original.s1 <- susie(X,y,estimate_prior_variance = FALSE)
      s1 <- suppressWarnings(susie_trendfilter(y,1,estimate_prior_variance = FALSE, standardize = TRUE, use_mad=FALSE))
      original.s1.ns.ni <- susie(X,y,standardize=FALSE,intercept=FALSE,estimate_prior_variance = FALSE)
      s1.ns.ni <- suppressWarnings(susie_trendfilter(y,1,standardize=FALSE,intercept=FALSE,estimate_prior_variance = FALSE, use_mad=FALSE))
      original.s1.ns <- susie(X,y,standardize=FALSE,intercept=TRUE,estimate_prior_variance = FALSE)
      s1.ns <- suppressWarnings(susie_trendfilter(y,1,standardize=FALSE,intercept=TRUE,estimate_prior_variance = FALSE, use_mad=FALSE))
      original.s1.ni <- susie(X,y,standardize=TRUE,intercept=FALSE,estimate_prior_variance = FALSE)
      s1.ni <- suppressWarnings(susie_trendfilter(y,1,standardize=TRUE,intercept=FALSE,estimate_prior_variance = FALSE, use_mad=FALSE))
    })

    expect_equal_susie(s1, original.s1)
    expect_equal_susie(s1.ns.ni, original.s1.ns.ni)
    expect_equal_susie(s1.ns, original.s1.ns)
    expect_equal_susie(s1.ni, original.s1.ni)
  })
  #order 2 trend filtering
  with(simulate_tf(2), {
    original.s2 = susie(X,y,estimate_prior_variance = FALSE)
    s2 = suppressWarnings(susie_trendfilter(y,2,estimate_prior_variance = FALSE, standardize = TRUE, use_mad=FALSE))
    original.s2.ns.ni = susie(X,y,standardize=FALSE,intercept=FALSE,estimate_prior_variance = FALSE)
    s2.ns.ni = suppressWarnings(susie_trendfilter(y,2,standardize=FALSE,intercept=FALSE,estimate_prior_variance = FALSE, use_mad=FALSE))
    original.s2.ns = susie(X,y,standardize=FALSE,intercept=TRUE,estimate_prior_variance = FALSE)
    s2.ns = suppressWarnings(susie_trendfilter(y,2,standardize=FALSE,intercept=TRUE,estimate_prior_variance = FALSE, use_mad=FALSE))
    original.s2.ni = susie(X,y,standardize=TRUE,intercept=FALSE,estimate_prior_variance = FALSE)
    s2.ni = suppressWarnings(susie_trendfilter(y,2,standardize=TRUE,intercept=FALSE,estimate_prior_variance = FALSE, use_mad=FALSE))

    expect_equal_susie(s2, original.s2)
    expect_equal_susie(s2.ns.ni, original.s2.ns.ni)
    expect_equal_susie(s2.ns, original.s2.ns)
    expect_equal_susie(s2.ni, original.s2.ni)
  })
})

#check if susie_trendfilter can fit different arguments
test_that("susie_trendfilter interface",{
  with(simulate_tf(0), {
    n = length(y)
    expect_error(susie_trendfilter(y,0,standardize = TRUE, estimate_prior_variance=TRUE, use_mad=FALSE), NA)
    expect_error(susie_trendfilter(y,0,standardize = TRUE, null_weight=1/(n+1),estimate_prior_variance = FALSE, use_mad=FALSE), NA)
    expect_error(susie_trendfilter(y,0,standardize = TRUE, estimate_prior_variance=TRUE, null_weight=1/(n+1), use_mad=FALSE), NA)
    expect_error(susie_trendfilter(y,0,standardize=FALSE,intercept=FALSE,estimate_prior_variance = FALSE, use_mad=FALSE), NA)
    expect_error(susie_trendfilter(y,0,standardize=FALSE,intercept=TRUE,estimate_prior_variance = FALSE, use_mad=FALSE), NA)
    expect_error(susie_trendfilter(y,0,standardize=TRUE,intercept=FALSE,estimate_prior_variance = FALSE, use_mad=FALSE), NA)
    expect_error(suppressWarnings(susie_trendfilter(y,0,standardize = TRUE, compute_univariate_zscore =TRUE,estimate_prior_variance = FALSE, use_mad=FALSE)), NA)
  })
})

context("test_susie.R")

# test_that("susie agrees with version 0.3", with(simulate(sparse=T), {
#   original.res  = readRDS('susiefit_original_res.rds')
#   original.res2 = readRDS('susiefit_original_res2.rds')
#   original.res3 = readRDS('susiefit_original_res3.rds')
#   original.res4 = readRDS('susiefit_original_res4.rds')
#
#   original.res$Xr = as.vector(original.res$Xr)
#   original.res2$Xr = as.vector(original.res2$Xr)
#   original.res3$Xr = as.vector(original.res3$Xr)
#   original.res4$Xr = as.vector(original.res4$Xr)
#   dense.res = susie(X, y, tol=1E-2, estimate_prior_variance = FALSE)
#   sparse.res = susie(X.sparse, y, tol=1E-2, estimate_prior_variance = FALSE)
#
#   dense.res2 = susie(X, y, standardize=TRUE, intercept = FALSE, tol=1E-2, estimate_prior_variance = FALSE)
#   sparse.res2 = susie(X.sparse, y, standardize=TRUE, intercept = FALSE,
#                       tol=1E-2, estimate_prior_variance = FALSE)
#
#   dense.res3 = susie(X, y, standardize=FALSE, intercept = TRUE, tol=1E-2, estimate_prior_variance = FALSE)
#   sparse.res3 = susie(X.sparse, y, standardize=FALSE, intercept = TRUE,
#                       tol=1E-2, estimate_prior_variance = FALSE)
#
#   dense.res4 = susie(X, y, standardize=FALSE, intercept = FALSE, tol=1E-2, estimate_prior_variance = FALSE)
#   sparse.res4 = susie(X.sparse, y, standardize=FALSE, intercept = FALSE,
#                       tol=1E-2, estimate_prior_variance = FALSE)
#   expect_equal_susie(sparse.res, original.res)
#   expect_equal_susie(dense.res, original.res)
#   expect_equal_susie(sparse.res2, original.res2)
#   expect_equal_susie(dense.res2, original.res2)
#   expect_equal_susie(sparse.res3, original.res3)
#   expect_equal_susie(dense.res3, original.res3)
#   expect_equal_susie(sparse.res4, original.res4)
#   expect_equal_susie(dense.res4, original.res4)
# }))

# context("test_small_resid.R")

# This test comes from Issue #7.
# test_that("susie works when residual variance is small",{
#   dat <- readRDS("full_data_1_sim_gaussian_null_1.rds")$data
#   fit <- susie(dat$X,dat$Y,L = 10,null_weight = 0)
#   expect_equal(fit$sigma2,1e-4)
#   expect_true(all(is.finite(fit$elbo)))
# })

context("test_init.R")

test_that("susie init", with(simulate(sparse=T), {
  original.res  = readRDS('susiefit_original_res.rds')
  original.res$lbf_variable = matrix(0, nrow(original.res$alpha), ncol(original.res$alpha))
  expect_message(capture_output(susie(X, y, L=2, tol=1E-2, s_init = original.res, estimate_prior_variance = FALSE,verbose=T)))
  expect_message(capture_output(susie(X, y, L=20, tol=1E-2, s_init = original.res, estimate_prior_variance = FALSE, verbose=T)))
}))

# Performs sum of single-effect (SuSiE) linear regression with z
# scores (with lambda). The summary data required are the p by p
# correlation matrix R, the p vector z. The summary stats should come
# from the same individuals. This function fits the regression model z
# = sum_l Rb_l + e, where e is N(0,residual_variance * R + lambda I)
# and the sum_l b_l is a p vector of effects to be estimated. The
# assumption is that each b_l has exactly one non-zero element, with
# all elements equally likely to be non-zero. The prior on the
# non-zero element is N(0,var = prior_variance).
#
#' @importFrom stats optimize
susie_rss_lambda = function(z, R, maf = NULL, maf_thresh = 0,
                            L = 10, lambda = 0,
                            prior_variance = 50, residual_variance = NULL,
                            r_tol = 1e-08, prior_weights = NULL,
                            null_weight = 0,
                            estimate_residual_variance = TRUE,
                            estimate_prior_variance = TRUE,
                            estimate_prior_method = c("optim", "EM", "simple"),
                            check_null_threshold = 0, prior_tol = 1e-9,
                            max_iter = 100, s_init = NULL, intercept_value = 0,
                            coverage = 0.95, min_abs_corr = 0.5,
                            tol = 1e-3, verbose = FALSE, track_fit = FALSE,
                            check_R = TRUE, check_z = FALSE) {

  # Check input R.
  if (nrow(R) != length(z))
    stop(paste0("The dimension of correlation matrix (",nrow(R)," by ",
                ncol(R),") does not agree with expected (",length(z)," by ",
                length(z),")"))
  if (!is_symmetric_matrix(R))
    stop("R is not a symmetric matrix")
  if (!(is.double(R) & is.matrix(R)) & !inherits(R,"CsparseMatrix"))
    stop("Input R must be a double-precision matrix or a sparse matrix")
  
  # MAF filter.
  if (!is.null(maf)) {
    if (length(maf) != length(z))
      stop(paste0("The length of maf does not agree with expected ",length(z)))
    id = which(maf > maf_thresh)
    R = R[id,id]
    z = z[id]
  }

  if (any(is.infinite(z)))
    stop("z contains infinite values")

  # Check for NAs in R.
  if (anyNA(R))
    stop("R matrix contains missing values")

  # Replace NAs in z with zero.
  if (anyNA(z)) {
    warning_message("NA values in z-scores are replaced with 0")
    z[is.na(z)] = 0
  }

  if (is.numeric(null_weight) && null_weight == 0)
    null_weight = NULL
  if (!is.null(null_weight)) {
    if (!is.numeric(null_weight))
      stop("Null weight must be numeric")
    if (null_weight < 0 || null_weight >= 1)
      stop("Null weight must be between 0 and 1")
    if (missing(prior_weights))
      prior_weights = c(rep(1/ncol(R)*(1-null_weight),ncol(R)),null_weight)
    else
      prior_weights = c(prior_weights * (1-null_weight),null_weight)
    R = cbind(rbind(R,0),0)
    z = c(z,0)
  }

  # Eigen decomposition for R, filter on eigenvalues.
  p = ncol(R)
  attr(R,"eigen") = eigen(R,symmetric = TRUE)
  if (check_R && any(attr(R,"eigen")$values < -r_tol))
    stop(paste0("The correlation matrix (",nrow(R)," by ",ncol(R),
                "is not a positive semidefinite matrix. ",
                "The smallest eigenvalue is ",min(attr(R,"eigen")$values),
                ". You can bypass this by \"check_R = FALSE\" which instead ",
                "sets negative eigenvalues to 0 to allow for continued ",
                "computations."))

  # Check whether z in space spanned by the non-zero eigenvectors of R.
  if (check_z) {
    proj = check_projection(R,z)
    if (!proj$status)
      warning_message("Input z does not lie in the space of non-zero eigenvectors ",
              "of R.")
    else
      message("Input z is in space spanned by the non-zero eigenvectors of ",
              "R.")
  }
  R = set_R_attributes(R,r_tol)
  
  if (lambda == "estimate"){
    colspace = which(attr(R,"eigen")$values > 0)
    if (length(colspace) == length(z))
      lambda = 0
    else {
      znull = crossprod(attr(R,"eigen")$vectors[,-colspace], z) # U2^T z
      lambda = sum(znull^2)/length(znull)
    }
  }

  # Initialize susie fit.
  s = init_setup_rss(p,L,prior_variance,residual_variance,prior_weights,
                     null_weight)
  if (!missing(s_init) && !is.null(s_init)) {
    if (!inherits(s_init,"susie"))
      stop("s_init should be a susie object")
    if (max(s_init$alpha) > 1 || min(s_init$alpha) < 0)
      stop("s_init$alpha has invalid values outside range [0,1]; please ",
           "check your input")
    # First, remove effects with s_init$V = 0
    s_init = susie_prune_single_effects(s_init)
    num_effects = nrow(s_init$alpha)
    if(missing(L)){
      L = num_effects
    }else if(L < num_effects){
      warning_message(paste("Specified number of effects L =",L,
                    "is smaller than the number of effects",num_effects,
                    "in input SuSiE model. It will have",num_effects,"effects."))
      L = num_effects
    }
    # expand s_init if L > num_effects.
    s_init = susie_prune_single_effects(s_init, L, s$V)
    s = modifyList(s,s_init)
    s = init_finalize_rss(s,R = R)
  } else
    s = init_finalize_rss(s)

  s$sigma2 = s$sigma2 - lambda
  estimate_prior_method = match.arg(estimate_prior_method)

  # Intialize elbo to NA.
  elbo = rep(NA,max_iter+1)
  elbo[1] = -Inf;
  tracking = list()

  attr(R,"lambda") = lambda
  Sigma = update_Sigma(R,s$sigma2,z)  # sigma2*R + lambda*I

  for (i in 1:max_iter) {
    if (track_fit)
      tracking[[i]] = susie_slim(s)
    s = update_each_effect_rss(R,z,s,Sigma,estimate_prior_variance,
                               estimate_prior_method,check_null_threshold)
    if (verbose)
      print(paste0("before estimate sigma2 objective:",
                   get_objective_rss(R,z,s)))

    # Compute objective before updating residual variance because part
    # of the objective s$kl has already been computed under the
    # residual variance, before the update.
    elbo[i+1] = get_objective_rss(R,z,s)
    if ((elbo[i+1] - elbo[i]) < tol) {
      s$converged = TRUE
      break
    }
    if (estimate_residual_variance) {
      if (lambda == 0) {
        est_sigma2 = (1/sum(attr(R,"eigen")$values != 0))*get_ER2_rss(1,R,z,s)
        if (est_sigma2 < 0)
          stop("Estimating residual variance failed: the estimated value ",
               "is negative")
        if (est_sigma2 > 1)
          est_sigma2 = 1
      } else {
        est_sigma2 = optimize(Eloglik_rss, interval = c(1e-4, 1-lambda),
                              R = R, z = z, s = s, maximum = TRUE)$maximum
        if(Eloglik_rss(est_sigma2, R, z, s) < Eloglik_rss(1-lambda, R, z, s)){
          est_sigma2 = 1-lambda
        }
      }
      s$sigma2 = est_sigma2

      if (verbose)
        print(paste0("after estimate sigma2 objective:",
                     get_objective_rss(R,z,s)))
      Sigma = update_Sigma(R,s$sigma2,z)
    }
  }

  # Remove first (infinite) entry, and trailing NAs.
  elbo = elbo[2:(i+1)]
  s$elbo = elbo
  s$niter = i
  s$lambda = lambda

  if (is.null(s$converged)) {
    warning(paste("IBSS algorithm did not converge in",max_iter,"iterations!"))
    s$converged = FALSE
  }

  s$intercept = intercept_value
  s$fitted = s$Rz

  s$X_column_scale_factors = attr(R,"scaled:scale")

  if (track_fit)
    s$trace = tracking

  # SuSiE CS and PIP.
  if (!is.null(coverage) && !is.null(min_abs_corr)) {
    R = muffled_cov2cor(R)
    s$sets = susie_get_cs(s,coverage = coverage,Xcorr = R,
                          min_abs_corr = min_abs_corr)
    s$pip = susie_get_pip(s,prune_by_cs = FALSE,prior_tol = prior_tol)
  }
  if (!is.null(names(z))) {
    variable_names = names(z)
    if (!is.null(null_weight))
      variable_names = c("null",variable_names)
    colnames(s$alpha) = variable_names
    colnames(s$mu) = variable_names
    colnames(s$mu2) = variable_names
    colnames(s$lbf_variable) = variable_names
    names(s$pip) = variable_names
  }

  return(s)
}

update_Sigma = function (R, sigma2, z) {
  Sigma = sigma2*R + attr(R,"lambda") * diag(length(z))
  eigenS = attr(R,"eigen")
  eigenS$values = sigma2 * eigenS$values + attr(R,"lambda")

  Dinv = 1/(eigenS$values)
  Dinv[is.infinite(Dinv)] = 0
  attr(Sigma,"eigenS") = eigenS

  # Sigma^(-1) R_j = U (sigma2 D + lambda)^(-1) D U^T e_j
  attr(Sigma,"SinvRj") = eigenS$vectors %*% (Dinv * attr(R,"eigen")$values *
                           t(eigenS$vectors))

  if (attr(R,"lambda") == 0)
    attr(Sigma,"RjSinvRj") = attr(R,"d")/sigma2
  else {
    tmp = t(eigenS$vectors)
    attr(Sigma,"RjSinvRj") =
      colSums(tmp * (Dinv*(attr(R,"eigen")$values^2) * tmp))
  }

  return(Sigma)
}

context("test_update_each_effect_rss.R")

# test_that("update_each_effect_rss (lambda = 0) agrees with previous version ", with(simulate(200, 500), {
#   original.res = readRDS("vbupdate_rss_lambda0_res.rds")$s
#
#   ss = univariate_regression(X, y)
#   R = cor(X)
#   z = ss$betahat/ss$sebetahat
#   s$Rz = rep(5, nrow(R))
#
#   R = set_R_attributes(R, 1e-08)
#   attr(R, "lambda") = 0
#   Sigma = update_Sigma(R, s$sigma2, z)
#
#   res = update_each_effect_rss(R, z, s, Sigma)
#   expect_equal_susie_rss_update(res, original.res, tolerance = 1e-4)
# }))

# test_that("update_each_effect_rss (lambda = 1) agrees with previous version ", with(simulate(200, 500), {
#   original.res = readRDS("vbupdate_rss_lambda1_res.rds")$s
#
#   ss = univariate_regression(X, y)
#   R = cor(X)
#   z = ss$betahat/ss$sebetahat
#   s$Rz = rep(5, nrow(R))
#
#   R = set_R_attributes(R, 1e-08)
#   attr(R, "lambda") = 1
#   Sigma = update_Sigma(R, s$sigma2, z)
#
#   res = update_each_effect_rss(R, z, s, Sigma)
#   expect_equal_susie_rss_update(res, original.res, tolerance = 1e-4)
# }))

context("test_single_effect_regression_rss.R")

# test_that("single_effect_regression_rss (lambda = 0) agrees with previous version", with(simulate(200, 500), {
#   original.res = readRDS('singleReg_rss_lambda0_res.rds')$s
#   # the git commit id to previous result is readRDS('singleReg_rss_res.rds')$git_commit_id
#
#   ss = univariate_regression(X, y)
#   R = cor(X)
#   z = ss$betahat/ss$sebetahat
#   R = set_R_attributes(R, 1e-08)
#   attr(R, 'lambda') = 0
#   Sigma = update_Sigma(R, s$sigma2, z)
#
#   s = single_effect_regression_rss(z, Sigma, optimize_V = "none")
#
#   expect_equal(attr(R, 'd'), rep(1, 500), tolerance=1e-4)
#   expect_equal_SER_suff_stat(s, original.res, tolerance=1e-4)
# }))

# test_that("single_effect_regression_rss (lambda = 1) agrees with previous version", with(simulate(200, 500), {
#   original.res = readRDS('singleReg_rss_lambda1_res.rds')$s
#   # the git commit id to previous result is readRDS('singleReg_rss_res.rds')$git_commit_id
#
#   ss = univariate_regression(X, y)
#   R = cor(X)
#   z = ss$betahat/ss$sebetahat
#   R = set_R_attributes(R, 1e-08)
#   attr(R, 'lambda') = 1
#   Sigma = update_Sigma(R, s$sigma2, z)
#
#   s = single_effect_regression_rss(z, Sigma, optimize_V = "none")
#
#   expect_equal(attr(R, 'd'), rep(1, 500), tolerance=1e-4)
#   expect_equal_SER_suff_stat(s, original.res, tolerance=1e-4)
# }))

context("test_SER_posterior_e_loglik_rss.R")

test_that("SER_posterior_e_loglik_rss (lambda = 0) agrees with previous version", with(simulate(200, 500), {
  original.res = readRDS('SER_rss_lambda0_res.rds')$s
  Ez = rep(1, p)
  Ez2 = rep(1, p)
  s2 = s$sigma2

  ss = univariate_regression(X, y)
  R = cor(X)
  z = ss$betahat/ss$sebetahat
  R = set_R_attributes(R, 1e-08)
  attr(R, 'lambda') = 0
  Sigma = update_Sigma(R, s2, z)

  res = SER_posterior_e_loglik_rss(R, Sigma, z,Ez,Ez2)

  expect_equal(res, original.res, tolerance=1e-4)
}))

test_that("SER_posterior_e_loglik_rss (lambda = 1) agrees with previous version", with(simulate(200, 500), {
  original.res = readRDS('SER_rss_lambda1_res.rds')$s
  Ez = rep(1, p)
  Ez2 = rep(1, p)
  s2 = s$sigma2

  ss = univariate_regression(X, y)
  R = cor(X)
  z = ss$betahat/ss$sebetahat
  R = set_R_attributes(R, 1e-08)
  attr(R, 'lambda') = 1
  Sigma = update_Sigma(R, s2, z)

  res = SER_posterior_e_loglik_rss(R, Sigma, z,Ez,Ez2)

  expect_equal(res, original.res, tolerance=1e-4)
}))

# @title Get objective function from data and susie fit object.
# @param R p by p corelation matrix
# @param z length p vector
# @param s a susie fit object
get_objective_rss = function (R, z, s) 
  Eloglik_rss(s$sigma2, R,z,s) - sum(s$KL)

# @title expected loglikelihood for a susie fit
Eloglik_rss = function (sigma2, R, z, s) {
  d = sigma2 * attr(R,"eigen")$values + attr(R,"lambda")
  if(attr(R,"lambda") == 0)
    result = -(sum(d != 0)/2) * log(2*pi*sigma2) - 0.5*get_ER2_rss(sigma2, R,z,s)
  else
    result = -(length(z)/2)*log(2*pi) - 0.5*sum(log(d)) - 0.5*get_ER2_rss(sigma2, R,z,s)
  return(result)
}

# @title expected squared residuals
# @importFrom Matrix diag
get_ER2_rss = function (sigma2, R, z, s) {
  d = sigma2 * attr(R,"eigen")$values + attr(R,"lambda")
  Dinv = 1/d
  Dinv[is.infinite(Dinv)] = 0
  SinvR = attr(R,"eigen")$vectors %*%
          ((Dinv*attr(R,"eigen")$values) * t(attr(R,"eigen")$vectors))
  Utz = crossprod(attr(R,"eigen")$vectors,z)
  zSinvz = sum(Utz * (Dinv * Utz))

  Z = s$alpha * s$mu
  if(attr(R,"lambda") == 0)
    RSinvR = R/sigma2
  else
    RSinvR = R %*% SinvR
  RZ2 = sum((Z%*%RSinvR) * Z)

  zbar = colSums(Z)
  postb2 = s$alpha * s$mu2 # Posterior second moment.
  return(zSinvz - 2*sum((SinvR %*% z) * zbar)
         + sum(zbar * (RSinvR %*% zbar))
         - RZ2 + sum(diag(RSinvR) * t(postb2)))
}

# @title posterior expected loglikelihood for a single effect regression
# @param R a p by p symmetric and positive semidefinite correlation matrix.
# @param Sigma residual_var * R + lambda I
# @param r residuals
# @param Eb the posterior mean of b (p vector) (alpha * mu)
# @param Eb2 the posterior second moment of b (p vector) (alpha * mu2)
SER_posterior_e_loglik_rss = function (R, Sigma, r, Ez, Ez2) {
  eigenS = attr(Sigma,"eigenS")
  Dinv = 1/(eigenS$values)
  Dinv[is.infinite(Dinv)] = 0
  rR = R %*% r
  SinvEz = eigenS$vectors %*% (Dinv * crossprod(eigenS$vectors, Ez))
  return(-0.5*(-2*sum(rR*SinvEz) +
               sum(attr(Sigma,"RjSinvRj") * as.vector(Ez2))))
}

# @title update each effect once
# @param XtX a p by p matrix, X'X
# @param Xty a p vector
# @param s_init a list with elements sigma2, V, alpha, mu, Xr
# @param estimate_prior_variance boolean indicating whether to
#   estimate prior variance
# @param estimate_prior_method The method used for estimating prior
#   variance, 'optim' or 'EM'.
# @param check_null_threshold float a threshold on the log scale to
#   compare likelihood between current estimate and zero the null
# 
#' @importFrom Matrix diag
update_each_effect_ss = function (XtX, Xty, s_init,
                                  estimate_prior_variance = FALSE,
                                  estimate_prior_method = "optim",
                                  check_null_threshold = 0) {
  if (!estimate_prior_variance)
    estimate_prior_method = "none"
  
  # Repeat for each effect to update.
  s = s_init
  L = nrow(s$alpha)
  if (L > 0) {
    for (l in 1:L) {
        
      # Remove lth effect from fitted values.
      s$XtXr = s$XtXr - XtX %*% (s$alpha[l,] * s$mu[l,])

      # Compute residuals.
      XtR = Xty - as.vector(s$XtXr)
      res = single_effect_regression_ss(as.matrix(XtR),attr(XtX,"d"),s$V[l],
              s$sigma2,s$pi,estimate_prior_method,check_null_threshold)
      
      # Update the variational estimate of the posterior mean.
      s$mu[l,]    = res$mu
      s$alpha[l,] = res$alpha
      s$mu2[l,]   = res$mu2
      s$V[l]      = res$V
      s$lbf[l]    = res$lbf_model
      s$lbf_variable[l,] = res$lbf
      s$KL[l]     = -res$lbf_model +
        SER_posterior_e_loglik_ss(attr(XtX,"d"),XtR,s$sigma2,
                                  res$alpha * res$mu,res$alpha * res$mu2)

      s$XtXr = s$XtXr + XtX %*% (s$alpha[l,] * s$mu[l,])
    }
  }
  s$XtXr = unname(as.matrix(s$XtXr))
  return(s)
}

#' @rdname single_effect_regression
#' 
#' @param Xty A p-vector.
#' 
#' @param dXtX A p-vector containing the diagonal elements of
#'   \code{crossprod(X)}.
#' 
#' @importFrom stats uniroot
#' @importFrom stats optim
#'
#' @keywords internal
#' 
single_effect_regression_ss =
  function (Xty, dXtX, V = 1, residual_variance = 1, prior_weights = NULL,
            optimize_V = c("none", "optim", "uniroot", "EM", "simple"),
            check_null_threshold = 0) {
  optimize_V = match.arg(optimize_V)
  betahat = (1/dXtX) * Xty
  shat2 = residual_variance/dXtX
  if (is.null(prior_weights))
    prior_weights = rep(1/length(dXtX),length(dXtX))

  if (optimize_V != "EM" && optimize_V != "none")
    V = optimize_prior_variance(optimize_V,betahat,shat2,prior_weights,
                                alpha = NULL,post_mean2 = NULL,V_init = V,
                                check_null_threshold = check_null_threshold)

  # log(po) = log(BF * prior) for each SNP
  lbf = dnorm(betahat,0,sqrt(V + shat2),log = TRUE) -
        dnorm(betahat,0,sqrt(shat2),log = TRUE)
  lpo = lbf + log(prior_weights + sqrt(.Machine$double.eps))

  # Deal with special case of infinite shat2 (e.g., happens if X does
  # not vary).
  lbf[is.infinite(shat2)] = 0 
  lpo[is.infinite(shat2)] = 0
  maxlpo = max(lpo)

  # w is proportional to
  #
  #   posterior odds = BF * prior,
  #
  # but subtract max for numerical stability.
  w_weighted = exp(lpo - maxlpo)
  weighted_sum_w = sum(w_weighted)
  alpha = w_weighted / weighted_sum_w
  post_var = (1/V + dXtX/residual_variance)^(-1) # Posterior variance.
  post_mean = (1/residual_variance) * post_var * Xty
  post_mean2 = post_var + post_mean^2 # Second moment.
  lbf_model = maxlpo + log(weighted_sum_w) # Analogue of loglik in the
                                           # non-summary case.

  if (optimize_V == "EM")
    V = optimize_prior_variance(optimize_V,betahat,shat2,prior_weights,alpha,
          post_mean2,check_null_threshold = check_null_threshold)
  return(list(alpha = alpha,mu = post_mean,mu2 = post_mean2,lbf = lbf,
              V = V,lbf_model = lbf_model))
}

# @title Computes standardized.X %*% b using sparse multiplication trick
# @param X an n by p unstandardized matrix with three attributes:
# attr(X,"scaled:center"), attr(X,"scaled:scale") and attr(X,"d")
# @param b a p vector
# @return an n vector
# 
#' @importFrom Matrix t
#' @importFrom Matrix tcrossprod
compute_Xb = function (X, b) {
  cm = attr(X,"scaled:center")
  csd = attr(X,"scaled:scale")
  
  # Scale Xb.
  if (!is.null(attr(X,"matrix.type")))

    # When X is a trend filtering matrix.
    scaled.Xb = compute_tf_Xb(attr(X,"order"),b/csd)
  else
      
    # When X is an ordinary sparse/dense matrix.
    scaled.Xb = tcrossprod(X,t(b/csd))
  
  # Center Xb.
  Xb = scaled.Xb - sum(cm*b/csd)
  return(as.numeric(Xb))
}

# @title Computes t(standardized.X) %*% y using sparse multiplication trick
# @param X an n by p unstandardized matrix with three attributes:
# attr(X,"scaled:center"), attr(X,"scaled:scale") and attr(X,"d")
# @param y an n vector
# @return a p vector
# 
#' @importFrom Matrix t
#' @importFrom Matrix crossprod
compute_Xty = function (X, y) {
  cm = attr(X,"scaled:center")
  csd = attr(X,"scaled:scale")
  ytX = crossprod(y,X)
  
  # Scale Xty.
  if (!is.null(attr(X,"matrix.type")))

    # When X is a trend filtering matrix.
    scaled.Xty = compute_tf_Xty(attr(X,"order"),y)/csd
  else

    # When X is an ordinary sparse/dense matrix.
    scaled.Xty = t(ytX/csd)
  
  # Center Xty.
  centered.scaled.Xty = scaled.Xty - cm/csd * sum(y)
  return(as.numeric(centered.scaled.Xty))
}

# @title Computes M %* %t(standardized.X) using sparse multiplication trick
# @param M a L by p matrix
# @param X an n by p unstandardized matrix with three attributes:
# attr(X,"scaled:center"), attr(X,"scaled:scale") and attr(X,"d")
# @return a L by n matrix
# 
#' @importFrom Matrix t
compute_MXt = function (M, X) {
  cm = attr(X,"scaled:center")
  csd = attr(X,"scaled:scale")
  
  if (!is.null(attr(X,"matrix.type")))

    # When X is a trend filtering matrix.
    return(as.matrix(t(apply(M,1,function(b) compute_Xb(X,b)))))
  else
    
    # When X is an ordinary sparse/dense matrix.
    return(as.matrix(t(X %*% (t(M)/csd)) - drop(M %*% (cm/csd))))

  # This should be the same as
  #
  #   t(apply(M, 1, function(b) compute_Xb(X, b))))
  #
  # as well as
  #
  #   M %*% (t(X)/csd) - drop(tcrossprod(M,t(cm/csd)))
  #
  # but should be more memory-efficient.
}

context("test_sparse_multiplication.R")

test_that("sparse version sparse_multiplication", with(simulate(sparse = TRUE), {
  suppressWarnings(RNGversion("3.5.0"))
  set.seed(1)
  L = 10
  M = matrix(rnorm(L*p), L, p)
  X.dense = set_X_attributes(X)
  X.standardized = t((t(X.dense) - attr(X.dense, 'scaled:center')) / attr(X.dense, 'scaled:scale'))
  X.sparse = set_X_attributes(X.sparse)

  expect_equal(compute_Xb(X.sparse, b), as.numeric(X.standardized%*%b))
  expect_equal(compute_Xb(X.dense, b), as.numeric(X.standardized%*%b))
  expect_equal(compute_Xty(X.sparse, y), as.numeric(t(X.standardized)%*%y))
  expect_equal(compute_Xty(X.dense, y), as.numeric(t(X.standardized)%*%y))
  expect_equal(compute_MXt(M, X.sparse),M%*%t(X.standardized),
               check.attributes =  FALSE,tol = 1e-12)
  expect_equal(compute_MXt(M, X.dense), M%*%t(X.standardized),
               check.attributes = FALSE,tol = 1e-12)
}))

# @title update each effect once
# @param X an n by p matrix of regressor variables
# @param y an n vector of response variable
# @param s a SuSiE fit
# @param estimate_prior_variance boolean indicating whether to
#   estimate prior variance
# @param check_null_threshold float a threshold on the log scale to
#   compare likelihood between current estimate and zero the null
update_each_effect = function (X, y, s, estimate_prior_variance = FALSE,
                               estimate_prior_method = "optim",
                               check_null_threshold) {
  if (!estimate_prior_variance)
    estimate_prior_method = "none"

  # Repeat for each effect to update.
  L = nrow(s$alpha)
  if (L > 0)
    for (l in 1:L) {

      # Remove lth effect from fitted values.
      s$Xr = s$Xr - compute_Xb(X,s$alpha[l,] * s$mu[l,])

      # Compute residuals.
      R = y - s$Xr

      res = single_effect_regression(R,X,s$V[l],s$sigma2,s$pi,
                                     estimate_prior_method,
                                     check_null_threshold)
      # Update the variational estimate of the posterior mean.
      s$mu[l,]    = res$mu
      s$alpha[l,] = res$alpha
      s$mu2[l,]   = res$mu2
      s$V[l]      = res$V
      s$lbf[l]    = res$lbf_model
      s$lbf_variable[l,] = res$lbf
      s$KL[l]     = -res$loglik +
        SER_posterior_e_loglik(X,R,s$sigma2,res$alpha * res$mu,
                               res$alpha * res$mu2)

      s$Xr = s$Xr + compute_Xb(X,s$alpha[l,] * s$mu[l,])
    }
  return(s)
}

context("test_update_each_effect.R")

# test_that("update_each_effect agrees with version 0.3", with(simulate(sparse=T), {
#   original.res = readRDS('vbupdate_original_res.rds')
#   original.res$Xr = as.vector(original.res$Xr)
#   scaledX = set_X_attributes(X)
#   scaledX.sparse = set_X_attributes(X.sparse)
#
#   dense.res = update_each_effect(scaledX,y,s)
#   sparse.res = update_each_effect(scaledX.sparse,y,s)
#
#   expect_equal_susie_update(sparse.res, original.res)
#   expect_equal_susie_update(dense.res, original.res)
# }))

#' @title Apply susie to trend filtering (especially changepoint
#'   problems), a type of non-parametric regression.
#'
#' @description Fits the non-parametric Gaussian regression model
#'   \eqn{y = mu + e}, where the mean \eqn{mu} is modelled as \eqn{mu =
#'   Xb}, X is a matrix with columns containing an appropriate basis,
#'   and b is vector with a (sparse) SuSiE prior. In particular, when
#'   \code{order = 0}, the jth column of X is a vector with the first j
#'   elements equal to zero, and the remaining elements equal to 1, so
#'   that \eqn{b_j} corresponds to the change in the mean of y between
#'   indices j and j+1. For background on trend filtering, see
#'   Tibshirani (2014). See also the "Trend filtering" vignette,
#'   \code{vignette("trend_filtering")}.
#'
#' @details This implementation exploits the special structure of X,
#'   which means that the matrix-vector product \eqn{X^Ty} is fast to
#'   compute; in particular, the computation time is \eqn{O(n)} rather
#'   than \eqn{O(n^2)} if \code{X} were formed explicitly. For
#'   implementation details, see the "Implementation of SuSiE trend
#'   filtering" vignette by running
#'   \code{vignette("trendfiltering_derivations")}.
#'
#' @param y An n-vector of observations ordered in time or space
#'   (assumed to be equally spaced).
#'
#' @param order An integer specifying the order of trend filtering.
#'   The default, \code{order = 0}, corresponds to "changepoint"
#'   problems (\emph{i.e.}, piecewise constant \eqn{mu}). Although
#'   \code{order > 0} is implemented, we do not recommend its use; in
#'   practice, we have found problems with convergence of the algorithm
#'   to poor local optima, producing unreliable inferences.
#' 
#' @param standardize Logical indicating whether to standardize the X
#'   variables ("basis functions"); \code{standardize = FALSE} is
#'   recommended as these basis functions already have a natural scale.
#' 
#' @param use_mad Logical indicating whether to use the "median
#'   absolute deviation" (MAD) method to the estimate residual
#'   variance. If \code{use_mad = TRUE}, susie is run twice, first by
#'   fixing the residual variance to the MAD value, then a second time,
#'   initialized to the first fit, but with residual variance estimated
#'   the usual way (by maximizing the ELBO). We have found this strategy
#'   typically improves reliability of the results by reducing a
#'   tendency to converge to poor local optima of the ELBO.
#' 
#' @param ... Other arguments passed to \code{\link{susie}}.
#' 
#' @return A "susie" fit; see \code{\link{susie}} for details.
#'
#' @references R. J. Tibshirani (2014). Adaptive piecewise polynomial
#'   estimation via trend filtering. \emph{Annals of Statistics}
#'   \bold{42}, 285-323.
#' 
#' @examples
#' set.seed(1)
#' mu = c(rep(0,50),rep(1,50),rep(3,50),rep(-2,50),rep(0,200))
#' y = mu + rnorm(400)
#' s = susie_trendfilter(y)
#' plot(y)
#' lines(mu,col = 1,lwd = 3)
#' lines(predict(s),col = 2,lwd = 2)
#'
#' # Calculate credible sets (indices of y that occur just before
#' # changepoints).
#' susie_get_cs(s)
#'
#' # Plot with credible sets for changepoints.
#' susie_plot_changepoint(s,y) 
#'
#' @importFrom Matrix sparseMatrix
#' 
#' @export
#' 
susie_trendfilter = function (y, order = 0, standardize = FALSE,
                              use_mad = TRUE, ...) {
  if (order > 0)
    warning_message("order > 0 is not recommended")
  n = length(y)
  X = sparseMatrix(i = NULL,j = NULL,dims = c(n,n))
  attr(X,"matrix.type") = "tfmatrix"
  attr(X,"order") = order
  if (use_mad && !("s_init" %in% names(list(...)))) {
    mad = estimate_mad_residual_variance(y)
    s_mad_init = suppressWarnings(susie(X = X,y = y,standardize = standardize,
      estimate_residual_variance = FALSE,residual_variance = mad,...))
    s = susie(X = X,y = y,standardize = standardize, s_init = s_mad_init,...)
  } else
    s = susie(X = X,y = y,standardize = standardize,...)
  return(s)
}

# @title estimate residual variance using MAD estimator
# @param y an n-vector
# @return a scalar of estimated residual variance
estimate_mad_residual_variance = function (y) {
  sigma2 = 0.5*(median(abs(diff(y))/0.6745)^2)
  if (sigma2 == 0) {
    stop("Cannot use median absolute deviation (MAD) to initialize residual variance because MAD = 0 for the input data. Please set 'use_mad = FALSE'")
  }
  return(sigma2)
}

#' @title Extract regression coefficients from susie fit
#'
#' @param object A susie fit.
#'
#' @param \dots Additional arguments passed to the generic \code{coef}
#'   method.
#'
#' @return A p+1 vector, the first element being an intercept, and the
#'   remaining p elements being estimated regression coefficients.
#'
#' @importFrom stats coef
#'
#' @method coef susie
#' 
#' @export coef.susie
#' 
#' @export
#'
coef.susie = function (object, ...) {
  s = object
  return(c(s$intercept,colSums(s$alpha*s$mu)/s$X_column_scale_factors))
}

#' @title Predict outcomes or extract coefficients from susie fit.
#'
#' @param object A susie fit.
#'
#' @param newx A new value for X at which to do predictions.
#'
#' @param type The type of output. For \code{type = "response"},
#'   predicted or fitted outcomes are returned; for \code{type =
#'   "coefficients"}, the estimated coefficients are returned.
#'
#' @param \dots Other arguments used by generic predict function. These
#'   extra arguments are not used here.
#'
#' @return For \code{type = "response"}, predicted or fitted outcomes
#'   are returned; for \code{type = "coefficients"}, the estimated
#'   coefficients are returned. If the susie fit has intercept =
#'   \code{NA} (which is common when using \code{susie_suff_stat}) then
#'   predictions are computed using an intercept of 0, and a warning is
#'   emitted.
#'
#' @importFrom stats coef
#'
#' @method predict susie
#' 
#' @export predict.susie
#' 
#' @export
#'
predict.susie = function (object, newx = NULL,
                          type = c("response","coefficients"), ...) {
  s = object
  type = match.arg(type)
  if (type == "coefficients") {
    if (!missing(newx))
      stop("Do not supply newx when predicting coefficients")
    return(coef(s))
  }
  if (missing(newx))
    return(s$fitted)
  if (is.na(s$intercept)) {
    warning("The prediction assumes intercept = 0")
    return(drop(newx %*% coef(s)[-1]))
  } else
    return(drop(s$intercept + newx %*% coef(s)[-1]))
}

#' @title Plot changepoint data and susie fit using ggplot2
#' 
#' @description Plots original data, y, overlaid with line showing
#'   susie fitted value and shaded rectangles showing credible sets for
#'   changepoint locations.
#' 
#' @param y An n-vector of observations that are ordered in time or
#'   space (assumed equally-spaced).
#' 
#' @param s A susie fit generated by
#'   \code{susie_trendfilter(y,order = 0)}.
#' 
#' @param line_col Color for the line showing fitted values.
#' 
#' @param line_size Size of the lines showing fitted values
#' 
#' @param cs_col Color of the shaded rectangles showing credible
#'   sets.
#' 
#' @return A ggplot2 plot object.
#' 
#' @examples
#' set.seed(1)
#' mu = c(rep(0,50),rep(1,50),rep(3,50),rep(-2,50),rep(0,300))
#' y = mu + rnorm(500)
#' # Here we use a less sensitive tolerance so that the example takes
#' # less time; in practice you will likely want to use a more stringent
#' # setting such as tol = 0.001.
#' s = susie_trendfilter(y,tol = 0.1)
#'
#' # Produces ggplot with credible sets for changepoints.
#' susie_plot_changepoint(s,y) 
#'
#' @importFrom ggplot2 ggplot
#' @importFrom ggplot2 aes_string
#' @importFrom ggplot2 geom_point
#' @importFrom ggplot2 geom_line
#' @importFrom ggplot2 annotate
#' 
#' @export
#' 
susie_plot_changepoint = function (s, y, line_col = "blue", line_size = 1.5,
                                   cs_col = "red") {
  df = data.frame(x = 1:length(y),y = y,mu = predict.susie(s))
  CS = susie_get_cs(s)$cs
  p = ggplot(df) +
    geom_point(data = df,aes_string(x = "x",y = "y")) +
    geom_line(color = line_col,data = df,aes_string(x = "x",y = "mu"),
              size = line_size)
  for(i in 1:length(CS)) 
    p = p + annotate("rect",fill = cs_col,alpha = 0.5,
                     xmin = min(CS[[i]]) - 0.5,xmax = max(CS[[i]]) + 0.5,
                     ymin = -Inf,ymax = Inf)
  return(p)
}

#' @rdname single_effect_regression
#' 
#' @param z A p-vector of z scores.
#' 
#' @param Sigma \code{residual_var*R + lambda*I}
#' 
#' @keywords internal
#' 
single_effect_regression_rss =
  function (z, Sigma, V = 1, prior_weights = NULL,
            optimize_V = c("none", "optim", "uniroot", "EM", "simple"),
            check_null_threshold = 0) {
  p = length(z)
  shat2 = 1/attr(Sigma,"RjSinvRj")
  if (is.null(prior_weights))
    prior_weights = rep(1/p,p)

  if (optimize_V != "EM" && optimize_V != "none") 
    V = optimize_prior_variance_rss(optimize_V,z,Sigma,prior_weights,
                                    alpha = NULL,post_mean2 = NULL,V_init = V,
                                    check_null_threshold=check_null_threshold)

  # log(po) = log(BF * prior) for each SNP
  lbf = sapply(1:p, function(j)
    -0.5 * log(1 + (V/shat2[j])) +
     0.5 * (V/(1 + (V/shat2[j]))) * sum(attr(Sigma,"SinvRj")[,j] * z)^2
  )
  lpo = lbf + log(prior_weights + sqrt(.Machine$double.eps))

  # Deal with special case of infinite shat2 (e.g., happens if X does not
  # vary).
  lbf[is.infinite(shat2)] = 0 
  lpo[is.infinite(shat2)] = 0
  maxlpo = max(lpo)

  # w is proportional to
  #
  #   posterior odds = BF * prior,
  #
  # but subtract max for numerical stability.
  w_weighted = exp(lpo - maxlpo)
  weighted_sum_w = sum(w_weighted)
  alpha = w_weighted / weighted_sum_w
  post_var = (attr(Sigma,"RjSinvRj") + 1/V)^(-1) # Posterior variance.
  post_mean = sapply(1:p,function(j) (post_var[j]) *
              sum(attr(Sigma,"SinvRj")[,j] * z))
  post_mean2 = post_var + post_mean^2 # Second moment.
  lbf_model = maxlpo + log(weighted_sum_w) # Analogue of loglik in the
                                           # non-summary case.

  if (optimize_V=="EM") 
    V = optimize_prior_variance_rss(optimize_V,z,Sigma,prior_weights,
        alpha,post_mean2,check_null_threshold = check_null_threshold)
  
  return(list(alpha = alpha,mu = post_mean,mu2 = post_mean2,lbf = lbf,
              V = V,lbf_model = lbf_model))
}

loglik_rss = function (V, z, Sigma, prior_weights) {
  p = length(z)
  shat2 = 1/attr(Sigma,"RjSinvRj")

  # log(bf) for each SNP.
  lbf = sapply(1:p,function (j)
    -0.5 * log(1 + (V/shat2[j])) +
     0.5 * (V/(1 + (V/shat2[j]))) * sum(attr(Sigma,"SinvRj")[,j] * z)^2)
  lpo = lbf + log(prior_weights + sqrt(.Machine$double.eps))
  
  # Deal with special case of infinite shat2 (e.g., happens if X does
  # not vary).
  lbf[is.infinite(shat2)] = 0 
  lpo[is.infinite(shat2)] = 0
  maxlpo = max(lpo)
  w_weighted = exp(lpo - maxlpo)
  weighted_sum_w = sum(w_weighted)
  return(log(weighted_sum_w) + maxlpo)
}

neg.loglik_z.logscale_rss = function (lV, z, Sigma, prior_weights)
  -loglik_rss(exp(lV),z,Sigma,prior_weights)

optimize_prior_variance_rss = function (optimize_V, z, Sigma, prior_weights,
                                        alpha = NULL, post_mean2 = NULL,
                                        V_init = NULL,
                                        check_null_threshold = 0) {
  V = V_init
  if (optimize_V != "simple") {
    if(optimize_V == "optim") {
      lV = optim(par = log(max(c((colSums(attr(Sigma,"SinvRj") * z)^2) -
                       (1/attr(Sigma,"RjSinvRj")),1e-6),na.rm = TRUE)),
                 fn = neg.loglik_z.logscale_rss,z = z,Sigma = Sigma,
                 prior_weights = prior_weights,method = "Brent",
                 lower = -30,upper = 15)$par
      ## if the estimated one is worse than current one, don't change it.
      if(neg.loglik_z.logscale_rss(lV, z = z,Sigma = Sigma,prior_weights = prior_weights) > 
         neg.loglik_z.logscale_rss(log(V), z = z,Sigma = Sigma,prior_weights = prior_weights)){
        lV = log(V)
      }
      V = exp(lV)
    } else if (optimize_V == "EM")
      V = sum(alpha * post_mean2)
    else
      stop("Invalid option for optimize_V")
  }
  
  # Set V exactly 0 if that beats the numerical value. By
  # check_null_threshold in loglik. check_null_threshold = 0.1 is
  # exp(0.1) = 1.1 on likelihood scale; it means that for parsimony
  # reasons we set estiate of V to zero, if its numerical estimate is
  # only "negligibly" different from zero. We use a likelihood ratio
  # of exp(check_null_threshold) to define "negligible" in this
  # context. This is fairly modest condition compared to, say, a
  # formal LRT with p-value 0.05. But the idea is to be lenient to
  # non-zeros estimates unless they are indeed small enough to be
  # neglible. See more intuition at
  # https://stephens999.github.io/fiveMinuteStats/LR_and_BF.html
  if (loglik_rss(0,z,Sigma,prior_weights) + check_null_threshold >=
      loglik_rss(V,z,Sigma,prior_weights))
    V = 0
  return(V)
}

#' @title Compute sufficient statistics for input to \code{susie_suff_stat}
#'
#' @description Computes the sufficient statistics \eqn{X'X, X'y, y'y}
#'   and \eqn{n} after centering (and possibly standardizing) the
#'   columns of \eqn{X} and centering \eqn{y} to have mean zero. We also
#'   store the column means of \eqn{X} and mean of \eqn{y}.
#'
#' @param X An n by p matrix of covariates.
#'
#' @param y An n vector.
#'
#' @param standardize Logical flag indicating whether to standardize
#'   columns of X to unit variance prior to computing summary data
#'
#' @return A list of sufficient statistics (\code{XtX, Xty, yty, n})
#'   and \code{X_colmeans}, \code{y_mean}.
#'
#' @importFrom methods as
#' @importFrom Matrix colMeans
#' @importFrom Matrix crossprod
#'
#' @examples
#' data(N2finemapping)
#' ss = compute_suff_stat(N2finemapping$X, N2finemapping$Y[,1])
#'
#' @export
#'
compute_suff_stat = function(X, y, standardize = FALSE) {
  y_mean = mean(y)
  y   = y - y_mean
  n   = nrow(X)
  mu  = colMeans(X)
  s   = compute_colSds(X)
  Xty = drop(y %*% X)
  XtX = crossprod(X)
  XtX = as.matrix(XtX)
  XtX = XtX - n*tcrossprod(mu)
  if (standardize) {
    XtX = XtX/s
    XtX = t(XtX)
    XtX = XtX/s
    Xty = Xty/s
  }
  n   = length(y)
  yty = sum(y^2)
  return(list(XtX = XtX,Xty = Xty,yty = yty,n = n,
              y_mean = y_mean,X_colmeans = mu))
}

#' @title Compute sufficient statistics for input to \code{susie_suff_stat}
#'
#' @description This is a synonym for \code{compute_suff_stat}
#'   included for historical reasons (deprecated).
#'
#' @param X An n by p matrix of covariates.
#'
#' @param y An n vector.
#'
#' @param standardize Logical flag indicating whether to standardize
#'   columns of X to unit variance prior to computing summary data.
#'
#' @return A list of sufficient statistics (X'X, X'y, y'y and n)
#'
#' @importFrom methods as
#'
#' @examples
#' data(N2finemapping)
#' ss = compute_ss(N2finemapping$X, N2finemapping$Y[,1])
#'
#' @export
#'
compute_ss = compute_suff_stat

# @title Update each effect once.
# @param R a p by p symmetric and positive semidefinite correlation matrix.
# @param z a p vector
# @param s_init a list with elements sigma2, V, alpha, mu, Xr
# @param Sigma sigma2*R + lambda I
# @param estimate_prior_variance boolean indicating whether to estimate
#   prior variance
# @param check_null_threshold float a threshold on the log scale to
#   compare likelihood between current estimate and zero the null
# 
#' @importFrom Matrix diag
update_each_effect_rss = function (R, z, s_init, Sigma,
                                   estimate_prior_variance = FALSE,
                                   estimate_prior_method = "optim",
                                   check_null_threshold = 0) {

  if (!estimate_prior_variance)
    estimate_prior_method = "none"
  
  # Repeat for each effect to update.
  s = s_init
  L = nrow(s$alpha)
  if(L > 0)
    for (l in 1:L) {
        
      # Remove lth effect from fitted values.
      s$Rz = s$Rz - R %*% (s$alpha[l,] * s$mu[l,])

      # Compute residuals.
      r = z - s$Rz
      res = single_effect_regression_rss(as.vector(r),Sigma,s$V[l],s$pi,
              estimate_prior_method,check_null_threshold)
      
      # Update the variational estimate of the posterior mean.
      s$mu[l,]    = res$mu
      s$alpha[l,] = res$alpha
      s$mu2[l,]   = res$mu2
      s$V[l]      = res$V
      s$lbf[l]    = res$lbf_model
      s$lbf_variable[l,] = res$lbf
      s$KL[l]     = -res$lbf_model +
        SER_posterior_e_loglik_rss(R,Sigma,r,res$alpha * res$mu,
                                   res$alpha * res$mu2)
      s$Rz = s$Rz + R %*% (s$alpha[l,] * s$mu[l,])
    }
  s$Rz = unname(as.matrix(s$Rz))
  return(s)
}

#' @name N2finemapping
#'
#' @title Simulated Fine-mapping Data with Two Effect Variables
#'
#' @docType data
#'
#' @description This data set contains a genotype matrix for 574
#'   individuals and 1,002 variables. The variables are genotypes after
#'   centering and scaling, and therefore retain the correlation
#'   structure of the original genotype data. Two of the variables have
#'   non-zero effects on the multivariate response. The response data
#'   are generated under a multivariate linear regression model. See
#'   Wang \emph{et al} (2020) for details.
#'
#' @format \code{N2finemapping} is a list with the following elements:
#'
#' \describe{
#'
#'   \item{X}{Centered and scaled genotype data.}
#'
#'   \item{chrom}{Chromomsome of the original data, in hg38 coordinates.}
#'
#'   \item{pos}{Chromomosomal position of the original data, in hg38
#'     coordinates. The information can be used to compare impact of using
#'     other genotype references of the same variables in \code{susie_rss}
#'     application.}
#'
#'   \item{true_coef}{Simulated effect sizes.}
#'
#'   \item{residual_variance}{Simulated residual covariance matrix.}
#'
#'   \item{Y}{Simulated multivariate response.}
#'
#'   \item{allele_freq}{Allele frequencies based on the original
#'     genotype data.}
#'
#'   \item{V}{Suggested prior covariance matrix for effect sizes of
#'      the two non-zero effect variables.}
#' }
#'
#' @keywords data
#'
#' @references
#' G. Wang, A. Sarkar, P. Carbonetto and M. Stephens (2020). A simple
#'   new approach to variable selection in regression, with application
#'   to genetic fine-mapping. \emph{Journal of the Royal Statistical
#'   Society, Series B} \doi{10.1101/501114}.
#'
#' @examples
#' data(N2finemapping)
NULL

#' @name N3finemapping
#'
#' @title Simulated Fine-mapping Data with Three Effect Variables.
#'
#' @docType data
#'
#' @description The data-set contains a matrix of 574
#' individuals and 1,001 variables. These variables are real-world
#' genotypes centered and scaled, and therefore retains the
#' correlation structure of variables in the original genotype data. 3
#' out of the variables have non-zero effects.  The response data is
#' generated under a multivariate linear regression model. See Wang
#' \emph{et al} (2020) for more details.
#'
#' @format \code{N3finemapping} is a list with the following elements:
#'
#' \describe{
#'
#'   \item{X}{N by P variable matrix of centered and scaled genotype
#' data.}
#'
#'   \item{chrom}{Chromomsome of the original data, in hg38 coordinate.}
#'
#'   \item{pos}{Chromomosomal positoin of the original data, in hg38
#' coordinate. The information can be used to compare impact of using
#' other genotype references of the same variables in susie_rss
#' application.}
#'
#'   \item{true_coef}{The simulated effect sizes.}
#'
#'   \item{residual_variance}{The simulated residual covariance matrix.}
#'
#'   \item{Y}{The simulated response variables.}
#'
#'   \item{allele_freq}{Allele frequency of the original genotype data.}
#'
#'   \item{V}{Prior covariance matrix for effect size of the three
#' non-zero effect variables.}  }
#'
#' @keywords data
#'
#' @references
#' G. Wang, A. Sarkar, P. Carbonetto and M. Stephens (2020). A simple
#'   new approach to variable selection in regression, with application
#'   to genetic fine-mapping. \emph{Journal of the Royal Statistical
#'   Society, Series B} \doi{10.1101/501114}.
#'
#' @examples
#' data(N3finemapping)
NULL

#' @name FinemappingConvergence
#'
#' @title Simulated Fine-mapping Data with Convergence Problem.
#'
#' @description Data simulated using real genotypes from 50,000
#'   individuals and 200 SNPs. Two of the SNPs have non-zero effects
#'   on the multivariate response. The response data are generated under
#'   a linear regression model. The simulated response and the columns
#'   of the genotype matrix are centered.
#'
#' @format \code{FinemappingConvergence} is a list with the following
#' elements:
#'
#' \describe{
#'
#'   \item{XtX}{Summary statistics computed using the centered and
#'     scaled genotype matrix.}
#'
#'   \item{Xty}{Summary statistics computed using the centered and
#'     scaled genotype data, and the centered simulated response.}
#'
#'   \item{yty}{yty is computed using the centered simulated response.}
#'
#'   \item{n}{The sample size (50,000).}
#'
#'   \item{true_coef}{The coefficients used to simulate the responses.}
#'
#'   \item{z}{z-scores from a simple (single-SNP) linear regression.}}
#'
#' @docType data
#'
#' @keywords data
#'
#' @seealso A similar data set with more SNPs is used in the
#'   \dQuote{Refine SuSiE model} vignette.
#'
#' @examples
#' data(FinemappingConvergence)
NULL

#' @name SummaryConsistency
#'
#' @title Simulated Fine-mapping Data with LD matrix From Reference Panel.
#'
#' @description Data simulated using real genotypes from 10,000
#'   individuals and 200 SNPs. One SNP have non-zero effect
#'   on the multivariate response. The response data are generated under
#'   a linear regression model. There is also one SNP with flipped allele
#'   between summary statistics and the reference panel.
#'
#' @format \code{SummaryConsistency} is a list with the following
#' elements:
#'
#' \describe{
#'
#'   \item{z}{z-scores computed by fitting univariate simple regression
#'     variable-by-variable.}
#'
#'   \item{ldref}{LD matrix estimated from the reference panel.}
#'
#'   \item{flip_id}{The index of the SNP with the flipped allele.}
#'
#'   \item{signal_id}{The index of the SNP with the non-zero effect.}}
#'
#' @seealso A similar data set with more samples is used in the
#'   \dQuote{Diagnostic for fine-mapping with summary statistics}
#'   vignette.
#'
#' @docType data
#'
#' @keywords data
#'
#' @examples
#' data(SummaryConsistency)
NULL

#' @rdname single_effect_regression
#'
#' @title Bayesian single-effect linear regression
#'
#' @description These methods fit the regression model \eqn{y = Xb +
#'   e}, where elements of e are \emph{i.i.d.}  \eqn{N(0,s^2)}, and b is
#'   a p-vector of effects to be estimated. The assumption is that b has
#'   exactly one non-zero element, with all elements equally likely to
#'   be non-zero. The prior on the coefficient of the non-zero element
#'   is \eqn{N(0,V)}.
#'
#' @details \code{single_effect_regression_ss} performs single-effect
#' linear regression with summary data, in which only the statistcs
#' \eqn{X^Ty} and diagonal elements of \eqn{X^TX} are provided to the
#' method.
#'
#' \code{single_effect_regression_rss} performs single-effect linear
#' regression with z scores. That is, this function fits the
#' regression model \eqn{z = R*b + e}, where e is \eqn{N(0,Sigma)},
#' \eqn{Sigma = residual_var*R + lambda*I}, and the b is a p-vector of
#' effects to be estimated. The assumption is that b has exactly one
#' non-zero element, with all elements equally likely to be non-zero.
#' The prior on the non-zero element is \eqn{N(0,V)}. The required
#' summary data are the p-vector \code{z} and the p by p matrix
#' \code{Sigma}. The summary statistics should come from the same
#' individuals.
#'
#' @param y An n-vector.
#'
#' @param X An n by p matrix of covariates.
#'
#' @param V A scalar giving the (initial) prior variance
#'
#' @param residual_variance The residual variance.
#'
#' @param prior_weights A p-vector of prior weights.
#'
#' @param optimize_V The optimization method to use for fitting the
#'   prior variance.
#'
#' @param check_null_threshold Scalar specifying threshold on the
#'   log-scale to compare likelihood between current estimate and zero
#'   the null.
#'
#' @return A list with the following elements:
#'
#' \item{alpha}{Vector of posterior inclusion probabilities;
#'   \code{alpha[i]} is posterior probability that the ith coefficient
#'   is non-zero.}
#'
#' \item{mu}{Vector of posterior means (conditional on inclusion).}
#'
#' \item{mu2}{Vector of posterior second moments (conditional on
#'   inclusion).}
#'
#' \item{lbf}{Vector of log-Bayes factors for each variable.}
#'
#' \item{lbf_model}{Log-Bayes factor for the single effect regression.}
#'
#' \code{single_effect_regression} and \code{single_effect_regression_ss}
#' additionally output:
#'
#' \item{V}{Prior variance (after optimization if \code{optimize_V !=
#'   "none"}).}
#'
#' \item{loglik}{The log-likelihood, \eqn{\log p(y | X, V)}.}
#'
#' @importFrom stats dnorm
#' @importFrom stats uniroot
#' @importFrom stats optim
#' @importFrom Matrix colSums
#'
#' @keywords internal
#'
single_effect_regression =
  function (y, X, V, residual_variance = 1, prior_weights = NULL,
            optimize_V = c("none", "optim", "uniroot", "EM", "simple"),
            check_null_threshold = 0) {
  optimize_V = match.arg(optimize_V)
  Xty = compute_Xty(X,y)
  betahat = (1/attr(X,"d")) * Xty
  shat2 = residual_variance/attr(X,"d")
  if (is.null(prior_weights))
    prior_weights = rep(1/ncol(X),ncol(X))
  if (optimize_V != "EM" && optimize_V != "none")
    V = optimize_prior_variance(optimize_V,betahat,shat2,prior_weights,
        alpha = NULL,post_mean2 = NULL,V_init = V,
        check_null_threshold = check_null_threshold)

  # log(bf) for each SNP
  lbf = dnorm(betahat,0,sqrt(V + shat2),log = TRUE) -
        dnorm(betahat,0,sqrt(shat2),log = TRUE)

  # Deal with special case of infinite shat2 (e.g., happens if X does
  # not vary).
  lbf[is.infinite(shat2)] = 0
  maxlbf = max(lbf)

  # w is proportional to BF, but subtract max for numerical stability.
  w = exp(lbf - maxlbf)

  # Posterior prob for each SNP.
  w_weighted = w * prior_weights
  weighted_sum_w = sum(w_weighted)
  alpha = w_weighted / weighted_sum_w
  post_var = (1/V + attr(X,"d")/residual_variance)^(-1) # Posterior variance.
  post_mean = (1/residual_variance) * post_var * Xty
  post_mean2 = post_var + post_mean^2 # Second moment.

  # BF for single effect model.
  lbf_model = maxlbf + log(weighted_sum_w)
  loglik = lbf_model + sum(dnorm(y,0,sqrt(residual_variance),log = TRUE))

  if(optimize_V == "EM")
    V = optimize_prior_variance(optimize_V,betahat,shat2,prior_weights,
                                alpha,post_mean2,
                                check_null_threshold = check_null_threshold)

  return(list(alpha = alpha,mu = post_mean,mu2 = post_mean2,lbf = lbf,
              lbf_model = lbf_model,V = V,loglik = loglik))
}

# Estimate prior variance.
est_V_uniroot = function (betahat, shat2, prior_weights) {
  V.u = uniroot(negloglik.grad.logscale,c(-10,10),extendInt = "upX",
                betahat = betahat,shat2 = shat2,prior_weights = prior_weights)
  return(exp(V.u$root))
}

optimize_prior_variance = function (optimize_V, betahat, shat2, prior_weights,
                                    alpha = NULL, post_mean2 = NULL,
                                    V_init = NULL, check_null_threshold = 0) {
  V = V_init
  if (optimize_V != "simple") {
    if(optimize_V == "optim") {
      lV = optim(par = log(max(c(betahat^2-shat2,1),na.rm = TRUE)),
          fn = neg.loglik.logscale,betahat = betahat,shat2 = shat2,
          prior_weights = prior_weights,method = "Brent",lower = -30,
          upper = 15)$par
      ## if the estimated one is worse than current one, don't change it.
      if(neg.loglik.logscale(lV, betahat = betahat,shat2 = shat2,prior_weights = prior_weights) >
         neg.loglik.logscale(log(V), betahat = betahat,
                             shat2 = shat2,prior_weights = prior_weights)){
        lV = log(V)
      }
      V = exp(lV)
    } else if (optimize_V == "uniroot")
      V = est_V_uniroot(betahat,shat2,prior_weights)
    else if (optimize_V == "EM")
      V = sum(alpha * post_mean2)
    else
     stop("Invalid option for optimize_V method")
  }

  # Set V exactly 0 if that beats the numerical value by
  # check_null_threshold in loglik. check_null_threshold = 0.1 is
  # exp(0.1) = 1.1 on likelihood scale; it means that for parsimony
  # reasons we set estiate of V to zero, if its numerical estimate is
  # only "negligibly" different from zero. We use a likelihood ratio
  # of exp(check_null_threshold) to define "negligible" in this
  # context. This is fairly modest condition compared to, say, a
  # formal LRT with p-value 0.05. But the idea is to be lenient to
  # non-zeros estimates unless they are indeed small enough to be
  # neglible. See more intuition at
  # https://stephens999.github.io/fiveMinuteStats/LR_and_BF.html
  if (loglik(0,betahat,shat2,prior_weights) +
      check_null_threshold >= loglik(V,betahat,shat2,prior_weights))
    V = 0

  return(V)
}

# In these functions, s2 represents residual_variance, and shat2 is an
# estimate of it.

# The log likelihood function for SER model (based on summary data
# betahat, shat2) as a function of prior variance V.
#
#' @importFrom Matrix colSums
#' @importFrom stats dnorm
loglik = function (V, betahat, shat2, prior_weights) {

  #log(bf) for each SNP
  lbf = dnorm(betahat,0,sqrt(V+shat2),log = TRUE) -
        dnorm(betahat,0,sqrt(shat2),log = TRUE)

  # Deal with special case of infinite shat2 (e.g., happens if X does
  # not vary).
  lbf[is.infinite(shat2)] = 0

  maxlbf = max(lbf)
  w = exp(lbf - maxlbf) # w = BF/BFmax
  w_weighted = w * prior_weights
  weighted_sum_w = sum(w_weighted)
  return(log(weighted_sum_w) + maxlbf)
}

neg.loglik.logscale = function(lV,betahat,shat2,prior_weights)
  -loglik(exp(lV),betahat,shat2,prior_weights)

#' @importFrom Matrix colSums
#' @importFrom stats dnorm
loglik.grad = function(V, betahat, shat2, prior_weights) {

  # log(bf) for each SNP.
  lbf = dnorm(betahat,0,sqrt(V + shat2),log = TRUE) -
        dnorm(betahat,0,sqrt(shat2),log = TRUE)

  # Deal with special case of infinite shat2 (e.g., happens if X does
  # not vary).
  lbf[is.infinite(shat2)] = 0

  maxlbf = max(lbf)
  w = exp(lbf - maxlbf) # w = BF/BFmax
  w_weighted = w * prior_weights
  weighted_sum_w = sum(w_weighted)
  alpha = w_weighted / weighted_sum_w
  return(sum(alpha * lbf.grad(V,shat2,betahat^2/shat2)))
}

# Define loglikelihood and gradient as function of lV:=log(V)
# to improve numerical optimization
negloglik.grad.logscale = function (lV, betahat, shat2, prior_weights)
  -exp(lV) * loglik.grad(exp(lV),betahat,shat2,prior_weights)

# Vector of gradients of logBF_j for each j, with respect to prior
# variance V.
lbf.grad = function (V, shat2, T2) {
  l = 0.5*(1/(V + shat2)) * ((shat2/(V + shat2))*T2 - 1)
  l[is.nan(l)] = 0
  return(l)
}

lbf = function (V, shat2, T2) {
  l = 0.5*log(shat2/(V + shat2)) + 0.5*T2*(V/(V + shat2))
  l[is.nan(l)] = 0
  return(l)
}

context("test_est_V.R")

test_that("est_V has V nonzero for the debug case", {
  debug = readRDS('est_V_debug.rds')
  betahat = debug$b
  shat2 = debug$s2
  prior_weights = debug$pw
  V_debug = est_V_uniroot(betahat, shat2, prior_weights)

  betahat_V0 = rep(0, 512)
  shat2_V0  = rep(1, 512)
  V_0 = est_V_uniroot(betahat_V0, shat2_V0, prior_weights)
  expect_false(V_debug==0)
  expect_equal(V_0, 0)
})

context("test_single_effect_regression.R")

# test_that("single_effect_regression agrees with version 0.3", with(simulate(sparse=T), {
#   original.res = readRDS('singleReg_original_res.rds')
#   V = 0.2
#
#   original.res$alpha = as.vector(original.res$alpha)
#   original.res$mu = as.vector(original.res$mu)
#   original.res$mu2 = as.vector(original.res$mu2)
#   original.res$Xr = as.vector(original.res$Xr)
#   original.res$lbf = as.vector(original.res$lbf)
#
#   scaledX = set_X_attributes(X)
#   scaledX.sparse = set_X_attributes(X.sparse)
#
#   dense.res = single_effect_regression(y,scaledX,V, optimize_V="none")
#   sparse.res = single_effect_regression(y,scaledX.sparse,V, optimize_V="none")
#
#   expect_equal_SER(sparse.res, original.res)
#   expect_equal_SER(dense.res, original.res)
# }))

#' @rdname susie_plots
#'
#' @title SuSiE Plots.
#'
#' @description \code{susie_plot} produces a per-variable summary of
#'   the SuSiE credible sets. \code{susie_plot_iteration} produces a
#'   diagnostic plot for the susie model fitting. For
#'   \code{susie_plot_iteration}, several plots will be created if
#'   \code{track_fit = TRUE} when calling \code{susie}.
#'
#' @param model A SuSiE fit, typically an output from
#'   \code{\link{susie}} or one of its variants. For \code{suse_plot},
#'   the susie fit must have \code{model$z}, \code{model$PIP}, and may
#'   include \code{model$sets}. \code{model} may also be a vector of
#'   z-scores or PIPs.
#'
#' @param y A string indicating what to plot: either \code{"z_original"} for
#'   z-scores, \code{"z"} for z-score derived p-values on (base-10) log-scale, 
#'   \code{"PIP"} for posterior inclusion probabilities,
#'   \code{"log10PIP"} for posterior inclusion probabiliities on the
#'   (base-10) log-scale. For any other setting, the data are plotted as
#'   is.
#'
#' @param add_bar If \code{add_bar = TRUE}, add horizontal bar to
#'   signals in credible interval.
#'
#' @param pos This can be either be (1) a numeric vector of indices of
#'   subset of variables to plot, or (2) a list with the following list
#'   elements: \code{pos$attr}, \code{pos$start} and \code{pos$end},
#'   where \code{pos$attr} is a character string of the name of index
#'   variable in \code{model} object, and \code{pos$start} and
#'   \code{pos$end} are boundaries of indices to plot. See the provided
#'   examples.
#'
#' @param b For simulated data, set \code{b = TRUE} to highlight
#'   "true" effects (highlights in red).
#'
#' @param max_cs The largest credible set to display, either based on
#'   purity (set \code{max_cs} between 0 and 1), or based on size (set
#'   \code{max_cs > 1}).
#'
#' @param add_legend If \code{add_legend = TRUE}, add a legend to
#'   annotate the size and purity of each CS discovered. It can also be
#'   specified as location where legends should be added, e.g.,
#'   \code{add_legend = "bottomright"} (default location is
#'   \code{"topright"}).
#'
#' @param \dots Additional arguments passed to
#'   \code{\link[graphics]{plot}}.
#'
#' @return Invisibly returns \code{NULL}.
#' 
#' @seealso \code{\link{susie_plot_changepoint}}
#' 
#' @examples
#' set.seed(1)
#' n = 1000
#' p = 1000
#' beta = rep(0,p)
#' beta[sample(1:1000,4)] = 1
#' X = matrix(rnorm(n*p),nrow = n,ncol = p)
#' X = scale(X,center = TRUE,scale = TRUE)
#' y = drop(X %*% beta + rnorm(n))
#' res = susie(X,y,L = 10)
#' susie_plot(res,"PIP")
#' susie_plot(res,"PIP",add_bar = TRUE)
#' susie_plot(res,"PIP",add_legend = TRUE)
#' susie_plot(res,"PIP", pos=1:500, add_legend = TRUE)
#' # Plot selected regions with adjusted x-axis position label
#' res$genomic_position = 1000 + (1:length(res$pip))
#' susie_plot(res,"PIP",add_legend = TRUE,
#'            pos = list(attr = "genomic_position",start = 1000,end = 1500))
#' # True effects are shown in red.
#' susie_plot(res,"PIP",b = beta,add_legend = TRUE)
#'
#' @importFrom utils head
#' @importFrom stats pnorm
#' @importFrom graphics plot
#' @importFrom graphics segments
#' @importFrom graphics points
#' @importFrom graphics legend
#' @importFrom graphics par
#'
#' @export
#'
susie_plot = function (model, y, add_bar = FALSE, pos = NULL, b = NULL,
                       max_cs = 400, add_legend = NULL, ...) {
  is_susie = inherits(model,"susie")
  ylab = y
  color = c(
    "dodgerblue2",
    "green4",
    "#6A3D9A", # purple
    "#FF7F00", # orange
    "gold1",
    "skyblue2", "#FB9A99", # lt pink
    "palegreen2",
    "#CAB2D6", # lt purple
    "#FDBF6F", # lt orange
    "gray70", "khaki2",
    "maroon", "orchid1", "deeppink1", "blue1", "steelblue4",
    "darkturquoise", "green1", "yellow4", "yellow3",
    "darkorange4", "brown"
  )
  if (y == "z") {
    if (is_susie) {
      if (is.null(model$z))
        stop("z-scores are not available from SuSiE fit; please set ",
             "compute_univariate_zscore = TRUE in susie() call")
      zneg = -abs(model$z)
    }
    else
      zneg = -abs(model)
    p = -log10(2*pnorm(zneg))
    ylab = "-log10(p)"
  } else if (y == "z_original") {
    if (is_susie) {
      if (is.null(model$z))
        stop("z-scores are not available from SuSiE fit; please set ",
             "compute_univariate_zscore = TRUE in susie() call")
      p = model$z
    } else {
      p = model
    }
    ylab = "z score"
  } else if (y == "PIP") {
    if (is_susie)
      p = model$pip
    else
      p = model
  } else if (y == "log10PIP") {
    if (is_susie)
      p = log10(model$pip)
    else
     p = log10(model)
    ylab = "log10(PIP)"
  } else {
    if (is_susie)
      stop("Need to specify z_original, z, PIP or log10PIP for SuSiE fits")
    p = model
  }
  if(is.null(b))
    b = rep(0,length(p))
  if(is.null(pos))
    pos = 1:length(p)
  start = 0
  if (inherits(pos,"list")) {

    # Check input.
    if (is.null(pos$attr) || is.null(pos$start) || is.null(pos$end))
      stop("pos argument should be a list of list(attr=,start=,end=)")
    if (!(pos$attr %in% names(model)))
      stop(paste("Cannot find attribute",pos$attr,"in input model object"))
    if (pos$start >= pos$end)
      stop("Position start should be smaller than end")
    start = min(min(model[[pos$attr]]),pos$start)
    end = max(max(model[[pos$attr]]),pos$end)

    # Add zeros to alpha and p.
    new_p = rep(NA,end - start + 1)
    pos_with_value = model[[pos$attr]] - start + 1
    new_p[pos_with_value] = p
    p = new_p

    # Adjust model$cs.
    if (!is.null(model$sets$cs)) {
      for (i in 1:length(model$sets$cs))
        model$sets$cs[[i]] = pos_with_value[model$sets$cs[[i]]]
    }

    # Change "pos" object to be indices.
    start_adj = -min(min(model[[pos$attr]]) - pos$start,0)
    end_adj = max(max(model[[pos$attr]]) - pos$end,0)
    pos = (1 + start_adj):(length(p) - end_adj)
  } else {
    if (!all(pos %in% 1:length(p))) 
      stop("Provided position is outside the range of variables")
    pos_with_value = 1:length(p)
  }
  legend_text = list(col = vector(),purity = vector(),size = vector())
  # scipen0 = options()$scipen
  # options(scipen = 10)
  args = list(...)
  if (!exists("xlab", args)) args$xlab = "variable"
  if (!exists("ylab", args)) args$ylab = ylab
  if (!exists("pch", args)) args$pch = 16
  args$x = pos + start
  args$y = p[pos]
  do.call(plot, args)
  if (is_susie && !is.null(model$sets$cs)) {
    for(i in rev(1:nrow(model$alpha))){
      if (!is.null(model$sets$cs_index) && !(i %in% model$sets$cs_index))
        next
      purity = model$sets$purity[which(model$sets$cs_index == i),1]
      if (!is.null(model$sets$purity) && max_cs < 1 && purity >= max_cs) {
        x0 = intersect(pos,model$sets$cs[[which(model$sets$cs_index == i)]])
        y1 = p[x0]
      } else if (n_in_CS(model, model$sets$requested_coverage)[i] < max_cs) {
        x0 = intersect(pos,
               pos[pos_with_value][which(in_CS(model,model$sets$requested_coverage)[i,] > 0)])
        y1 = p[x0]
      } else {
        x0 = NULL
        y1 = NULL
      }
      if (is.null(x0))
        next
      if (add_bar) {
        y0 = rep(0,length(x0))
        x1 = x0
        segments(x0+start,y0,x1+start,y1,lwd = 1.5,col = "gray")
      }
      points(x0+start,y1,col = head(color,1),cex = 1.5,lwd = 2.5)
      legend_text$col = append(head(color,1), legend_text$col)

      # Rotate color.
      color = c(color[-1],color[1])
      legend_text$purity = append(round(purity,4),legend_text$purity)
      legend_text$size = append(length(x0),legend_text$size)
    }
    if (length(legend_text$col) > 0 && !is.null(add_legend) &&
        !identical(add_legend, FALSE)) {

      # Plot legend.
      text = vector()
      for (i in 1:length(legend_text$col)) {
        if (legend_text$size[i] == 1)
          text[i] = paste0("L",i,": C=1")
        else
          text[i] = paste0("L",i,": C=",legend_text$size[i],"/R=",
                           legend_text$purity[i])
      }
      if (!(add_legend %in% c("bottomright", "bottom", "bottomleft", "left", 
        "topleft", "top", "topright", "right", "center"))) {
          add_legend = "topright"
        }
      legend(add_legend,text,bty = "n",col = legend_text$col,cex = 0.65,
             pch = 15)
    }
  }
  points(pos[b != 0] + start,p[b != 0] + start,col = 2,pch = 16)
  # options(scipen = scipen0)
  return(invisible())
}

#' @rdname susie_plots
#'
#' @param L An integer specifying the number of credible sets to plot.
#'
#' @param file_prefix Prefix to path of output plot file. If not
#'   specified, the plot, or plots, will be saved to a temporary
#'   directory generated using \code{\link{tempdir}}.
#'
#' @param pos Indices of variables to plot. If \code{pos = NULL} all
#'   variables are plotted.
#'
#' @examples
#' set.seed(1)
#' n = 1000
#' p = 1000
#' beta = rep(0,p)
#' beta[sample(1:1000,4)] = 1
#' X = matrix(rnorm(n*p),nrow = n,ncol = p)
#' X = scale(X,center = TRUE,scale = TRUE)
#' y = drop(X %*% beta + rnorm(n))
#' res = susie(X,y,L = 10)
#' susie_plot_iteration(res, L=10)
#'
#' @importFrom grDevices pdf
#' @importFrom grDevices dev.off
#' @importFrom reshape melt
#' @importFrom ggplot2 ggplot
#' @importFrom ggplot2 aes_string
#' @importFrom ggplot2 geom_col
#' @importFrom ggplot2 ggtitle
#' @importFrom ggplot2 theme_classic
#'
#' @export
#'
susie_plot_iteration = function (model, L, file_prefix, pos = NULL) {
  get_layer = function (obj, k, idx, vars) {
    alpha = melt(obj$alpha[1:k,vars,drop = FALSE])
    colnames(alpha) = c("L","variables","alpha")
    alpha$L = as.factor(alpha$L)
    ggplot(alpha,aes_string("variables","alpha",group = "L")) +
      geom_col(aes_string(fill = "L")) +
      ggtitle(paste("Iteration",idx)) +
      theme_classic()
  }
  k = min(nrow(model$alpha),L)
  if (is.null(pos))
    vars = 1:ncol(model$alpha)
  else
    vars = pos
  if (missing(file_prefix))
    file_prefix = file.path(tempdir(),"susie_plot")
  pdf(paste0(file_prefix,".pdf"),8,3)
  if (is.null(model$trace))
    print(get_layer(model,k,model$niter,vars))
  else {
    for (i in 2:length(model$trace))
      print(get_layer(model$trace[[i]],k,i-1,vars))
  }
  dev.off()
  format = ".pdf"
  if (!is.null(model$trace)) {
    cmd = paste("convert -delay 30 -loop 0 -density 300 -dispose previous",
                paste0(file_prefix,".pdf"),
                "\\( -clone 0 -set delay 300 \\) -swap 0 +delete",
                "\\( +clone -set delay 300 \\) +swap +delete -coalesce",
                "-layers optimize",paste0(file_prefix,".gif"))
    message("Creating GIF animation...")
    if (file.exists(paste0(file_prefix,".gif")))
      file.remove(paste0(file_prefix,".gif"))
    output = try(system(cmd))
    if (inherits(output,"try-error"))
      stop("Cannot create GIF animation because convert command failed")
    else
      format = ".gif"
  }
  message(paste0("Iterplot saved to ",file_prefix,format,"\n"))
  return(invisible())
}

remove_null_effects = function (s) {
  null_indices = (s$V == 0)
  s$alpha = s$alpha[!null_indices,,drop = FALSE]
  s$mu    = s$mu[!null_indices,,drop = FALSE]
  s$mu2   = s$mu2[!null_indices,,drop = FALSE]
  s$lbf_variable = s$lbf_variable[!null_indices,,drop = FALSE]
  s$V     = s$V[!null_indices,drop = FALSE]
  return(s)
}

add_null_effect = function (s, V) {
  p = ncol(s$alpha)
  s$alpha = rbind(s$alpha,1/p)
  s$mu    = rbind(s$mu,rep(0,p))
  s$mu2   = rbind(s$mu2,rep(0,p))
  s$lbf_variable = rbind(s$lbf_variable,rep(0,p))
  s$V     = c(s$V,V)
  return(s)
}

# @title Estimate residual variance
# @param X an n by p matrix of covariantes
# @param y an n vector of data
# @param s a susie fit
estimate_residual_variance = function (X, y, s) {
  n = nrow(X)
  return((1/n)*get_ER2(X,y,s))
}

# @title Estimate residual variance for summary statistics
# @param XtX a p by p matrix
# @param Xty a p vector
# @param s a susie fit
# @param yty a scaler, y'y, where y is centered to have mean 0
# @param n sample size
estimate_residual_variance_ss = function (XtX, Xty, s, yty, n)
  (1/n)*get_ER2_ss(XtX,Xty,s,yty)

# @title sets the attributes for the R matrix
# @param R a p by p LD matrix
# @param r_tol tolerance level for eigen value check of positive
#   semidefinite matrix of R.
# @return R with attribute e.g., attr(R, 'eigenR') is the eigen
#   decomposition of R.
set_R_attributes = function (R, r_tol) {
  if (is.null(attr(R,"eigen")))
    eigenR = eigen(R,symmetric = TRUE)
  else
    eigenR = attr(R,"eigen")

  # Drop small eigenvalues.
  eigenR$values[abs(eigenR$values) < r_tol] = 0
  if(any(eigenR$values < 0)) {
    min_lambda = min(eigenR$values)
    eigenR$values[eigenR$values < 0] = 0
    warning_message(paste0("The input correlation matrix has negative eigenvalues ",
                   "(smallest one is ", min_lambda, "). The correlation ",
                   "matrix is adjusted such that these negative eigenvalues ",
                   "are now zeros. You can ignore this message, only if you ",
                   "believe the negative eigenvalue is result of numerical ",
                   "rounding errors."))
  }
  res = eigenR$vectors %*% (t(eigenR$vectors) * eigenR$values)

  attr(res,"eigen") = eigenR
  attr(res,"d") = diag(res)
  attr(res,"scaled:scale") = rep(1,length = nrow(R))
  return(res)
}

#' @rdname susie_get_methods
#'
#' @title Inferences From Fitted SuSiE Model
#'
#' @description These functions access basic properties or draw
#'   inferences from a fitted susie model.
#'
#' @param res A susie fit, typically an output from
#'   \code{\link{susie}} or one of its variants. For
#'   \code{susie_get_pip} and \code{susie_get_cs}, this may instead be
#'   the posterior inclusion probability matrix, \code{alpha}.
#'
#' @param last_only If \code{last_only = FALSE}, return the ELBO from
#'   all iterations; otherwise return the ELBO from the last iteration
#'   only.
#'
#' @param warning_tol Warn if ELBO is decreasing by this
#'   tolerance level.
#'
#' @return \code{susie_get_objective} returns the evidence lower bound
#' (ELBO) achieved by the fitted susie model and, optionally, at each
#' iteration of the IBSS fitting procedure.
#'
#' \code{susie_get_residual_variance} returns the (estimated or
#' fixed) residual variance parameter.
#'
#' \code{susie_get_prior_variance} returns the (estimated or fixed)
#' prior variance parameters.
#'
#' \code{susie_get_posterior_mean} returns the posterior mean for the
#' regression coefficients of the fitted susie model.
#'
#' \code{susie_get_posterior_sd} returns the posterior standard
#' deviation for coefficients of the fitted susie model.
#'
#' \code{susie_get_niter} returns the number of model fitting
#' iterations performed.
#'
#' \code{susie_get_pip} returns a vector containing the posterior
#' inclusion probabilities (PIPs) for all variables.
#'
#' \code{susie_get_lfsr} returns a vector containing the average lfsr
#' across variables for each single-effect, weighted by the posterior
#' inclusion probability (alpha).
#'
#' \code{susie_get_posterior_samples} returns a list containing the
#' effect sizes samples and causal status with two components: \code{b},
#' an \code{num_variables} x \code{num_samples} matrix of effect
#' sizes; \code{gamma}, an \code{num_variables} x \code{num_samples}
#' matrix of causal status random draws.
#'
#' \code{susie_get_cs} returns credible sets (CSs) from a susie fit,
#' as well as summaries of correlation among the variables included in
#' each CS. If desired, one can filter out CSs that do not meet a
#' specified \dQuote{purity} threshold; to do this, either \code{X} or
#' \code{Xcorr} must be supplied. It returns a list with the following
#' elements:
#'
#' \item{cs}{A list in which each list element is a vector containing
#'   the indices of the variables in the CS.}
#'
#' \item{coverage}{The nominal coverage specified for each CS.}
#'
#' \item{purity}{If \code{X} or \code{Xcorr} iis provided), the
#'   purity of each CS.}
#'
#' \item{cs_index}{If \code{X} or \code{Xcorr} is provided) the index
#'   (number between 1 and L) of each reported CS in the supplied susie
#'   fit.}
#'
#' @examples
#' set.seed(1)
#' n = 1000
#' p = 1000
#' beta = rep(0,p)
#' beta[1:4] = 1
#' X = matrix(rnorm(n*p),nrow = n,ncol = p)
#' X = scale(X,center = TRUE,scale = TRUE)
#' y = drop(X %*% beta + rnorm(n))
#' s = susie(X,y,L = 10)
#' susie_get_objective(s)
#' susie_get_objective(s, last_only=FALSE)
#' susie_get_residual_variance(s)
#' susie_get_prior_variance(s)
#' susie_get_posterior_mean(s)
#' susie_get_posterior_sd(s)
#' susie_get_niter(s)
#' susie_get_pip(s)
#' susie_get_lfsr(s)
#'
#' @export
#'
susie_get_objective = function (res, last_only = TRUE, warning_tol = 1e-6) {
  if (!all(diff(res$elbo) >= (-1*warning_tol)))
    warning("Objective is decreasing")
  if (last_only)
    return(res$elbo[length(res$elbo)])
  else
    return(res$elbo)
}

#' @rdname susie_get_methods
#'
#' @export
#'
susie_get_posterior_mean = function (res, prior_tol = 1e-9) {

  # Drop the single-effects with estimated prior of zero.
  if (is.numeric(res$V))
    include_idx = which(res$V > prior_tol)
  else
    include_idx = 1:nrow(res$alpha)

  # Now extract relevant rows from alpha matrix.
  if (length(include_idx) > 0)
    return(colSums((res$alpha*res$mu)[include_idx,,drop=FALSE])/
           res$X_column_scale_factors)
  else
    return(numeric(ncol(res$mu)))
}

#' @rdname susie_get_methods
#'
#' @export
#'
susie_get_posterior_sd = function (res, prior_tol = 1e-9) {

  # Drop the single-effects with estimated prior of zero.
  if (is.numeric(res$V))
    include_idx = which(res$V > prior_tol)
  else
    include_idx = 1:nrow(res$alpha)

  # Now extract relevant rows from alpha matrix.
  if (length(include_idx) > 0)
    return(sqrt(colSums((res$alpha * res$mu2 -
                         (res$alpha*res$mu)^2)[include_idx,,drop=FALSE]))/
           (res$X_column_scale_factors))
  else
    return(numeric(ncol(res$mu)))
}

#' @rdname susie_get_methods
#'
#' @export
#'
susie_get_niter = function (res)
  res$niter

#' @rdname susie_get_methods
#'
#' @export
#'
susie_get_prior_variance = function (res)
  res$V

#' @rdname susie_get_methods
#'
#' @export
#'
susie_get_residual_variance = function (res)
  res$sigma2

#' @rdname susie_get_methods
#'
#' @importFrom stats pnorm
#'
#' @export
#'
susie_get_lfsr = function (res) {
  pos_prob = pnorm(0,mean = t(res$mu),sd = sqrt(res$mu2 - res$mu^2))
  neg_prob = 1 - pos_prob
  return(1 - rowSums(res$alpha * t(pmax(pos_prob,neg_prob))))
}

#' @rdname susie_get_methods
#'
#' @param susie_fit A susie fit, an output from \code{\link{susie}}.
#'
#' @param num_samples The number of draws from the posterior
#'   distribution.
#'
#' @importFrom stats rmultinom
#' @importFrom stats rnorm
#'
#' @export
#'
susie_get_posterior_samples = function (susie_fit, num_samples) {

  # Remove effects having estimated prior variance equals zero.
  if (is.numeric(susie_fit$V))
    include_idx = which(susie_fit$V > 1e-9)
  else
    include_idx = 1:nrow(susie_fit$alpha)

  posterior_mean = sweep(susie_fit$mu,2,susie_fit$X_column_scale_factors,"/")
  posterior_sd = sweep(sqrt(susie_fit$mu2 - (susie_fit$mu)^2),2,
                       susie_fit$X_column_scale_factors,"/")

  pip = susie_fit$alpha
  L = nrow(pip)
  num_snps = ncol(pip)
  b_samples = matrix(as.numeric(NA),num_snps,num_samples)
  gamma_samples = matrix(as.numeric(NA),num_snps,num_samples)
  for (sample_i in 1:num_samples) {
    b = 0
    if (length(include_idx) > 0) {
      for (l in include_idx) {
        gamma_l = rmultinom(1,1,pip[l,])
        effect_size = rnorm(1,mean = posterior_mean[l,which(gamma_l != 0)],
                            sd = posterior_sd[l,which(gamma_l != 0)])
        b_l = gamma_l * effect_size
        b = b + b_l
      }
    }
    b_samples[, sample_i] = b
    gamma_samples[, sample_i] = as.numeric(b != 0)
  }
  return(list(b = b_samples,gamma = gamma_samples))
}

#' @rdname susie_get_methods
#'
#' @param X n by p matrix of values of the p variables (covariates) in
#'   n samples. When provided, correlation between variables will be
#'   computed and used to remove CSs whose minimum correlation among
#'   variables is smaller than \code{min_abs_corr}.
#'
#' @param Xcorr p by p matrix of correlations between variables
#'   (covariates). When provided, it will be used to remove CSs whose
#'   minimum correlation among variables is smaller than
#'   \code{min_abs_corr}.
#'
#' @param coverage A number between 0 and 1 specifying desired
#'   coverage of each CS.
#'
#' @param min_abs_corr A "purity" threshold for the CS. Any CS that
#'   contains a pair of variables with correlation less than this
#'   threshold will be filtered out and not reported.
#'
#' @param dedup If \code{dedup = TRUE}, remove duplicate CSs.
#'
#' @param squared If \code{squared = TRUE}, report min, mean and
#' median of squared correlation instead of the absolute correlation.
#'
#' @param check_symmetric If \code{check_symmetric = TRUE}, perform a
#'   check for symmetry of matrix \code{Xcorr} when \code{Xcorr} is
#'   provided (not \code{NULL}).
#'
#' @param n_purity The maximum number of credible set (CS) variables
#'   used in calculating the correlation (\dQuote{purity})
#'   statistics. When the number of variables included in the CS is
#'   greater than this number, the CS variables are randomly subsampled.
#'
#' @param use_rfast Use the Rfast package for the purity calculations.
#'   By default \code{use_rfast = TRUE} if the Rfast package is
#'   installed.
#'
#'
#' @export
#'
susie_get_cs = function (res, X = NULL, Xcorr = NULL, coverage = 0.95,
                         min_abs_corr = 0.5, dedup = TRUE, squared = FALSE,
                         check_symmetric = TRUE, n_purity = 100, use_rfast) {
  if (!is.null(X) && !is.null(Xcorr))
    stop("Only one of X or Xcorr should be specified")
  if (check_symmetric) {
    if (!is.null(Xcorr) && !is_symmetric_matrix(Xcorr)) {
      warning_message("Xcorr is not symmetric; forcing Xcorr to be symmetric",
                  "by replacing Xcorr with (Xcorr + t(Xcorr))/2")
      Xcorr = Xcorr + t(Xcorr)
      Xcorr = Xcorr/2
    }
  }

  null_index = 0
  include_idx = rep(TRUE,nrow(res$alpha))
  if (!is.null(res$null_index)) null_index = res$null_index
  if (is.numeric(res$V)) include_idx = res$V > 1e-9
  # L x P binary matrix.
  status = in_CS(res$alpha,coverage)

  # L-list of CS positions.
  cs = lapply(1:nrow(status),function(i) which(status[i,]!=0))
  claimed_coverage = sapply(1:length(cs),
                            function (i) sum(res$alpha[i,][cs[[i]]]))
  include_idx = include_idx * (lapply(cs,length) > 0)

  # FIXME: see issue 21
  # https://github.com/stephenslab/susieR/issues/21
  if (dedup)
    include_idx = include_idx * (!duplicated(cs))
  include_idx = as.logical(include_idx)
  if (sum(include_idx) == 0)
    return(list(cs = NULL,
                coverage = NULL,
                requested_coverage = coverage))
  cs = cs[include_idx]
  claimed_coverage = claimed_coverage[include_idx]

  # Compute and filter by "purity".
  if (missing(use_rfast))
    use_rfast = requireNamespace("Rfast",quietly = TRUE)
  if (is.null(Xcorr) && is.null(X)) {
    names(cs) = paste0("L",which(include_idx))
    return(list(cs = cs,
                coverage = claimed_coverage,
                requested_coverage = coverage))
  } else {
    purity = NULL
    for (i in 1:length(cs)) {
      if (null_index > 0 && null_index %in% cs[[i]])
        purity = rbind(purity,c(-9,-9,-9))
      else
        purity =
          rbind(purity,
            matrix(get_purity(cs[[i]],X,Xcorr,squared,n_purity,use_rfast),1,3))
    }
    purity = as.data.frame(purity)
    if (squared)
      colnames(purity) = c("min.sq.corr","mean.sq.corr","median.sq.corr")
    else
      colnames(purity) = c("min.abs.corr","mean.abs.corr","median.abs.corr")
    threshold = ifelse(squared,min_abs_corr^2,min_abs_corr)
    is_pure = which(purity[,1] >= threshold)
    if (length(is_pure) > 0) {
      cs        = cs[is_pure]
      purity    = purity[is_pure,]
      row_names = paste0("L",which(include_idx)[is_pure])
      names(cs) = row_names
      rownames(purity) = row_names

      # Re-order CS list and purity rows based on purity.
      ordering = order(purity[,1],decreasing = TRUE)
      return(list(cs       = cs[ordering],
                  purity   = purity[ordering,],
                  cs_index = which(include_idx)[is_pure[ordering]],
                  coverage = claimed_coverage[ordering],
                  requested_coverage=coverage))
    } else
      return(list(cs = NULL,coverage = NULL,requested_coverage = coverage))
  }
}

#' @title Get Correlations Between CSs, using Variable with Maximum PIP From Each CS
#'
#' @description This function evaluates the correlation between single effect
#'   CSs. It is not part of the SuSiE inference. Rather, it is designed as
#'   a diagnostic tool to assess how correlated the reported CS are.
#'
#' @param model A SuSiE fit, typically an output from
#'   \code{\link{susie}} or one of its variants.
#'
#' @param X n by p matrix of values of the p variables (covariates) in
#'   n samples. When provided, correlation between variables will be
#'   computed and used to remove CSs whose minimum correlation among
#'   variables is smaller than \code{min_abs_corr}.
#'
#' @param Xcorr p by p matrix of correlations between variables
#'   (covariates). When provided, it will be used to remove CSs whose
#'   minimum correlation among variables is smaller than
#'   \code{min_abs_corr}.
#'
#' @param max When \code{max = FAFLSE}, return a matrix of CS
#'   correlations. When \code{max = TRUE}, return only the maximum
#'   absolute correlation among all pairs of correlations.
#'
#' @return A matrix of correlations between CSs, or the maximum
#'   absolute correlation when \code{max = TRUE}.
#'
#' @export
#'
get_cs_correlation = function (model, X = NULL, Xcorr = NULL, max = FALSE) {
  if (is.null(model$sets$cs) || length(model$sets$cs) == 1) return(NA)
  if (!is.null(X) && !is.null(Xcorr))
    stop("Only one of X or Xcorr should be specified")
  if (is.null(Xcorr) && is.null(X))
    stop("One of X or Xcorr must be specified")
  if (!is.null(Xcorr) && !is_symmetric_matrix(Xcorr)) {
    warning_message("Xcorr is not symmetric; forcing Xcorr to be symmetric",
                "by replacing Xcorr with (Xcorr + t(Xcorr))/2")
    Xcorr = Xcorr + t(Xcorr)
    Xcorr = Xcorr/2
  }
  # Get index for the best PIP per CS
  max_pip_idx = sapply(model$sets$cs, function(cs) cs[which.max(model$pip[cs])])
  if (is.null(Xcorr)) {
    X_sub = X[,max_pip_idx]
    cs_corr = muffled_corr(as.matrix(X_sub))
  } else {
    cs_corr = Xcorr[max_pip_idx, max_pip_idx]
  }
  if (max) {
    cs_corr = max(abs(cs_corr[upper.tri(cs_corr)]))
  }
  rownames(cs_corr) = colnames(cs_corr) = names(model$sets$cs)
  return(cs_corr)
}

#' @rdname susie_get_methods
#'
#' @param prune_by_cs Whether or not to ignore single effects not in
#'   a reported CS when calculating PIP.
#'
#' @param prior_tol Filter out effects having estimated prior variance
#'   smaller than this threshold.
#'
#' @export
#'
susie_get_pip = function (res, prune_by_cs = FALSE, prior_tol = 1e-9) {

  if (inherits(res,"susie")) {

    # Drop null weight columns.
    if (!is.null(res$null_index) && res$null_index > 0)
      res$alpha = res$alpha[,-res$null_index,drop=FALSE]

    # Drop the single-effects with estimated prior of zero.
    if (is.numeric(res$V))
      include_idx = which(res$V > prior_tol)
    else
      include_idx = 1:nrow(res$alpha)

    # Only consider variables in reported CS.
    # This is not what we do in the SuSiE paper.
    # So by default prune_by_cs = FALSE means we do not run the
    # following code.
    if (!is.null(res$sets$cs_index) && prune_by_cs)
      include_idx = intersect(include_idx,res$sets$cs_index)
    if (is.null(res$sets$cs_index) && prune_by_cs)
      include_idx = numeric(0)

    # now extract relevant rows from alpha matrix
    if (length(include_idx) > 0)
      res = res$alpha[include_idx,,drop = FALSE]
    else
      res = matrix(0,1,ncol(res$alpha))
  }

  return(as.vector(1 - apply(1 - res,2,prod)))
}

# Find how many variables in the CS.
# x is a probability vector.
#' @keywords internal
n_in_CS_x = function (x, coverage = 0.9)
  sum(cumsum(sort(x,decreasing = TRUE)) < coverage) + 1

# Return binary vector indicating if each point is in CS.
# x is a probability vector.
#' @keywords internal
in_CS_x = function (x, coverage = 0.9) {
  n = n_in_CS_x(x,coverage)
  o = order(x,decreasing = TRUE)
  result = rep(0,length(x))
  result[o[1:n]] = 1
  return(result)
}

# Returns an l-by-p binary matrix indicating which variables are in
# susie credible sets.
#' @keywords internal
in_CS = function (res, coverage = 0.9) {
  if (inherits(res,"susie"))
    res = res$alpha
  return(t(apply(res,1,function(x) in_CS_x(x,coverage))))
}

#' @keywords internal
n_in_CS = function(res, coverage = 0.9) {
  if (inherits(res,"susie"))
    res = res$alpha
  return(apply(res,1,function(x) n_in_CS_x(x,coverage)))
}

# Subsample and compute min, mean, median and max abs corr.
#
#' @importFrom stats median
get_purity = function (pos, X, Xcorr, squared = FALSE, n = 100,
                       use_rfast) {
  if (missing(use_rfast))
    use_rfast = requireNamespace("Rfast",quietly = TRUE)
  if (use_rfast) {
    get_upper_tri = Rfast::upper_tri
    get_median    = Rfast::med
  } else {
    get_upper_tri = function (R) R[upper.tri(R)]
    get_median    = stats::median
  }
  if (length(pos) == 1)
    return(c(1,1,1))
  else {

    # Subsample the columns if necessary.
    if (length(pos) > n)
      pos = sample(pos,n)

    if (is.null(Xcorr)) {
      X_sub = X[,pos]
      X_sub = as.matrix(X_sub)
      value = abs(get_upper_tri(muffled_corr(X_sub)))
    } else
      value = abs(get_upper_tri(Xcorr[pos,pos]))
    if (squared)
      value = value^2
    return(c(min(value),
             sum(value)/length(value),
             get_median(value)))
  }
}

# Correlation function with specified warning muffled.
#
#' @importFrom stats cor
muffled_corr = function (x)
  withCallingHandlers(cor(x),
                      warning = function(w) {
                        if (grepl("the standard deviation is zero",w$message))
                          invokeRestart("muffleWarning")
                      })

# cov2cor function with specified warning muffled.
#
#' @importFrom stats cov2cor
#' @keywords internal
muffled_cov2cor = function (x)
  withCallingHandlers(cov2cor(x),
    warning = function(w) {
      if (grepl("had 0 or NA entries; non-finite result is doubtful",
                w$message))
          invokeRestart("muffleWarning")
      })

# Check for symmetric matrix.
#' @keywords internal
is_symmetric_matrix = function (x) {
  if (requireNamespace("Rfast",quietly = TRUE))
    return(Rfast::is.symmetric(x))
  else
    return(isSymmetric(x))
}

# Compute standard error for regression coef.
# S = (X'X)^-1 \Sigma
#' @keywords internal
calc_stderr = function (X, residuals)
  sqrt(diag(sum(residuals^2)/(nrow(X) - 2) * chol2inv(chol(crossprod(X)))))

# Return residuals of Y after removing the linear effects of the susie
# model.
#
#' @importFrom stats coef
#' @keywords internal
get_R = function (X, Y, s)
  Y - X %*% coef(s)

# Slim the result of fitted susie model.
#' @keywords internal
susie_slim = function (res)
  list(alpha = res$alpha,niter = res$niter,V = res$V,sigma2 = res$sigma2)

# Prune single effects to given number L in susie model object.
#' @keywords internal
susie_prune_single_effects = function (s,L = 0,V = NULL) {
  num_effects = nrow(s$alpha)
  if (L == 0) {
    # Filtering will be based on non-zero elements in s$V.
    if (!is.null(s$V))
      L = length(which(s$V > 0))
    else
      L = num_effects
  }
  if (L == num_effects) {
    s$sets = NULL
    return(s)
  }
  if (!is.null(s$sets$cs_index))
    effects_rank = c(s$sets$cs_index,setdiff(1:num_effects,s$sets$cs_index))
  else
    effects_rank = 1:num_effects

  if (L > num_effects) {
    message(paste("Specified number of effects L =",L,
                  "is greater the number of effects",num_effects,
                  "in input SuSiE model. The SuSiE model will be expanded",
                  "to have",L,"effects."))

    s$alpha = rbind(s$alpha[effects_rank,],
                    matrix(1/ncol(s$alpha),L - num_effects,ncol(s$alpha)))
    for (n in c("mu","mu2","lbf_variable"))
      if (!is.null(s[[n]]))
        s[[n]] = rbind(s[[n]][effects_rank,],
                       matrix(0,L - num_effects,ncol(s[[n]])))
    for (n in c("KL", "lbf"))
      if (!is.null(s[[n]]))
        s[[n]] = c(s[[n]][effects_rank],rep(NA, L-num_effects))
    if (!is.null(V)) {
      if (length(V) > 1)
        V[1:num_effects] = s$V[effects_rank]
      else V = rep(V,L)
    }
    s$V = V
  }
  s$sets = NULL
  return(s)
}

#' @title Estimate s in \code{susie_rss} Model Using Regularized LD
#'
#' @description The estimated s gives information about the
#'   consistency between the z scores and LD matrix. A larger \eqn{s}
#'   means there is a strong inconsistency between z scores and LD
#'   matrix. The \dQuote{null-mle} method obtains mle of \eqn{s} under
#'   \eqn{z | R ~ N(0,(1-s)R + s I)}, \eqn{0 < s < 1}. The
#'   \dQuote{null-partialmle} method obtains mle of \eqn{s} under
#'   \eqn{U^T z | R ~ N(0,s I)}, in which \eqn{U} is a matrix containing
#'   the of eigenvectors that span the null space of R; that is, the
#'   eigenvectors corresponding to zero eigenvalues of R. The estimated
#'   \eqn{s} from \dQuote{null-partialmle} could be greater than 1. The
#'   \dQuote{null-pseudomle} method obtains mle of \eqn{s} under
#'   pseudolikelihood \eqn{L(s) = \prod_{j=1}^{p} p(z_j | z_{-j}, s,
#'   R)}, \eqn{0 < s < 1}.
#'
#' @param z A p-vector of z scores.
#'
#' @param R A p by p symmetric, positive semidefinite correlation
#'   matrix.
#'
#' @param n The sample size. (Optional, but highly recommended.)
#'
#' @param r_tol Tolerance level for eigenvalue check of positive
#'   semidefinite matrix of R.
#'
#' @param method a string specifies the method to estimate \eqn{s}.
#'
#' @return A number between 0 and 1.
#'
#' @examples
#' set.seed(1)
#' n = 500
#' p = 1000
#' beta = rep(0,p)
#' beta[1:4] = 0.01
#' X = matrix(rnorm(n*p),nrow = n,ncol = p)
#' X = scale(X,center = TRUE,scale = TRUE)
#' y = drop(X %*% beta + rnorm(n))
#' input_ss = compute_suff_stat(X,y,standardize = TRUE)
#' ss = univariate_regression(X,y)
#' R = cor(X)
#' attr(R,"eigen") = eigen(R,symmetric = TRUE)
#' zhat = with(ss,betahat/sebetahat)
#'
#' # Estimate s using the unadjusted z-scores.
#' s0 = estimate_s_rss(zhat,R)
#' 
#' # Estimate s using the adjusted z-scores.
#' s1 = estimate_s_rss(zhat,R,n)
#'
#' @importFrom stats dnorm
#' @importFrom stats optim
#'
#' @export
#'
estimate_s_rss = function (z, R, n, r_tol = 1e-08, method = "null-mle") {

  # Check and process input arguments z, R.
  z[is.na(z)] = 0
  if (is.null(attr(R,"eigen")))
    attr(R,"eigen") = eigen(R,symmetric = TRUE)
  eigenld = attr(R,"eigen")
  if (any(eigenld$values < -r_tol))
    warning_message("The matrix R is not positive semidefinite. Negative ",
            "eigenvalues are set to zero")
  eigenld$values[eigenld$values < r_tol] = 0

  # Check input n, and adjust the z-scores if n is provided.
  if (missing(n))
    warning_message("Providing the sample size (n), or even a rough estimate of n, ",
            "is highly recommended. Without n, the implicit assumption is ",
            "n is large (Inf) and the effect sizes are small (close to zero).")
  else if (n <= 1)
    stop("n must be greater than 1")
  if (!missing(n)) {
    sigma2 = (n-1)/(z^2 + n - 2)
    z = sqrt(sigma2) * z
  }

  if (method == "null-mle") {
    negloglikelihood = function(s, z, eigenld)
      0.5 * sum(log((1-s)*eigenld$values + s)) +
      0.5 * sum(z * eigenld$vectors %*% ((t(eigenld$vectors) *
               (1/((1-s)*eigenld$values + s))) %*% z))
    s = optim(0.5,fn = negloglikelihood,z = z,eigenld = eigenld,
              method = "Brent",lower = 0,upper = 1)$par
  } else if (method == "null-partialmle") {
    colspace = which(eigenld$values > 0)
    if (length(colspace) == length(z))
      s = 0
    else{
      znull = crossprod(eigenld$vectors[,-colspace], z) # U2^T z
      s = sum(znull^2)/length(znull)
    }
  } else if (method == "null-pseudomle") {
    pseudolikelihood = function(s, z, eigenld) {
      precision = eigenld$vectors %*% (t(eigenld$vectors) *
                  (1/((1-s)*eigenld$values + s)))
      postmean = rep(0,length(z))
      postvar = rep(0,length(z))
      for (i in 1:length(z)) {
        postmean[i] = -(1/precision[i,i])*precision[i,-i] %*% z[-i]
        postvar[i] = 1/precision[i,i]
      }
      return(-sum(dnorm(z,mean = postmean,sd = sqrt(postvar),log = TRUE)))
    }
    s = optim(0.5,fn = pseudolikelihood,z = z,eigenld = eigenld,
              method = "Brent",lower = 0,upper = 1)$par
  }
  else
    stop("The method is not implemented")
  return(s)
}

#' @title Compute Distribution of z-scores of Variant j Given Other z-scores, and Detect Possible Allele Switch Issue
#'
#' @description Under the null, the rss model with regularized LD
#'   matrix is \eqn{z|R,s ~ N(0, (1-s)R + s I))}. We use a mixture of
#'   normals to model the conditional distribution of z_j given other z
#'   scores, \eqn{z_j | z_{-j}, R, s ~ \sum_{k=1}^{K} \pi_k
#'   N(-\Omega_{j,-j} z_{-j}/\Omega_{jj}, \sigma_{k}^2/\Omega_{jj})},
#'   \eqn{\Omega = ((1-s)R + sI)^{-1}}, \eqn{\sigma_1, ..., \sigma_k}
#'   is a grid of fixed positive numbers. We estimate the mixture
#'   weights \eqn{\pi}  We detect the possible allele switch issue
#'   using likelihood ratio for each variant.
#'
#' @param z A p-vector of z scores.
#'
#' @param R A p by p symmetric, positive semidefinite correlation
#'   matrix.
#'
#' @param n The sample size. (Optional, but highly recommended.)
#'
#' @param r_tol Tolerance level for eigenvalue check of positive
#'   semidefinite matrix of R.
#'
#' @param s an estimated s from \code{estimate_s_rss}
#'
#' @return a list containing a ggplot2 plot object and a table. The plot
#'   compares observed z score vs the expected value. The possible allele
#'   switched variants are labeled as red points (log LR > 2 and abs(z) > 2).
#'   The table summarizes the conditional distribution for each variant
#'   and the likelihood ratio test. The table has the following columns:
#'   the observed z scores, the conditional expectation, the conditional
#'   variance, the standardized differences between the observed z score
#'   and expected value, the log likelihood ratio statistics.
#'
#' @importFrom stats dnorm
#' @importFrom ggplot2 ggplot
#' @importFrom ggplot2 geom_point
#' @importFrom ggplot2 geom_abline
#' @importFrom ggplot2 theme_bw
#' @importFrom ggplot2 labs
#' @importFrom ggplot2 aes
#' @importFrom mixsqp mixsqp
#'
#' @examples
#' # See also the vignette, "Diagnostic for fine-mapping with summary
#' # statistics."
#' set.seed(1)
#' n = 500
#' p = 1000
#' beta = rep(0,p)
#' beta[1:4] = 0.01
#' X = matrix(rnorm(n*p),nrow = n,ncol = p)
#' X = scale(X,center = TRUE,scale = TRUE)
#' y = drop(X %*% beta + rnorm(n))
#' ss = univariate_regression(X,y)
#' R = cor(X)
#' attr(R,"eigen") = eigen(R,symmetric = TRUE)
#' zhat = with(ss,betahat/sebetahat)
#' cond_dist = kriging_rss(zhat,R,n = n)
#' cond_dist$plot
#'
#' @export
#'
kriging_rss = function (z, R, n, r_tol = 1e-08,
                        s = estimate_s_rss(z,R,n,r_tol,method = "null-mle")) {

  # Check and process input arguments z, R.
  z[is.na(z)] = 0
  if (is.null(attr(R,"eigen")))
    attr(R,"eigen") = eigen(R,symmetric = TRUE)
  eigenld = attr(R,"eigen")
  if (any(eigenld$values < -r_tol))
    warning_message("The matrix R is not positive semidefinite. Negative ",
            "eigenvalues are set to zero.")
  eigenld$values[eigenld$values < r_tol] = 0

  # Check and progress input argument s.
  force(s)
  if (s > 1) {
    warning_message("The given s is greater than 1. We replace it with 0.8.")
    s = 0.8
  } else if (s < 0)
    stop("The s must be non-negative")
  
  # Check input n, and adjust the z-scores if n is provided.
  if ((!missing(n)) && (n <= 1))
    stop("n must be greater than 1")
  if (missing(n))
    warning_message("Providing the sample size (n), or even a rough estimate of n, ",
            "is highly recommended. Without n, the implicit assumption is ",
            "n is large (Inf) and the effect sizes are small (close to zero).")
  else {
    sigma2 = (n-1)/(z^2 + n - 2)
    z = sqrt(sigma2) * z
  }
  
  dinv = 1/((1-s) * eigenld$values + s)
  dinv[is.infinite(dinv)] = 0
  precision = eigenld$vectors %*% (t(eigenld$vectors) * dinv)
  condmean = rep(0,length(z))
  condvar = rep(0,length(z))
  for (i in 1:length(z)) {
    condmean[i] = -(1/precision[i,i]) * precision[i,-i] %*% z[-i]
    condvar[i]  = 1/precision[i,i]
  }
  z_std_diff = (z-condmean)/sqrt(condvar)

  # obtain grid
  a_min = 0.8
  if (max(z_std_diff^2) < 1)
    a_max = 2
  else
    a_max = 2*sqrt(max(z_std_diff^2))
  npoint = ceiling(log2(a_max/a_min)/log2(1.05))
  a_grid = 1.05^(seq(-npoint,0)) * a_max

  # compute likelihood
  sd_mtx      = outer(sqrt(condvar),a_grid)
  matrix_llik = dnorm(z - condmean,sd = sd_mtx,log = TRUE)
  lfactors    = apply(matrix_llik,1,max)
  matrix_llik = matrix_llik - lfactors

  # estimate weight
  w = mixsqp(matrix_llik,log = TRUE,control = list(verbose = FALSE))$x

  # Compute denominators in likelihood ratios.
  logl0mix = drop(log(exp(matrix_llik) %*% (w + 1e-15))) + lfactors

  # Compute numerators in likelihood ratios.
  matrix_llik = dnorm(z + condmean,sd = sd_mtx,log = TRUE)
  lfactors    = apply(matrix_llik,1,max)
  matrix_llik = matrix_llik - lfactors
  logl1mix    = drop(log(exp(matrix_llik) %*% (w + 1e-15))) + lfactors

  # Compute (log) likelihood ratios.
  logLRmix = logl1mix - logl0mix

  res = data.frame(z = z,condmean = condmean,condvar = condvar,
                   z_std_diff = z_std_diff,logLR = logLRmix)

  p = ggplot(res) + geom_point(aes(y = z, x = condmean)) +
        labs(y = "Observed z scores", x = "Expected value") +
        geom_abline(intercept = 0, slope = 1) +
        theme_bw()
  idx = which(logLRmix > 2 & abs(z) > 2)
  if (length(idx) > 0)
    p = p + geom_point(data = res[idx,],
                       aes(y = z, x = condmean),col = "red")
  return(list(plot = p,conditional_dist = res))
}

# Compute the column means of X, the column standard deviations of X,
# and rowSums(Y^2), where Y is the centered and/or scaled version of
# X.
#
#' @importFrom Matrix rowSums
#' @importFrom Matrix colMeans
compute_colstats = function (X, center = TRUE, scale = TRUE) {
  n = nrow(X)
  p = ncol(X)
  if (!is.null(attr(X,"matrix.type"))) {

    # X is a trend filtering matrix.
    cm  = compute_tf_cm(attr(X,"order"),p)
    csd = compute_tf_csd(attr(X,"order"),p)
    d   = compute_tf_d(attr(X,"order"),p,cm,csd,scale,center)
    if (!center)
      cm = rep(0,p)
    if (!scale)
      csd = rep(1,p)
  } else {

    # X is an ordinary dense or sparse matrix. Set sd = 1 when the
    # column has variance 0.
    if (center)
      cm = colMeans(X,na.rm = TRUE)
    else
      cm = rep(0,p)
    if (scale) {
      csd = compute_colSds(X)
      csd[csd == 0] = 1
    } else
      csd = rep(1,p)

    # These two lines of code should give the same result as
    #
    #   Y = (t(X) - cm)/csd
    #   d = rowSums(Y^2)
    #
    # for all four combinations of "center" and "scale", but do so
    # without having to modify X, or create copies of X in memory. In
    # particular the first line should be equivalent to colSums(X^2).
    d = n*colMeans(X)^2 + (n-1)*compute_colSds(X)^2
    d = (d - n*cm^2)/csd^2
  }

  return(list(cm = cm,csd = csd,d = d))
}

# @title computes column standard deviations for any type of matrix
# @details This should give the same result as matrixStats::colSds(X),
#   but allows for sparse matrices as well as dense ones.
# @param X an n by p matrix of any type, e.g. sparse, dense.
# @return a p vector of column standard deviations.
#
#' @importFrom Matrix rowSums
#' @importFrom Matrix colSums
#' @importFrom Matrix rowMeans
#' @importFrom matrixStats colSds
compute_colSds = function(X) {
  if (is.matrix(X))
    y = colSds(X)
  else {
    n = nrow(X)
    X = t(X)
    X = X - rowMeans(X)
    y = sqrt(rowSums(X^2)/(n-1))
  }
  return(y)
}

# @title Check whether A is positive semidefinite
# @param A a symmetric matrix
# @return a list of result:
# \item{matrix}{The matrix with eigen decomposition}
# \item{status}{whether A is positive semidefinite}
# \item{eigenvalues}{eigenvalues of A truncated by r_tol}
check_semi_pd = function (A, tol) {
  attr(A,"eigen") = eigen(A,symmetric = TRUE)
  v = attr(A,"eigen")$values
  v[abs(v) < tol] = 0
  return(list(matrix      = A,
              status      = !any(v < 0),
              eigenvalues = v))
}

# @title Check whether b is in space spanned by the non-zero eigenvectors
#   of A
# @param A a p by p matrix
# @param b a length p vector
# @return a list of result:
# \item{status}{whether b in space spanned by the non-zero
#  eigenvectors of A}
# \item{msg}{msg gives the difference between the projected b and b if
#   status is FALSE}
check_projection = function (A, b) {
  if (is.null(attr(A,"eigen")))
    attr(A,"eigen") = eigen(A,symmetric = TRUE)
  v = attr(A,"eigen")$values
  B = attr(A,"eigen")$vectors[,v > .Machine$double.eps]
  msg = all.equal(as.vector(B %*% crossprod(B,b)),as.vector(b),
                  check.names = FALSE)
  if (!is.character(msg))
    return(list(status = TRUE,msg = NA))
  else
    return(list(status = FALSE,msg = msg))
}

# @title Utility function to display warning messages as they occur
# @param ... warning message
# @param style either "warning" or "hint"
#'@importFrom crayon combine_styles
warning_message = function(..., style=c("warning", "hint")) {
  style = match.arg(style)
  if (style=="warning" && getOption("warn")>=0) {
    alert <- combine_styles("bold", "underline", "red")	
    message(alert("WARNING:"), " ", ...)
  } else {
    alert <- combine_styles("bold", "underline", "magenta")	
    message(alert("HINT:"), " ", ...)
  }
}

context("test_compute_colsds.R")

test_that(paste("Check that compute_colSds gives same result as",
                "matrixStats::colSds for a dense matrix"),{
  set.seed(1)
  X  <- matrix(rnorm(24),4,6)
  y1 <- compute_colSds(X)
  y2 <- matrixStats::colSds(X)
  expect_equal(y1,y2,scale = 1,tolerance = 1e-15)
})

test_that(paste("Check that compute_colSds gives same result as",
                "matrixStats::colSds for a sparse matrix"),{
  set.seed(1)
  X  <- matrix(rnorm(24),4,6)
  X[runif(24) < 0.3] <- 0
  Y  <- as(X,"CsparseMatrix")
  y1 <- compute_colSds(Y)
  y2 <- matrixStats::colSds(X)
  expect_equal(y1,y2,scale = 1,tolerance = 1e-15)
})

context("test_get_samples.R")
test_that("Sampling from posterior distribution agrees with expected value", with(simulate(200, 500), {
  set.seed(1)
  res1 = susie(X, y)
  post_samples1 = susie_get_posterior_samples(res1, 10000)
  expect_equal(dim(post_samples1$b), c(500, 10000))
  expect_equal(dim(post_samples1$gamma), c(500, 10000))
  expect_equal(rowMeans(post_samples1$gamma), res1$pip)
  expect_equal(rowMeans(post_samples1$b),
               susie_get_posterior_mean(res1), tolerance=0.01)

  ss = univariate_regression(X, y)
  R = cor(X)
  z = ss$betahat/ss$sebetahat
  res2 = susie_rss(z,R, n=n)
  post_samples2 = susie_get_posterior_samples(res2, 10000)
  expect_equal(dim(post_samples2$b), c(500, 10000))
  expect_equal(dim(post_samples2$gamma), c(500, 10000))
  expect_equal(rowMeans(post_samples2$gamma), res2$pip, tolerance = 1e-4)
  expect_equal(rowMeans(post_samples2$b), susie_get_posterior_mean(res2), tolerance=0.01)
}))

 context("test_get_pip.R")
 test_that("PIP computation is correct in light of null weight and estimated prior",{
   res = list(alpha = matrix(c(rep(1,10),rep(2,10),rep(3,10))/10, 3, 10, byrow=T), sets=list(cs_index = c(2)), null_index = 10, V = rep(1,3))
   class(res) = 'susie'
   expect_equal(susie_get_pip(res), rep(0.496, 9))
   #
   expect_equal(susie_get_pip(res, prune_by_cs=T), rep(0.2, 9))
   res$V = rep(0,3)
   expect_equal(susie_get_pip(res), rep(0,9))
   res$V = rep(1,3)
   res$sets$cs_index=NULL
   expect_equal(susie_get_pip(res,prune_by_cs=T), rep(0,9))
 })

#' @title Summarize Susie Fit.
#' 
#' @description \code{summary} method for the \dQuote{susie} class.
#' 
#' @param object A susie fit.
#'
#' @param \dots Additional arguments passed to the generic \code{summary}
#'   or \code{print.summary} method.
#' 
#' @return \code{summary.susie} returns a list containing a data frame
#'   of variables and a data frame of credible sets.
#' 
#' @method summary susie
#' 
#' @export summary.susie
#' 
#' @export
#' 
summary.susie = function (object, ...) {
  if (is.null(object$sets))
    stop("Cannot summarize SuSiE object because credible set information ",
         "is not available")
  variables = data.frame(cbind(1:length(object$pip),object$pip,-1))
  colnames(variables) = c("variable","variable_prob","cs")
  rownames(variables) = NULL
  if (object$null_index > 0)
    variables = variables[-object$null_index,]
  if (!is.null(object$sets$cs)) {
    cs = data.frame(matrix(NA,length(object$sets$cs),5))
    colnames(cs) = c("cs","cs_log10bf","cs_avg_r2","cs_min_r2","variable")
      for (i in 1:length(object$sets$cs)) {
        variables$cs[variables$variable %in% object$sets$cs[[i]]] =
          object$sets$cs_index[[i]]
        cs$cs[i] = object$sets$cs_index[[i]]
        cs$cs_log10bf[i] = log10(exp(object$lbf[cs$cs[i]]))
        cs$cs_avg_r2[i] = object$sets$purity$mean.abs.corr[i]^2
        cs$cs_min_r2[i] = object$sets$purity$min.abs.corr[i]^2
        cs$variable[i] = paste(object$sets$cs[[i]],collapse=",")
      }
      variables = variables[order(variables$variable_prob,decreasing = TRUE),]
  } else
    cs = NULL
  out = list(vars = variables,cs = cs)
  class(out) = c("summary.susie","list")
  return(out)
}

#' @rdname summary.susie
#' 
#' @param x A susie summary.
#'
#' @method print summary.susie
#' 
#' @export print.summary.susie
#' 
#' @export
#' 
print.summary.susie = function (x, ...) {
  cat("\nVariables in credible sets:\n\n")
  print.data.frame(x$vars[which(x$vars$cs > 0),],row.names = FALSE)
  cat("\nCredible sets summary:\n\n")
  print.data.frame(x$cs,row.names = FALSE)
}

#' @title Initialize a susie object using regression coefficients
#'
#' @param coef_index An L-vector containing the the indices of the
#'   nonzero coefficients.
#'
#' @param coef_value An L-vector containing initial coefficient
#' estimates.
#'
#' @param p A scalar giving the number of variables.
#'
#' @return A list with elements \code{alpha}, \code{mu} and \code{mu2}
#'   to be used by \code{susie}.
#'
#' @examples
#' set.seed(1)
#' n = 1000
#' p = 1000
#' beta = rep(0,p)
#' beta[sample(1:1000,4)] = 1
#' X = matrix(rnorm(n*p),nrow = n,ncol = p)
#' X = scale(X,center = TRUE,scale = TRUE)
#' y = drop(X %*% beta + rnorm(n))
#'
#' # Initialize susie to ground-truth coefficients.
#' s = susie_init_coef(which(beta != 0),beta[beta != 0],length(beta))
#' res = susie(X,y,L = 10,s_init=s)
#'
#' @export
#'
susie_init_coef = function (coef_index, coef_value, p) {
  L = length(coef_index)
  if (L <= 0)
    stop("Need at least one non-zero effect")
  if (!all(coef_value != 0))
    stop("Input coef_value must be non-zero for all its elements")
  if (L != length(coef_value))
    stop("Inputs coef_index and coef_value must of the same length")
  if (max(coef_index) > p)
    stop("Input coef_index exceeds the boundary of p")
  alpha = matrix(0,nrow = L,ncol = p)
  mu = matrix(0,nrow = L,ncol = p)
  for(i in 1:L){
    alpha[i,coef_index[i]] = 1
    mu[i,coef_index[i]] = coef_value[i]
  }
  out = list(alpha = alpha,mu = mu,mu2 = mu*mu)
  class(out) = c("susie","list")
  return(out)
}

# Set default susie initialization.
init_setup = function (n, p, L, scaled_prior_variance, residual_variance,
                       prior_weights, null_weight, varY, standardize) {
  if (!is.numeric(scaled_prior_variance) || scaled_prior_variance < 0)
    stop("Scaled prior variance should be positive number")
  if (scaled_prior_variance > 1 && standardize)
    stop("Scaled prior variance should be no greater than 1 when ",
         "standardize = TRUE")
  if(is.null(residual_variance))
    residual_variance = varY
  if(is.null(prior_weights)){
    prior_weights = rep(1/p,p)
  }else{
    if(all(prior_weights == 0)){
      stop("Prior weight should greater than 0 for at least one variable.")
    }
    prior_weights = prior_weights / sum(prior_weights)
  }
  if(length(prior_weights) != p)
    stop("Prior weights must have length p")
  if (p < L)
    L = p
  s = list(alpha  = matrix(1/p,nrow = L,ncol = p),
           mu     = matrix(0,nrow = L,ncol = p),
           mu2    = matrix(0,nrow = L,ncol = p),
           Xr     = rep(0,n),
           KL     = rep(as.numeric(NA),L),
           lbf    = rep(as.numeric(NA),L),
           lbf_variable = matrix(as.numeric(NA),L,p),
           sigma2 = residual_variance,
           V      = scaled_prior_variance*varY,
           pi     = prior_weights)
  if (is.null(null_weight))
    s$null_index = 0
  else
    s$null_index = p
  class(s) = "susie"
  return(s)
}

# Update a susie fit object in order to initialize susie model.
init_finalize = function (s, X = NULL, Xr = NULL) {
  if(length(s$V) == 1)
    s$V = rep(s$V,nrow(s$alpha))

  # Check sigma2.
  if (!is.numeric(s$sigma2))
    stop("Input residual variance sigma2 must be numeric")

  # Avoid problems with dimension if input is a 1 x 1 matrix.
  s$sigma2 = as.numeric(s$sigma2)
  if (length(s$sigma2) != 1)
    stop("Input residual variance sigma2 must be a scalar")
  if (s$sigma2 <= 0)
    stop("Residual variance sigma2 must be positive (is your var(Y) zero?)")

  # check prior variance
  if (!is.numeric(s$V))
    stop("Input prior variance must be numeric")
  if (!all(s$V >= 0))
    stop("prior variance must be non-negative")
  if (!all(dim(s$mu) == dim(s$mu2)))
    stop("dimension of mu and mu2 in input object do not match")
  if (!all(dim(s$mu) == dim(s$alpha)))
    stop("dimension of mu and alpha in input object do not match")
  if (nrow(s$alpha) != length(s$V))
    stop("Input prior variance V must have length of nrow of alpha in ",
         "input object")

  # Update Xr.
  if (!missing(Xr))
    s$Xr = Xr
  if (!missing(X))
    s$Xr = compute_Xb(X,colSums(s$mu * s$alpha))

  # Reset KL and lbf.
  s$KL = rep(as.numeric(NA),nrow(s$alpha))
  s$lbf = rep(as.numeric(NA),nrow(s$alpha))
  class(s) = "susie"
  return(s)
}

#' @title Perform Univariate Linear Regression Separately for Columns of X
#' 
#' @description This function performs the univariate linear
#'   regression y ~ x separately for each column x of X. Each regression
#'   is implemented using \code{.lm.fit()}. The estimated effect size
#'   and stardard error for each variable are outputted.
#' 
#' @param X n by p matrix of regressors.
#' 
#' @param y n-vector of response variables.
#' 
#' @param Z Optional n by k matrix of covariates to be included in all
#'   regresions. If Z is not \code{NULL}, the linear effects of
#'   covariates are removed from y first, and the resulting residuals
#'   are used in place of y.
#' 
#' @param center If \code{center = TRUE}, center X, y and Z.
#' 
#' @param scale If \code{scale = TRUE}, scale X, y and Z.
#' 
#' @param return_residuals Whether or not to output the residuals if Z
#'   is not \code{NULL}.
#'
#' @return A list with two vectors containing the least-squares
#'   estimates of the coefficients (\code{betahat}) and their standard
#'   errors (\code{sebetahat}). Optionally, and only when a matrix of
#'   covariates \code{Z} is provided, a third vector \code{residuals}
#'   containing the residuals is returned.
#' 
#' @examples
#' set.seed(1)
#' n = 1000
#' p = 1000
#' beta = rep(0,p)
#' beta[1:4] = 1
#' X = matrix(rnorm(n*p),nrow = n,ncol = p)
#' X = scale(X,center = TRUE,scale = TRUE)
#' y = drop(X %*% beta + rnorm(n))
#' res = univariate_regression(X,y)
#' plot(res$betahat/res$sebetahat)
#' 
#' @importFrom stats lm
#' @importFrom stats .lm.fit
#' @importFrom stats coef
#' @importFrom stats summary.lm
#'
#' @export
#' 
univariate_regression = function (X, y, Z = NULL, center = TRUE,
                                  scale = FALSE, return_residuals = FALSE) {
  y_na = which(is.na(y))
  if (length(y_na)) {
    X = X[-y_na,]
    y = y[-y_na]
  }
  if (center) {
    y = y - mean(y)
    X = scale(X,center = TRUE,scale = scale)
  } else 
    X = scale(X,center = FALSE,scale = scale)
  X[is.nan(X)] = 0
  if (!is.null(Z)) {
    if (center)
      Z = scale(Z,center = TRUE,scale = scale)
    y = .lm.fit(Z,y)$residuals
  }
  output = try(do.call(rbind,
                       lapply(1:ncol(X), function (i) {
                         g = .lm.fit(cbind(1,X[,i]),y)
                         return(c(coef(g)[2],calc_stderr(cbind(1,X[,i]),
                                                         g$residuals)[2]))
                       })),
               silent = TRUE)
  
  # Exception occurs, fall back to a safer but slower calculation.
  if (inherits(output,"try-error")) {
    output = matrix(0,ncol(X),2)
    for (i in 1:ncol(X)) {
      fit = summary(lm(y ~ X[,i]))$coef
      if (nrow(fit) == 2)
        output[i,] = as.vector(summary(lm(y ~ X[,i]))$coef[2,1:2])
      else
        output[i,] = c(0,0)
    }
  }
  if (return_residuals && !is.null(Z)) 
    return(list(betahat = output[,1],sebetahat = output[,2],residuals = y))
  else
    return(list(betahat = output[,1],sebetahat = output[,2]))
}

# Computes the z-scores (t-statistics) for association between Y and
# each column of X.
calc_z = function (X, Y, center = FALSE, scale = FALSE) {
  univariate_z = function(X,Y,center,scale) {
    out = univariate_regression(X,Y,center = center,scale = scale)
    return(out$betahat/out$sebetahat)
  }
  if (is.null(dim(Y)))
    return(univariate_z(X,Y,center,scale))
  else
    return(do.call(cbind,lapply(1:ncol(Y),
                                function(i) univariate_z(X,Y[,i],
                                                         center = center,
                                                         scale = scale))))
}

context("test_sparse_set_X_attributes.R")

test_that("sparse version set_X_attributes", with(simulate(sparse=T), {
  
  dense.res = set_X_attributes(X)
  sparse.res = set_X_attributes(X.sparse)
  
  expect_equal(sparse.res, X.sparse, check.attributes=FALSE)
  expect_equal(attr(dense.res, 'scaled:center'), attr(sparse.res, 'scaled:center'))
  expect_equal(attr(dense.res, 'scaled:scale'), attr(sparse.res, 'scaled:scale'))
  expect_equal(attr(dense.res, 'd'), attr(sparse.res, 'd'))
}))

#!/bin/bash
#
# This script will be executed every time you run "git commit". It
# will update the 4th digit of package version by revision number.
#
# To use this script, copy it to the .git/hooks directory of your
# local repository to filename `pre-commit`, and make it executable.
#
ROOT_DIR=`git rev-parse --show-toplevel`
MSG="[WARNING] Auto-versioning disabled because string 'Version: x.y.z.r' cannot be found in DESCRIPTION file."
GREP_REGEX='^Version: [0-9]*\.[0-9]*\.[0-9]*\.[0-9]*'
SED_REGEX='^Version: \([0-9]*\.[0-9]*\.[0-9]*\)\.[0-9]*'
# `git diff HEAD` shows both staged and unstaged changes
if [[ -z `git diff HEAD` ]] || [[ ! -f $ROOT_DIR/DESCRIPTION ]]; then
    exit 0
elif [[ -z `grep "$GREP_REGEX" $ROOT_DIR/DESCRIPTION` ]]; then
    echo -e "\e[1;31m$MSG\e[0m"
else
    REV_ID=`git log --oneline | wc -l`
    REV_ID=`printf "%04d\n" $((REV_ID+1))`
    DATE=`date +%Y-%m-%d`
    echo "Version string bumped to revision $REV_ID on $DATE"
    sed -i "s/$SED_REGEX/Version: \1.$REV_ID/" $ROOT_DIR/DESCRIPTION
    sed -i "s/^Date: .*/Date: $DATE/" $ROOT_DIR/DESCRIPTION
    if [[ `git rev-parse --abbrev-ref HEAD` -eq "master" ]]; then
        cd $ROOT_DIR
        echo "Updating documentation ..."
        Rscript -e 'devtools::document()' &> /dev/null && git add man/*.Rd
        echo "Running unit tests ..."
        Rscript -e 'devtools::test()'
        Rscript -e 'library(susieR); tests = testthat::test_examples(".")'
    fi
fi

#!/bin/bash
#
# This script will be executed every time you run "git commit". It
# will commit changes made to package DESCRIPTION by the pre-commit hook
#
# To use this script, copy it to the .git/hooks directory of your
# local repository to filename `post-commit`, and make it executable.
#
ROOT_DIR=`git rev-parse --show-toplevel`
# Only commit DESCRIPTION file when it is not staged (due to changes by pre-commit hook)
if [[ -z `git diff HEAD` ]] || [[ ! -f $ROOT_DIR/DESCRIPTION ]] || [[ -z `git diff $ROOT_DIR/DESCRIPTION` ]]; then
    exit 0
else
    git add $ROOT_DIR/DESCRIPTION
    git commit --amend -C HEAD --no-verify
    echo "Amend current commit to incorporate version bump"
fi

---
title: "test.Rmd"
author: "Matthew Stephens"
date: "4/14/2018"
output: html_document
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = TRUE)
```

# simulate data

This is Lei's example
```{r}
set.seed(777)
library(susieR)
X <- matrix(rnorm(1010 * 1000), 1010, 1000)
beta <- rep(0, 1000)
beta[1 : 200] <- 100
y <- X %*% beta + rnorm(1010)
s = susie(X,y,L=200)

plot(coef(s),beta)
s$sigma2

# fit <- lm(y ~ X - 1)
# mlr.p <- log(summary(fit)$coefficients[, 4])
# 
mar.p <- c()
mar.betahat = c()
for (i in 1 : 1000) {
 fit <- lm(y ~ X[, i] - 1)
  mar.p[i] <- log(summary(fit)$coefficients[, 4])
  mar.betahat[i] <- summary(fit)$coefficients[, 1]
}
# 
# pdf("pvalue.pdf", width = 10, height = 5)
# par(mfrow = c(1, 2))
# plot(mlr.p, ylab = "log(p-value)", main = "Multiple Linear Regression")
# abline(h = log(0.05 / 1000), lty = 2, col = "red")
# legend("right", lty = 2, col = "red", "log(0.05/p)")
# 
# plot(mar.p, ylab = "log(p-value)", main = "One-on-One Linear Regression")
# abline(h = log(0.05 / 1000), lty = 2, col = "red")
```

Notice that the coefficients are monotonic with betahat. Some shrinkage of zero values is evident, but it is not enough... presumably because sigma2 is way over-estimated. And further we see excessive shrinkage of true signals, presumably because sa2 is too small.
```{r}
plot(coef(s),mar.betahat)
```


Here we try fixing $L$ and residual variance to true value.
```{r}
strue = susie(X,y,L=200,residual_variance =1,estimate_residual_variance =FALSE)
plot(coef(strue),beta)
strue$elbo
```
it works!!

```{r}
plot(strue$alpha[1,])
plot(strue$alpha[2,])
```

Try with very small residual variance
```{r}
s3 = susie(X,y,L=200, residual_variance = 0.01,estimate_residual_variance = FALSE)
plot(coef(s3))
s4 = susie(X,y,s_init = s3)
plot(coef(s4))
s4$elbo
```
That is weird it goes away from the solution!

Try with estimating prior:
```{r}
s5 = susie(X,y,s_init = s3, estimate_prior_variance = TRUE)
plot(coef(s5))
s5$elbo
sqrt(s5$sa2)
sqrt(s4$sa2)
```
much better!


Now try too many effects
```{r}
s3.300 = susie(X,y,L=300, residual_variance = 0.01,estimate_residual_variance = FALSE)
s5.300 = susie(X,y,s_init = s3.300, estimate_prior_variance = TRUE)
plot(coef(s3.300))
plot(coef(s5.300))
s3.300$elbo
s5.300$elbo
sum(s5.300$sa2>0)
```

Now try too many effects but just a very small number of iterations for initial case:
```{r}
s3.300.5 = susie(X,y,L=300, residual_variance = 0.01,estimate_residual_variance = FALSE, max_iter = 5)
s5.300.5 = susie(X,y,s_init = s3.300.5, estimate_prior_variance = TRUE)
plot(coef(s3.300.5))
plot(coef(s5.300.5))
s3.300.5$elbo
s5.300.5$elbo
plot(colSums(s5.300.5$alpha))
```

Q: does the initial run with small variance gradually find the smaller effects, or does it get them from the first iteration?
Could look at that by doing one iteration at a time.
```{r}
s3.300.1 = susie(X,y,L=300, residual_variance = 0.01,estimate_residual_variance = FALSE, max_iter = 1)
plot(coef(s3.300.1))
s3.300.2 = susie(X,y,s_init=s3.300.1,estimate_residual_variance = FALSE, max_iter = 1)
plot(coef(s3.300.2))
s3.300 = susie(X,y,s_init=s3.300.2,estimate_residual_variance = TRUE)

```

---
title: "optimize"
author: "Matthew Stephens"
date: "4/15/2018"
output: html_document
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = TRUE)
```

Diagnose optimization issues with Lei's example
```{r}
set.seed(777)
devtools::load_all(".")
X <- matrix(rnorm(1010 * 1000), 1010, 1000)
beta <- rep(0, 1000)
beta[1 : 200] <- 100
y <- X %*% beta + rnorm(1010)
s = susie(X,y,L=1,estimate_residual_variance = TRUE)
Y = y-s$Xr
s2 = s$sigma2
x = seq(1,100000,length=100)
l  = rep(0,100)
lg = rep(0,100)
for(i in 1:100){
  l[i] = loglik(x[i],Y,X,s2)
  lg[i] = loglik.grad(x[i],Y,X,s2)
}
plot(x,l)
plot(x,lg)
# > which.max(l)
# [1] 23
# > lg[23]
# [1] -2.398905e-07
# > lg[22]
# [1] 6.282734e-07

lx = log(x)
l2=l
lg2=lg
for(i in 1:100){
  l2[i] = negloglik.logscale(lx[i],Y,X,s2)
  lg2[i] = negloglik.grad.logscale(lx[i],Y,X,s2)
}
plot(lx,l2)
plot(lx,lg2)

y = seq(0,20,length=100)
l3=l2
lg3=lg2
for(i in 1:100){
  l3[i] = negloglik.logscale(y[i],Y,X,s2)
  lg3[i] = negloglik.grad.logscale(y[i],Y,X,s2)
}
plot(y,l3)
plot(y,lg3)
uniroot(negloglik.grad.logscale,c(-20,20),extendInt = "upX",Y=Y,X=X,s2=s2)

```

So, to summarize, problem seems to be that optim has issues with
very flat initial gradient near 0.

---
title: "A minimal example"
author: "Matthew Stephens"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{minimal example}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(collapse = TRUE,comment = "#",fig.width = 4.5,
                      fig.height = 3,fig.align = "center",
                      fig.cap = "&nbsp;",dpi = 120)
```

In this short vignette, we fit a sparse linear regression model with
up to $L > 0$ non-zero effects. Generally, there is no harm in
over-stating $L$ (that is, the method is pretty robust to overfitting),
except that computation will grow as $L$ grows.

Here is a minimal example:

```{r}
library(susieR)
set.seed(1)
n    <- 1000
p    <- 1000
beta <- rep(0,p)
beta[c(1,2,300,400)] <- 1
X   <- matrix(rnorm(n*p),nrow=n,ncol=p)
y   <- X %*% beta + rnorm(n)
res <- susie(X,y,L=10)
plot(coef(res)[-1],pch = 20)
```

Plot the ground-truth outcomes vs. the predicted outcomes:

```{r fig.height=3.5, fig.width=3.5}
plot(y,predict(res),pch = 20)
```

## Session information

Here are some details about the computing environment, including the
versions of R, and the R packages, used to generate these results.

```{r}
sessionInfo()
```

---
title: "Diagnostic for fine-mapping with summary statistics"
author: "Yuxin Zou"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Diagnostic for fine-mapping with summary statistics}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(collapse = TRUE,comment = "#",fig.width = 5,
                      fig.height = 3,fig.align = "center",
                      fig.cap = "&nbsp;",dpi = 120)
```

This vignette demonstrates the use of the diagnostic plot for
assessing consistency of the summary statistics and the reference LD
matrix.

The `susie_rss` assumes the LD matrix accurately estimate the
correlations among SNPs from the original GWAS genotype
data. Typically, the LD matrix comes from some public database of
genotypes in a suitable reference population. The inaccurate LD
information leads to unreliable fine-mapping result.

The diagnostic for consistency between summary statistics and
refenrence LD matrix is based on the RSS model under the null with
regularized LD matrix.
$$
\hat{z} | R, \lambda \sim N(0, (1-\lambda)R + \lambda I), 0<\lambda<1
$$
The parameter $\lambda$ is estimated by maximum likelihood. A larger
$\lambda$ means a greater inconsistency between summary statistics and
the LD matrix. The expected z score is computed for each SNP,
$E(\hat{z}_j | \hat{z}_{-j})$, and plotted against the observed z
scores.

```{r}
library(susieR)
library(curl)
```

## LD information from the original genotype data

We demonstrate the diagnostic plot in a simple case, the LD matrix is
estimated from the original genotype data. In this case, we expect the
diagnostic plot to confirm that the LD matrix is consistent with the
z scores.

We use the same simulated data as in
[fine mapping vignette](finemapping.html).

```{r}
data("N3finemapping")
n = nrow(N3finemapping$X)
b = N3finemapping$true_coef[,1]
sumstats <- univariate_regression(N3finemapping$X, N3finemapping$Y[,1])
z_scores <- sumstats$betahat / sumstats$sebetahat
Rin = cor(N3finemapping$X)
attr(Rin, "eigen") = eigen(Rin, symmetric = TRUE)
susie_plot(z_scores, y = "z", b=b)
```

The estimated $\lambda$ is
```{r}
lambda = estimate_s_rss(z_scores, Rin, n=n)
lambda
```

The plot for the observed z scores vs the expected z scores is 
```{r}
condz_in = kriging_rss(z_scores, Rin, n=n)
condz_in$plot
```

Summary of SuSiE Credible Sets:
```{r}
fit <- susie_rss(z_scores, Rin, n=n, estimate_residual_variance = TRUE)
susie_plot(fit,y = "PIP", b=b)
```

## LD information from the reference panel

We use another simulated data where the LD matrix is estimated from a
reference panel. In this example data set, there is one association
signal in the simulated data (red point), and there is one SNP with
mismatched reference and alternative allele between summary statistics
and the reference panel (yellow point).

**Note:** In some versions of [PLINK][plink], these mismatches can
occur when [PLINK automatically flips the alleles to make the minor
allele be the effect
allele](https://github.com/stephenslab/susieR/issues/148), leading to
different allele encodings in the z scores and LD matrix. Adding the
flag `--keep-allele-order` will disable this behaviour in PLINK.

```{r}
data_file <- tempfile(fileext = ".RData")
data_url <- paste0("https://raw.githubusercontent.com/stephenslab/susieR/",
                   "master/inst/datafiles/SummaryConsistency1k.RData")
curl_download(data_url,data_file)
load(data_file)
zflip = SummaryConsistency$z
ld = SummaryConsistency$ldref
n=10000
b = numeric(length(zflip))
b[SummaryConsistency$signal_id] = zflip[SummaryConsistency$signal_id]
plot(zflip, pch = 16, col = "#767676", main = "Marginal Associations", 
     xlab="SNP", ylab = "z scores")
points(SummaryConsistency$signal_id, zflip[SummaryConsistency$signal_id], col=2, pch=16)
points(SummaryConsistency$flip_id, zflip[SummaryConsistency$flip_id], col=7, pch=16)
```
Using the data with misaligned allele, SuSiE-RSS identifies a true positive CS containing the true effect SNP; and a false positive CS that incorrectly contains the mismatched SNP.
```{r}
fit = susie_rss(zflip, ld, n=n)
susie_plot(fit, y='PIP', b=b)
points(SummaryConsistency$flip_id, fit$pip[SummaryConsistency$flip_id], col=7, pch=16)
```

The estimated $\lambda$ is
```{r}
lambda = estimate_s_rss(zflip, ld, n=n)
lambda
```

In the diagnostic plot, the mismatched SNP shows the largest difference between observed and expected z-scores, and therefore appears furthest away from the diagonal.

```{r}
condz = kriging_rss(zflip, ld, n=n)
condz$plot
```

After fixing the allele encoding, SuSiE-RSS identifies a single true positive CS containing the true-effect SNP, and the formerly mismatched SNP is (correctly) not included in a CS.

```{r}
z = zflip
z[SummaryConsistency$flip_id] = -z[SummaryConsistency$flip_id]
fit = susie_rss(z, ld, n=n)
susie_plot(fit, y='PIP', b=b)
```

## Session information

Here are some details about the computing environment, including the
versions of R, and the R packages, used to generate these results.

```{r session-info}
sessionInfo()
```

[plink]: https://www.cog-genomics.org/plink

---
title: "Refine SuSiE model"
author: "Yuxin Zou"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Refine SuSiE model}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(collapse = TRUE,comment = "#",fig.width = 5,
                      fig.height = 3,fig.align = "center",
                      fig.cap = "&nbsp;",dpi = 120)
```

In this vignette, we demonstrate a procedure that helps SuSiE get out of local optimum.

We simulate phenotype using UK Biobank genotypes from 50,000 individuals. There are 1001 SNPs.
It is simulated to have exactly 2 non-zero effects at 234, 287.

```{r}
library(susieR)
library(curl)
data_file <- tempfile(fileext = ".RData")
data_url <- paste0("https://raw.githubusercontent.com/stephenslab/susieR/",
                   "master/inst/datafiles/FinemappingConvergence1k.RData")
curl_download(data_url,data_file)
load(data_file)
b <- FinemappingConvergence$true_coef
susie_plot(FinemappingConvergence$z, y = "z", b=b)
```

The strongest marginal association is a non-effect SNP. 

Since the sample size is large, we use sufficient statistics ($X^\intercal X, X^\intercal y, y^\intercal y$ and sample size $n$) to fit susie model. It identifies 2 Credible Sets, one of them is false positive. This is because `susieR` get stuck around a local minimum. 

```{r}
fitted <- with(FinemappingConvergence,
               susie_suff_stat(XtX = XtX, Xty = Xty, yty = yty, n = n))
susie_plot(fitted, y="PIP", b=b, main=paste0("ELBO = ", round(susie_get_objective(fitted),2)))
```

Our refine procedure to get out of local optimum is

1. fit a susie model, $s$ (suppose it has $K$ CSs).

2. for CS in $s$, set SNPs in CS to have prior weight 0, fit susie model --> we have K susie models: $t_1, \cdots, t_K$.

3. for each $k = 1, \cdots, K$, fit susie with initialization at $t_k$ ($\alpha, \mu, \mu^2$) --> $s_k$

4. if $\max_k \text{elbo}(s_k) > \text{elbo}(s)$, set $s = s_{kmax}$ where $kmax = \arg_k \max \text{elbo}(s_k)$ and go to step 2; if no, break.

We fit susie model with above procedure by setting `refine = TRUE`.

```{r}
fitted_refine <- with(FinemappingConvergence,
                      susie_suff_stat(XtX = XtX, Xty = Xty, yty = yty,
					                  n = n, refine=TRUE))
susie_plot(fitted_refine, y="PIP", b=b, main=paste0("ELBO = ", round(susie_get_objective(fitted_refine),2)))
```

With the refine procedure, it identifies 2 CSs with the true signals, and the achieved evidence lower bound (ELBO) is higher.

## Session information

Here are some details about the computing environment, including the
versions of R, and the R packages, used to generate these results.

```{r}
sessionInfo()
```

---
title: "Test susie auto"
author: "Matthew Stephens"
date: "5/2/2018"
output: html_document
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = TRUE)
```

The goal here is to test the function `susie_auto` which tries
to make susie run well even in some tricky situations (eg where $L$ is big,
which susie is not well suited to).

This is an example Lei Sun showed us from the paper demonstrating that
false positives occur early on the Lasso path. Here I change L to 20 instead
of 200 to make it run fast to begin with.
```{r}
set.seed(777)
library(susieR)
L <- 20
X <- matrix(rnorm(1010 * 1000), 1010, 1000)
beta <- rep(0, 1000)
beta[1 : L] <- 100
y <- X %*% beta + rnorm(1010)
```

```{r}
s <- susie_auto(X,y,verbose=TRUE)
s$sa2
```


Now try L=200
```{r}
set.seed(777)
L <- 200
X <- matrix(rnorm(1010 * 1000), 1010, 1000)
beta <- rep(0, 1000)
beta[1 : L] <- 100
y <- X %*% beta + rnorm(1010)
s2 <- susie_auto(X,y,verbose=TRUE)
s2$sa2
```

---
title: "Compare susie_rss variants"
author: "Peter Carbonetto"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Compare susie_rss variants}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

In this vignette, we briefly illustrate the different ways
[susie_rss][susie_rss] can be called, and draw connections between
running `susie_rss` on summary data, and running `susie` on
individual-level data.

```{r setup, include=FALSE}
knitr::opts_chunk$set(collapse = TRUE,comment = "#",fig.width = 5,
                      fig.height = 3,fig.align = "center",
                      dpi = 120)
```

```{r load-pkgs}
library(susieR)
```

Simulate a data set with 200 samples and 1,000 variables, in which the
only first 4 variables affect the outcome.

```{r simdata}
set.seed(1)
n <- 200
p <- 1000
beta <- rep(0,p)
beta[1:4] <- 1
X <- matrix(rnorm(n*p),nrow = n,ncol = p)
X <- scale(X,center = TRUE,scale = FALSE)
y <- drop(X %*% beta + rnorm(n))
```

Compute summary statistics $\hat{b}_j, \hat{s}_j$ and the correlation
matrix, ${\bf R}$. These quantities will be provided as input to
susie_rss.

```{r sumstats-no-standardize}
ss  <- univariate_regression(X,y)
dat <- compute_suff_stat(X,y,standardize = FALSE)
R   <- cov2cor(dat$XtX)
```

The susie and susie_rss analyses produce the exact same results when
the summary statistics `bhat`, `shat`, `var_y` and `n` are provided to
susie_rss (and when `R` is an "in sample" correlation estimate---that
is, when it was computed from the same matrix `X` that was used to
obtain the other statistics). If the covariate effects are removed from 
the genotypes in univariate regression, the in-sample LD matrix should 
compute from the genotype residuals where the covariate effects have 
been removed.

```{r first-comparison, fig.height=3.5, fig.width=3}
res1 <- susie(X,y,L = 10)
res2 <- susie_rss(bhat = ss$betahat,shat = ss$sebetahat,R = R,n = n,
                  var_y = var(y),L = 10,estimate_residual_variance = TRUE)
plot(coef(res1),coef(res2),pch = 20,xlab = "susie",ylab = "susie_rss")
abline(a = 0,b = 1,col = "skyblue",lty = "dashed")
```

When some but not all of these statistics are provided, the results
should be similar, but not exactly the same.

Next let's compare the susie and susie_rss outputs when ${\bf X},
y$ are *standardized* before computing the summary statistics (by
"standardize", we mean that $y$ and the columns of ${\bf X}$ are each
divided by the sample standard deviation so that they each have the
same standard deviation).

```{r sumstats-standardize-1}
ss  <- univariate_regression(scale(X),scale(y))
dat <- compute_suff_stat(X,y,standardize = TRUE)
R   <- cov2cor(dat$XtX)
```

Then we compute the *z*-scores:

```{r sumstats-standardize-2}
zhat <- ss$betahat/ss$sebetahat
```

When standardizing, providing susie_rss with summary data `z` (or
`bhat`, `shat`), `R` and `n` is sufficient for susie_rss to recover
the same results as susie:

```{r second-comparison, fig.height=3.5, fig.width=6, message=FALSE}
res1 <- susie(scale(X),scale(y),L = 10)
res2 <- susie_rss(bhat = ss$betahat,shat = ss$sebetahat,R = R,n = n,
                  L = 10,estimate_residual_variance = TRUE)
res3 <- susie_rss(zhat,R,n = n,L = 10,estimate_residual_variance = TRUE)
layout(matrix(1:2,1,2))
plot(coef(res1),coef(res2),pch = 20,xlab = "susie",
          ylab = "susie_rss(bhat,shat)")
abline(a = 0,b = 1,col = "skyblue",lty = "dashed")
plot(coef(res1),coef(res3),pch = 20,xlab = "susie",ylab = "susie_rss(z)")
abline(a = 0,b = 1,col = "skyblue",lty = "dashed")
```

When the residual variance is not estimated in susie_rss, the
susie_rss results may be close to susie, but may no longer be
exactly the same:

```{r third-comparison, fig.height=3.5, fig.width=3, message=FALSE}
res4 <- susie_rss(zhat,R,n = n,L = 10)
plot(coef(res1),coef(res4),pch = 20,xlab = "susie",ylab = "susie_rss")
abline(a = 0,b = 1,col = "skyblue",lty = "dashed")
```

Whenever `R` is an "in sample" correlation matrix, we recommend
estimating the residual variance.

Without providing the sample size, `n`, the coefficients are
interpreted as the "noncentrality parameters" (NCPs), and are
(roughly) related to the susie parameters by a factor of $\sqrt{n}$:

```{r fourth-comparison, fig.height=3.5, fig.width=3, message=FALSE, warning=FALSE}
res5 <- susie_rss(zhat,R,L = 10)
plot(coef(res1),coef(res5)/sqrt(n),pch = 20,xlab = "susie",
ylab = "susie_rss/sqrt(n)")
abline(a = 0,b = 1,col = "skyblue",lty = "dashed")
```

Whenever possible, the sample size, or a reasonable estimate of the
sample size, should be provided.

[susie_rss]: https://stephenslab.github.io/susieR/reference/susie_rss.html

---
title: "Evaluation of sparse version of SuSiE"
author: "Kaiqian Zhang"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{SuSiE with sparse matrix operations}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(collapse = TRUE,comment = "#",fig.width = 4.5,
                      fig.height = 3,fig.align = "center",
                      fig.cap = "&nbsp;",dpi = 120)
```

## Set up environment

```{r, warning=FALSE}
library(Matrix)
library(susieR)
set.seed(1)
```

## Overview

In this vignette, we provide line profiles for revised version SuSiE,
which allows for a sparse matrix structure. We compare speed
performance when the form of the matrix `X` is dense and sparse.

In this minimal example, we observe that given a large sparse matrix,
if it is in the dense form, the speed is around `40%` slower than that
in a sparse form.

## Simulate data

We randomly simulate a `n=1000` by `p=1000` dense matrix and a sparse
matrix at sparsity $99\%$, i.e. $99\%$ entries are zeros.

```{r}
create_sparsity_mat = function(sparsity, n, p) {
  nonzero          <- round(n*p*(1-sparsity))
  nonzero.idx      <- sample(n*p, nonzero)
  mat              <- numeric(n*p)
  mat[nonzero.idx] <- 1
  mat              <- matrix(mat, nrow=n, ncol=p)
  return(mat)
}
```

```{r}
n <- 1000
p <- 1000
beta <- rep(0,p)
beta[c(1,300,400,1000)] <- 10 
X.dense  <- create_sparsity_mat(0.99,n,p)
X.sparse <- as(X.dense,"CsparseMatrix")
y <- c(X.dense %*% beta + rnorm(n))
```

## `X` in a dense form

```{r}
susie.dense <- susie(X.dense,y)
```

## `X` in a sparse form

```{r}
susie.sparse <- susie(X.sparse,y)
```

## Further step

We encourage people who are insterested in improving SuSiE can get
insights from those line profiles provided.

---
title: "Fine-mapping with summary statistics"
author: "Yuxin Zou and Gao Wang"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Fine-mapping with summary statistics}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(collapse = TRUE,comment = "#",fig.width = 5,
                      fig.height = 3,fig.align = "center",
                      fig.cap = "&nbsp;",dpi = 120)
```

This vignette demonstrates how to use `susieR` with "summary
statistics" in the context of genetic fine-mapping.  We use the same
simulated data as in [fine mapping vignette](finemapping.html). The
simulated data are expression levels of a gene ($y$) in $N \approx 600$
individuals.  We want to identify with the genotype matrix $X_{N\times
P}$ ($P=1001$) the genetic variables that causes changes in expression
level. This data set is shipped with `susieR`. It is simulated to have
exactly three non-zero effects.

```{r}
library(susieR)
set.seed(1)
```

## The data-set

```{r}
data(N3finemapping)
attach(N3finemapping)
n = nrow(X)
```

Notice that we've simulated two sets of $Y$ as two simulation
replicates. Here we'll focus on the first data-set.

```{r}
dim(Y)
```

Here are the three true signals in the first data-set:

```{r}
b <- true_coef[,1]
plot(b, pch=16, ylab='effect size')
```

```{r}
which(b != 0)
```

So the underlying causal variables are 403, 653 and 773.

## Summary statistics from simple regression

Summary statistics of genetic association studies typically contain
effect sizes ($\hat{\beta}$ coefficient from regression) and p-values.
These statisticscan be used to perform fine-mapping with given an
additional input of correlation matrix between variables.  The
correlation matrix in genetics is typically referred to as an "LD
matrix" (LD is short for linkage disequilibrium). One may use external
reference panels to estimate it when this matrix cannot be obtained
from samples directly. *Importantly, the LD matrix has to be a matrix
containing estimates of the correlation, $r$, and not $r^2$ or $|r|$.*
See also [this vignette](susierss_diagnostic.html) for how to check the
consistency of the LD matrix with the summary statistics.

The `univariate_regression` function can be used to compute summary
statistics by fitting univariate simple regression variable by
variable.  The results are $\hat{\beta}$ and $SE(\hat{\beta})$ from
which z-scores can be derived. Alternatively, you can obtain z-scores
from $\hat{\beta}$ and p-values if you are provided with those
information. For example,

```{r}
sumstats <- univariate_regression(X, Y[,1])
z_scores <- sumstats$betahat / sumstats$sebetahat
susie_plot(z_scores, y = "z", b=b)
```

For this example, the correlation matrix can be computed directly from
data provided:

```{r}
R <- cor(X)
```

## Fine-mapping with `susieR` using summary statistics

By default, SuSiE assumes at most 10 causal variables, with `L =
10`, although we note that SuSiE is generally robust to the choice of
`L`.

Since the individual-level data is available for us here, we can
easily compute the "in-sample LD" matrix, as well as the variance of
$y$, which is `r round(var(Y[,1]),digits = 4)`.  (By "in-sample", we
means the LD was computed from the exact same matrix `X` that was used
to obtain the other statistics.) When we fit SuSiE regression with
summary statistics, $\hat{\beta}$, $SE(\hat{\beta})$, $R$, $n$, and
var_y these are also the *sufficient statistics.* With an in-sample
LD, we can also estimate the residual variance using these sufficient
statistics. (Note that if the covariate effects are removed from the
genotypes in the univariate regression, it is recommended that the
"in-sample" LD matrix also be computed from the genotypes after the
covariate effects have been removed.)

```{r}
fitted_rss1 <- susie_rss(bhat = sumstats$betahat, shat = sumstats$sebetahat, n = n, R = R, var_y = var(Y[,1]), L = 10,
                         estimate_residual_variance = TRUE)
```

Using `summary`, we can examine the posterior inclusion probability
(PIP) for each variable, and the 95% credible sets:

```{r}
summary(fitted_rss1)$cs
```

The three causal signals have been captured by the three CSs. Note the
third CS contains many variables, including the true causal
variable 403.

We can also plot the posterior inclusion probabilities (PIPs):

```{r}
susie_plot(fitted_rss1, y="PIP", b=b)
```

The true causal variables are shown in red. The 95% CSs are
shown by three different colours (green, purple, blue).

Note this result is *exactly the same* as running `susie` on the
original individual-level data:

```{r}
fitted = susie(X, Y[,1], L = 10)
all.equal(fitted$pip, fitted_rss1$pip)
all.equal(coef(fitted)[-1], coef(fitted_rss1)[-1])
```

If, on the other hand, the variance of $y$ is unknown, we fit can SuSiE
regression with summary statistics, $\hat{\beta}$, $SE(\hat{\beta})$,
$R$ and $n$ (or *z*-scores, $R$ and $n$). The outputted effect estimates
are now on the standardized $X$ and $y$ scale. Still, we can estimate the
residual variance because we have the in-sample LD matrix:

```{r}
fitted_rss2 = susie_rss(z = z_scores, R = R, n = n, L = 10,
                        estimate_residual_variance = TRUE)
```

The result is same as if we had run `susie` on the individual-level
data, but the output effect estimates are on different scale:

```{r, fig.height=4, fig.width=3.5}
all.equal(fitted$pip, fitted_rss2$pip)
plot(coef(fitted)[-1], coef(fitted_rss2)[-1], xlab = 'effects from SuSiE', ylab = 'effects from SuSiE-RSS', xlim=c(-1,1), ylim=c(-0.3,0.3))
```

Specifically, without the variance of $y$, these estimates are same as
if we had applied SuSiE to a standardized $X$ and $y$; that is, as if
$y$ and the columns of $X$ had been normalized so that $y$ and each
column of $X$ had a standard deviation of 1.

```{r}
fitted_standardize = susie(scale(X), scale(Y[,1]), L = 10)
all.equal(coef(fitted_standardize)[-1], coef(fitted_rss2)[-1])
```


## Fine-mapping with `susieR` using LD matrix from reference panel

When the original genotypes are not available, one may use a separate
reference panel to estimate the LD matrix.

To illustrate this, we randomly generated 500 samples from $N(0,R)$
and treated them as reference panel genotype matrix `X_ref`.

```{r echo=F}
set.seed(1)
tmp = matrix(rnorm(500*1001), 500, 1001)
eigenR = eigen(R)
eigenR$values[eigenR$values < 1e-10] = 0
X_ref = tmp %*% (sqrt(eigenR$values) * t(eigenR$vectors))
R_ref = cor(X_ref)
```

We fit the SuSiE regression using out-of sample LD matrix. The
residual variance is fixed at 1 because estimating residual variance
sometimes produces very inaccurate estimates with out-of-sample LD
matrix. The output effect estimates are on the standardized $X$ and
$y$ scale.

```{r}
fitted_rss3 <- susie_rss(z_scores, R_ref, n=n, L = 10)
```

```{r}
susie_plot(fitted_rss3, y="PIP", b=b)
```

In this particular example, the SuSiE result with out-of-sample LD is
very similar to using the in-sample LD matrix because the LD matrices
are quite similar.

```{r, fig.width=3.5,fig.height=4}
plot(fitted_rss1$pip, fitted_rss3$pip, ylim=c(0,1), xlab='SuSiE PIP', ylab='SuSiE-RSS PIP')
```

In some rare cases, the sample size $n$ is unknown. When the sample
size is not provided as input to `susie_rss`, this is in effect
assuming the sample size is infinity and all the effects are small,
and the estimated PVE for each variant will be close to zero. The
outputted effect estimates are on the "noncentrality parameter" scale.

```{r}
fitted_rss4 = susie_rss(z_scores, R_ref, L = 10)
susie_plot(fitted_rss4, y="PIP", b=b)
```

## Session information

Here are some details about the computing environment, including the
versions of R, and the R packages, used to generate these results.

```{r}
sessionInfo()
```

---
title: "Fine-mapping example"
author: "Gao Wang"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Fine-mapping example}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(collapse = TRUE,comment = "#",fig.width = 5,
                      fig.height = 3,fig.align = "center",
                      fig.cap = "&nbsp;",dpi = 120)
```

This vignette demonstrates `susieR` in the context of genetic
fine-mapping.  We use simulated data of expression level of a gene
($y$) in $N \approx 600$ individuals.  We want to identify with the
genotype matrix $X_{N\times P}$ ($P=1001$) the genetic variables that
causes changes in expression level.

The simulated data set is simulated to have exactly 3 non-zero
effects.

```{r}
library(susieR)
set.seed(1)
```

## The data-set

```{r}
data(N3finemapping)
attach(N3finemapping)
```

The loaded dataset contains regression data $X$ and $y$, along with some other
relevant properties in the context of genetic studies. It also
contains the "true" regression coefficent the data is simulated from.

Notice that we've simulated 2 sets of $Y$ as 2 simulation
replicates. Here we'll focus on the first data-set.

```{r}
dim(Y)
```

Here are the 3 "true" signals in the first data-set:

```{r}
b <- true_coef[,1]
plot(b, pch=16, ylab='effect size')
```

```{r}
which(b != 0)
```

So the underlying causal variables are 403, 653 and 773.

## Simple regression summary statistics

`univariate_regression` function can be used to compute 
summary statistics by fitting univariate simple regression variable by variable.
The results are $\hat{\beta}$ and $SE(\hat{\beta})$ from which z-scores can be
derived. Again we focus only on results from the first data-set:

```{r}
sumstats <- univariate_regression(X, Y[,1])
z_scores <- sumstats$betahat / sumstats$sebetahat
susie_plot(z_scores, y = "z", b=b)
```

## Fine-mapping with `susieR`

For starters, we assume there are at most 10 causal variables, i.e.,
set `L = 10`, although SuSiE is robust to the choice of `L`.

The `susieR` function call is:

```{r}
fitted <- susie(X, Y[,1],
                L = 10,
		verbose = TRUE)
```

### Credible sets

By default, we output 95% credible set:

```{r}
print(fitted$sets)
```

The 3 causal signals have been captured by the 3 CS reported here. The
3rd CS contains many variables, including the true causal variable
`403`. The minimum absolute correlation is 0.86.

If we use the default 90% coverage for credible sets, we still
capture the 3 signals, but "purity" of the 3rd CS is now 0.91 and size
of the CS is also a bit smaller.

```{r}
sets <- susie_get_cs(fitted,
                     X = X,
	  	     coverage = 0.9,
                     min_abs_corr = 0.1)
```

```{r}
print(sets)
```

### Posterior inclusion probabilities

Previously we've determined that summing over 3 single effect
regression models is approperate for our application. Here we
summarize the variable selection results by posterior inclusion
probability (PIP):

```{r}
susie_plot(fitted, y="PIP", b=b)
```

The true causal variables are colored red. The 95% CS identified are
circled in different colors. Of interest is the cluster around
position 400. The true signal is 403 but apparently it does not have
the highest PIP. To compare ranking of PIP and original z-score in
that CS:

```{r}
i  <- fitted$sets$cs[[3]]
z3 <- cbind(i,z_scores[i],fitted$pip[i])
colnames(z3) <- c('position', 'z-score', 'PIP')
z3[order(z3[,2], decreasing = TRUE),]
```

### Choice of priors

Notice that by default SuSiE estimates prior effect size from data. For
fine-mapping applications, however, we sometimes have knowledge of SuSiE prior effect
size since it is parameterized as percentage of variance explained (PVE) by a non-zero effect, 
which, in the context of fine-mapping, is related to per-SNP heritability. It is possible
to use `scaled_prior_variance` to specify this PVE and explicitly set `estimate_prior_variance=FALSE`
to fix the prior effect to given value.

In this data-set, SuSiE is robust to choice of priors. Here we
set PVE to 0.2,
and compare with previous results:

```{r}
fitted = susie(X, Y[,1],
               L = 10,
               estimate_residual_variance = TRUE, 
               estimate_prior_variance = FALSE, 
               scaled_prior_variance = 0.2)
susie_plot(fitted, y='PIP', b=b)
```

which largely remains unchanged. 

### A note on covariate adjustment

To include covariate `Z` in SuSiE, one approach is to regress it out from both `y`
and `X`, and then run SuSiE on the residuals. The code below illustrates the procedure:

```{r, eval=FALSE}
remove.covariate.effects <- function (X, Z, y) {
  # include the intercept term
  if (any(Z[,1]!=1)) Z = cbind(1, Z)
  A   <- forceSymmetric(crossprod(Z))
  SZy <- as.vector(solve(A,c(y %*% Z)))
  SZX <- as.matrix(solve(A,t(Z) %*% X))
  y <- y - c(Z %*% SZy)
  X <- X - Z %*% SZX
  return(list(X = X,y = y,SZy = SZy,SZX = SZX))
}

out = remove.covariate.effects(X, Z, Y[,1])
fitted_adjusted = susie(out$X, out$y, L = 10)
```

Note that the covariates `Z` should have a column of ones as the first column. If not, the above function `remove.covariate.effects`
will add such a column to `Z` before regressing it out. 
Data will be centered as a result. Also the scale of data is changed after regressing out `Z`. 
This introduces some subtleties in terms of interpreting the results.
For this reason, we provide covariate adjustment procedure as a tip in the documentation
and not part of `susieR::susie()` function. Cautions should be taken when applying this procedure and interpreting the result from it.

## Session information

Here are some details about the computing environment, including the
versions of R, and the R packages, used to generate these results.

```{r}
sessionInfo()
```

[N3finemapping]: https://github.com/stephenslab/susieR/blob/master/inst/datafiles/N3finemapping.rds

---
title: "Sparse matrix multiplication strategy"
author: "Kaiqian Zhang"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Sparse vs. dense matrix operations}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(collapse = TRUE,comment = "#",fig.width = 4.5,
                      fig.height = 3,fig.align = "center",
                      fig.cap = "&nbsp;",dpi = 120)
```

## Set up environment

```{r, warning=FALSE, message=FALSE}
library(susieR)
library(Matrix)
library(microbenchmark)
library(ggplot2)
set.seed(1)
```

## Goal

Our intention is to use sparse matrix multiplications to help
reduce computation time.

## General strategy

Given a large sparse matrix `X`, we want to compute some matrix
multiplications associated with a scaled $\tilde{X}$. We notice
that after scaling, `X` becomes a dense matrix and is not possible for
a sparse matrix multiplication. So we construct formulae to apply
sparse matrix multiplication first on a standardized `X` since
standardization does not affect its sparsity. Then we perform
centering to get the same result.

## Types of matrix multiplications

There are two types of matrix multiplications we want to investigate:

+ Compute $\tilde{X}b$, where $\tilde{X}$ is an n by p scaled matrix
  and $b$ is a p vector.

+ Compute $\tilde{X}^Ty$, where $\tilde{X}$ is an n by p scaled matrix
  and $y$ is an n vector.

## Results

This strategy has a decent performance when computing both
$\tilde{X}b$ and $\tilde{X}^Ty$, compared to simple matrix
multiplication `%*%`.

## Strategy formulae details

### Computing $\boldsymbol{\tilde{X}b}$

Suppose we want to compute $\tilde{X}b$, where $\tilde{X}$ is a scaled
n by p matrix and $b$ is a p vector. Our goal is to express
$\tilde{X}b$ into a term involving unscaled $X$ matrix multiplication
to achieve sparse matrix operation.

\begin{equation}
\begin{aligned}
\tilde{X}b
&= \sum_{j=1}^{p} \tilde{X}_{\cdot j} b_j \\
&= \sum_{j=1}^{p} \frac{X_{\cdot j}-\mu_j}{\sigma_j}b_j \\
&= \sum_{j=1}^{p}\frac{X_{\cdot j}}{\sigma_j}b_j -
   \sum_{j=1}^{p} \frac{\mu_j}{\sigma_j}b_j \\
&= X b / \sigma - \mu^Tb/\sigma,
\end{aligned}
\end{equation}

where $\mu$ is a *p*-vector of column means, and $\sigma$ is a
*p*-vector of column standard deviations.

### Computing $\boldsymbol{\tilde{X}^Ty}$

Suppose we want to compute $\tilde{X}^Ty$, where $\tilde{X}$ is a
scaled n by p matrix and $y$ is an n vector. Similarly, we express
$\tilde{X}^Ty$ using unscaled $X$ so that we can perform sparse matrix
multiplication. We have the following:

\begin{equation}
\begin{aligned}
\tilde{X}^Ty &= \sum_{i=1}^{n} \tilde{X}_{i.}y_i \\
&= \sum_{i=1}^{n} \frac{X_{i.} - \mu}{\sigma}y_i \\
&= \frac{1}{\sigma}\sum_{i=1}^{n}X_{i.}y_i -
   \frac{\mu}{\sigma}\sum_{i=1}^{n} y_i \\
&= \frac{1}{\sigma}(X^Ty) - \frac{\mu}{\sigma}y^T 1,
\end{aligned}
\end{equation}

where $\mu$ is a *p*-vector of column means, and $\sigma$ is a
*p*-vector of columnwise standard deviations.

## Simulations

We simulate an `n = 1000` by `p = 10000` matrix `X` at sparsity
$99\%$, i.e. $99\%$ entries are zeros. We compare results between
normal matrix computation and our sparse strategy as well as comparing
speed using microbenchmark.

```{r}
create_sparsity_mat <- function(sparsity, n, p) {
  nonzero          <- round(n*p*(1-sparsity))
  nonzero.idx      <- sample(n*p, nonzero)
  mat              <- numeric(n*p)
  mat[nonzero.idx] <- 1
  mat              <- matrix(mat, nrow=n,ncol=p)
  return(mat)
}
n <- 1000
p <- 10000
```

```{r}
X.dense  <- create_sparsity_mat(0.99,n,p)
X.sparse <- as(X.dense,"dgCMatrix")
X.tilde  <- susieR:::set_X_attributes(X.dense) #returns a scaled X if input is a dense matrix
X <- susieR:::set_X_attributes(X.sparse) #return an unsacled sparse X if input is a sparse matrix 
                                     #but computes column means and standard deviations
```

```{r}
b <- rnorm(p)
y <- rnorm(n)
```

### Benchmark for computing $\boldsymbol{\tilde{X}b}$

The final results of two methods when computing $\tilde{X}b$ are very
close.

```{r}
res1 <- X.tilde %*% b
res2 <- susieR:::compute_Xb(X,b)
max(abs(res1 - res2))
```

```{r}
compute_Xb_benchmark <- microbenchmark(
  dense  = (use.normal.Xb <- X.tilde%*%b),
  sparse = (use.sparse.Xb <- susieR:::compute_Xb(X,b)),
  times = 20,unit = "s")
```

Our sparse strategy demonstrates an obvious advantage over the normal
matrix multiplication in computing $\tilde{X}b$.

```{r}
autoplot(compute_Xb_benchmark)
```

### Benchmark for computing $\boldsymbol{\tilde{X}^Ty}$

The final results of two methods when computing $\tilde{X}^Ty$ are
almost the same.

```{r}
res3 <- t(X.tilde) %*% y
res4 <- susieR:::compute_Xty(X,y)
max(abs(res3 - res4))
```

```{r}
compute_Xty_benchmark = microbenchmark(
  dense  = (use.normal.Xty <- t(X.tilde)%*%y),
  sparse = (use.sparse.Xty <- susieR:::compute_Xty(X, y)),
  times = 20,unit = "s")
```

Our sparse strategy evidently has a better performance than the normal
method in computing $\tilde{X}^Ty$.

```{r}
autoplot(compute_Xty_benchmark)
```

---
title: "Evaluation of sparse version of SuSiE"
author: "Kaiqian Zhang"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{SuSiE with sparse matrix operations}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(collapse = TRUE,comment = "#",fig.width = 4.5,
                      fig.height = 3,fig.align = "center",
                      fig.cap = "&nbsp;",dpi = 120)
```

## Set up environment

```{r, warning=FALSE}
library(Matrix)
library(susieR)
set.seed(1)
```

## Overview

In this vignette, we provide line profiles for revised version SuSiE,
which allows for a sparse matrix structure. We compare speed
performance when the form of the matrix `X` is dense and sparse.

In this minimal example, we observe that given a large sparse matrix,
if it is in the dense form, the speed is around `40%` slower than that
in a sparse form.

## Simulate data

We randomly simulate a `n=1000` by `p=1000` dense matrix and a sparse
matrix at sparsity $99\%$, i.e. $99\%$ entries are zeros.

```{r}
create_sparsity_mat = function(sparsity, n, p) {
  nonzero          <- round(n*p*(1-sparsity))
  nonzero.idx      <- sample(n*p, nonzero)
  mat              <- numeric(n*p)
  mat[nonzero.idx] <- 1
  mat              <- matrix(mat, nrow=n, ncol=p)
  return(mat)
}
```

```{r}
n <- 1000
p <- 1000
beta <- rep(0,p)
beta[c(1,300,400,1000)] <- 10 
X.dense  <- create_sparsity_mat(0.99,n,p)
X.sparse <- as(X.dense,"dgCMatrix")
y <- c(X.dense %*% beta + rnorm(n))
```

## `X` in a dense form

```{r}
susie.dense <- susie(X.dense,y)
```

## `X` in a sparse form

```{r}
susie.sparse <- susie(X.sparse,y)
```

## Further step

We encourage people who are insterested in improving SuSiE can get
insights from those line profiles provided.

---
title: "Diagnostic for fine-mapping with summary statistics"
author: "Yuxin Zou"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Diagnostic for fine-mapping with summary statistics}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(collapse = TRUE,comment = "#",fig.width = 5,
                      fig.height = 3,fig.align = "center",
                      fig.cap = "&nbsp;",dpi = 120)
```

This vignette demonstrates diagnostic plot for consistency between summary statistics and reference LD matrix. 

The `susie_rss` assumes the LD matrix accurately estimate the correlations among SNPs from the original GWAS genotype data. Typically, the LD matrix comes from some public database of genotypes in a suitable reference population. The inaccurate LD information leads to unreliable fine-mapping result.

The diagnostic for consistency between summary statistics and refenrence LD matrix is based on the RSS model under the null with regularized LD matrix.
$$
\hat{z} | R, \lambda \sim N(0, (1-\lambda)R + \lambda I), 0<\lambda<1
$$
The parameter $\lambda$ is estimated by maximum likelihood. A larger $\lambda$ means a greater inconsistency between summary statistics and the LD matrix. The expected z score is computed for each SNP, $E(\hat{z}_j | \hat{z}_{-j})$, and plotted against the observed z scores.

```{r}
library(susieR)
library(curl)
```

## LD information from the original genotype data

We demonstrate the diagnostic plot in a simple case, the LD matrix is estimated from the original genotype data. We use the same simulated data as in [fine mapping vignette](finemapping.html). 

```{r}
data("N3finemapping")
n = nrow(N3finemapping$X)
b = N3finemapping$true_coef[,1]
sumstats <- univariate_regression(N3finemapping$X, N3finemapping$Y[,1])
z_scores <- sumstats$betahat / sumstats$sebetahat
Rin = cor(N3finemapping$X)
attr(Rin, "eigen") = eigen(Rin, symmetric = TRUE)
susie_plot(z_scores, y = "z", b=b)
```

The estimated $\lambda$ is
```{r}
lambda = estimate_s_rss(z_scores, Rin, n=n)
lambda
```

The plot for the observed z scores vs the expected z scores is 
```{r}
condz_in = kriging_rss(z_scores, Rin, n=n)
condz_in$plot
```

Summary of SuSiE Credible Sets:
```{r}
fit <- susie_rss(z_scores, Rin, n=n, estimate_residual_variance = TRUE)
susie_plot(fit,y = "PIP", b=b)
```

## LD information from the reference panel

We use another simulated data where the LD matrix is estimated from a reference panel. There is one signal in the simulated data (red point). There is one SNP with mismatched reference and alternative allele between summary statistics and the reference panel (yellow point).

```{r}
data_file <- tempfile(fileext = ".RData")
data_url <- paste0("https://raw.githubusercontent.com/stephenslab/susieR/",
                   "master/inst/datafiles/SummaryConsistency1k.RData")
curl_download(data_url,data_file)
load(data_file)
zflip = SummaryConsistency$z
ld = SummaryConsistency$ldref
n=10000
b = numeric(length(zflip))
b[SummaryConsistency$signal_id] = zflip[SummaryConsistency$signal_id]
plot(zflip, pch = 16, col = "#767676", main = "Marginal Associations", 
     xlab="SNP", ylab = "z scores")
points(SummaryConsistency$signal_id, zflip[SummaryConsistency$signal_id], col=2, pch=16)
points(SummaryConsistency$flip_id, zflip[SummaryConsistency$flip_id], col=7, pch=16)
```
Using the data with misaligned allele, SuSiE-RSS identifies a true positive CS containing the true effect SNP; and a false positive CS that incorrectly contains the mismatched SNP.
```{r}
fit = susie_rss(zflip, ld, n=n)
susie_plot(fit, y='PIP', b=b)
points(SummaryConsistency$flip_id, fit$pip[SummaryConsistency$flip_id], col=7, pch=16)
```

The estimated $\lambda$ is
```{r}
lambda = estimate_s_rss(zflip, ld, n=n)
lambda
```

In the diagnostic plot, the mismatched SNP shows the largest difference between observed and expected z-scores, and therefore appears furthest away from the diagonal.

```{r}
condz = kriging_rss(zflip, ld, n=n)
condz$plot
```

After fixing the allele encoding, SuSiE-RSS identifies a single true positive CS containing the true-effect SNP, and the formerly mismatched SNP is (correctly) not included in a CS.

```{r}
z = zflip
z[SummaryConsistency$flip_id] = -z[SummaryConsistency$flip_id]
fit = susie_rss(z, ld, n=n)
susie_plot(fit, y='PIP', b=b)
```

## Session information

Here are some details about the computing environment, including the
versions of R, and the R packages, used to generate these results.

```{r}
sessionInfo()
```

---
title: "Fine-mapping with summary statistics"
author: "Yuxin Zou and Gao Wang"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Fine-mapping with summary statistics}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(collapse = TRUE,comment = "#",fig.width = 5,
                      fig.height = 3,fig.align = "center",
                      fig.cap = "&nbsp;",dpi = 120)
```

This vignette demonstrates how to use `susieR` with "summary
statistics" in the context of genetic fine-mapping.  We use the same
simulated data as in [fine mapping vignette](finemapping.html). The
simulated data are expression levels of a gene ($y$) in $N \approx 600$
individuals.  We want to identify with the genotype matrix $X_{N\times
P}$ ($P=1001$) the genetic variables that causes changes in expression
level. This data set is shipped with `susieR`. It is simulated to have
exactly three non-zero effects.

```{r}
library(susieR)
set.seed(1)
```

## The data-set

```{r}
data(N3finemapping)
attach(N3finemapping)
n = nrow(X)
```

Notice that we've simulated two sets of $Y$ as two simulation
replicates. Here we'll focus on the first data-set.

```{r}
dim(Y)
```

Here are the three true signals in the first data-set:

```{r}
b <- true_coef[,1]
plot(b, pch=16, ylab='effect size')
```

```{r}
which(b != 0)
```

So the underlying causal variables are 403, 653 and 773.

## Summary statistics from simple regression

Summary statistics of genetic association studies typically contain
effect sizes ($\hat{\beta}$ coefficient from regression) and p-values.
These statisticscan be used to perform fine-mapping with given an
additional input of correlation matrix between variables.  The
correlation matrix in genetics is typically referred to as an "LD
matrix" (LD is short for linkage disequilibrium).  One may use
external reference panels to estimate it when this matrix cannot be
obtained from samples directly. Importantly, the LD matrix has to be a
matrix containing estimates of the correlation, $r$, and not $r^2$ or
$|r|$.

The `univariate_regression` function can be used to compute summary
statistics by fitting univariate simple regression variable by
variable.  The results are $\hat{\beta}$ and $SE(\hat{\beta})$ from
which z-scores can be derived. Alternatively, you can obtain z-scores
from $\hat{\beta}$ and p-values if you are provided with those
information. For example,

```{r}
sumstats <- univariate_regression(X, Y[,1])
z_scores <- sumstats$betahat / sumstats$sebetahat
susie_plot(z_scores, y = "z", b=b)
```

For this example, the correlation matrix can be computed directly from
data provided:

```{r}
R <- cor(X)
```

## Fine-mapping with `susieR` using summary statistics

By default, SuSiE assumes at most 10 causal variables, with `L =
10`, although we note that SuSiE is generally robust to the choice of
`L`.

Since the individual-level data is available for us here, we can
easily compute the "in-sample LD" matrix, as well as the variance of
$y$, which is `r round(var(Y[,1]),digits = 4)`.  (By "in-sample", we
means the LD was computed from the exact same matrix `X` that was used
to obtain the other statistics.) When we fit SuSiE regression with
summary statistics, $\hat{\beta}$, $SE(\hat{\beta})$, $R$, $n$, and
var_y these are also the *sufficient statistics.* With an in-sample
LD, we can also estimate the residual variance using these sufficient
statistics. (Note that if the covariate effects are removed from the
genotypes in the univariate regression, it is recommended that the
"in-sample" LD matrix also be computed from the genotypes after the
covariate effects have been removed.)

```{r}
fitted_rss1 <- susie_rss(bhat = sumstats$betahat, shat = sumstats$sebetahat, n = n, R = R, var_y = var(Y[,1]), L = 10,
                         estimate_residual_variance = TRUE)
```

Using `summary`, we can examine the posterior inclusion probability
(PIP) for each variable, and the 95% credible sets:

```{r}
summary(fitted_rss1)$cs
```

The three causal signals have been captured by the three CSs. Note the
third CS contains many variables, including the true causal
variable 403.

We can also plot the posterior inclusion probabilities (PIPs):

```{r}
susie_plot(fitted_rss1, y="PIP", b=b)
```

The true causal variables are shown in red. The 95% CSs are
shown by three different colours (green, purple, blue).

Note this result is *exactly the same* as running `susie` on the
original individual-level data:

```{r}
fitted = susie(X, Y[,1], L = 10)
all.equal(fitted$pip, fitted_rss1$pip)
all.equal(coef(fitted)[-1], coef(fitted_rss1)[-1])
```

If, on the other hand, the variance of &y& is unknown, we fit can SuSiE
regression with summary statistics, $\hat{\beta}$, $SE(\hat{\beta})$,
$R$ and $n$ (or *z*-scores, $R$ and $n$). The outputted effect estimates
are now on the standardized $X$ and $y$ scale. Still, we can estimate the
residual variance because we have the in-sample LD matrix:

```{r}
fitted_rss2 = susie_rss(z = z_scores, R = R, n = n, L = 10,
                        estimate_residual_variance = TRUE)
```

The result is same as if we had run `susie` on the individual-level
data, but the output effect estimates are on different scale:

```{r, fig.height=4, fig.width=3.5}
all.equal(fitted$pip, fitted_rss2$pip)
plot(coef(fitted)[-1], coef(fitted_rss2)[-1], xlab = 'effects from SuSiE', ylab = 'effects from SuSiE-RSS', xlim=c(-1,1), ylim=c(-0.3,0.3))
```

Specifically, without the variance of $y$, these estimates are same as
if we had applied SuSiE to a standardized $X$ and $y$; that is, as if
$y$ and the columns of $X$ had been normalized so that $y$ and each
column of $X$ had a standard deviation of 1.

```{r}
fitted_standardize = susie(scale(X), scale(Y[,1]), L = 10)
all.equal(coef(fitted_standardize)[-1], coef(fitted_rss2)[-1])
```


## Fine-mapping with `susieR` using LD matrix from reference panel

When the original genotypes are not available, one may use a separate
reference panel to estimate the LD matrix.

To illustrate this, we randomly generated 500 samples from $N(0,R)$
and treated them as reference panel genotype matrix `X_ref`.

```{r echo=F}
set.seed(1)
tmp = matrix(rnorm(500*1001), 500, 1001)
eigenR = eigen(R)
eigenR$values[eigenR$values < 1e-10] = 0
X_ref = tmp %*% (sqrt(eigenR$values) * t(eigenR$vectors))
R_ref = cor(X_ref)
```

We fit the SuSiE regression using out-of sample LD matrix. The
residual variance is fixed at 1 because estimating residual variance
sometimes produces very inaccurate estimates with out-of-sample LD
matrix. The output effect estimates are on the standardized $X$ and
$y$ scale.

```{r}
fitted_rss3 <- susie_rss(z_scores, R_ref, n=n, L = 10)
```

```{r}
susie_plot(fitted_rss3, y="PIP", b=b)
```

In this particular example, the SuSiE result with out-of-sample LD is
very similar to using the in-sample LD matrix because the LD matrices
are quite similar.

```{r, fig.width=3.5,fig.height=4}
plot(fitted_rss1$pip, fitted_rss3$pip, ylim=c(0,1), xlab='SuSiE PIP', ylab='SuSiE-RSS PIP')
```

In some rare cases, the sample size $n$ is unknown. When the sampel
size is not provided as input to `susie_rss`, this is in effect
assuming the sample size is infinity and all the effects are small,
and the estimated PVE for each variant will be close to zero. The
outputted effect estimates are on the "noncentrality parameter" scale.

```{r}
fitted_rss4 = susie_rss(z_scores, R_ref, L = 10)
susie_plot(fitted_rss4, y="PIP", b=b)
```

## Session information

Here are some details about the computing environment, including the
versions of R, and the R packages, used to generate these results.

```{r}
sessionInfo()
```

---
title: "Trend filtering"
author: "Matthew Stephens"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Trend filtering demo}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(collapse = TRUE,comment = "#",fig.width = 4.5,
                      fig.height = 3,fig.align = "center",
                      fig.cap = "&nbsp;",dpi = 120)
```

# Introduction

Although we developed SuSiE primarily with the goal of performing
variable selection in highly sparse settings -- and, in particular,
for genetic fine-mapping -- the approach also has considerable
potential for application to other large-scale regression
problems. Here we briefly illustrate this potential by applying it to
a non-parametric regression problem that at first sight seems to be
ill-suited to our approach. In particular, it does not involve strict
sparsity, and the underlying correlation structure of the explanatory
variables is very different from the "blocky" covariance structure of
genetic data that SuSiE was designed for. Nonetheless, we will see
that SuSiE performs well here despite this (partly due to its ability
to capture non-sparse signals via Bayesian Model Averaging).

Specifically, consider the non-parametric regression:
$$y_t = \mu_t + e_t \quad t=1,\dots,T$$
where the goal is to estimate the underlying mean, $\mu_t$, under the assumption that it varies smoothly (or, more precisely, in a spatially-structured way) with $t$.
One very simple way to capture spatial structure in $\mu$ is to model it
as a (sparse) linear combination of step functions:
$$\mu = Xb$$ 
where the $j$th column of $X$ is the step function with a step at $j$ ($j = 1,\dots,(T-1)$); that is $x_{tj}=0$ for $t<=j$ and 1 for $t>j$.
The $j$th element of $b$ therefore determines the change in the mean $|\mu_j-\mu_{j+1}|$, and an assumption that $b$ is sparse encapsulates an
assumption that $\mu$ is spatially structured (indeed, piecewise constant).

This very simple approach is essentially 0th-order trend filtering
(Tibshirani, 2014, *Annals of Statistics* 42, 285--323). Note that
higher-order trend filtering can be similarly implemented using
different basis functions; here we use 0th order only for simplicity.


# Examples

Here we apply SuSiE to perform 0th order trend filtering in some simple simulated examples. We have implemented in `susieR` 0.6.0 a funciton `susie_trendfilter` which internally creates $X$ matrix with step functions in the columns to match input $y$. The algebra have been optimized to work on such trendfiltering matrices. Here we simulate some data where $\mu$ is a step function with four steps, a 0th order trendfiltering problem. The regression is truly sparse and SuSiE should do well.

```{r}
library(susieR)
set.seed(1)
n=1000
mu = c(rep(0,100),rep(1,100),rep(3,100),rep(-2,100),rep(0,600))
y = mu + rnorm(n)
s = susie_trendfilter(y, 0, L=10)
```

Plot results: the truth is black, and susie estimate is red.
```{r}
plot(y,pch=".")
lines(mu,col=1,lwd=3)
lines(predict(s),col=2,lwd=2)
s$sigma2
```

In the next example mu increases linearly. Thus we are approximating a
linear function by step functions. Here the truth is not trully
"sparse", so we might expect performance to be poor, but it is not too
bad.

```{r}
set.seed(1)
mu = seq(0,4,length=1000)
y = mu + rnorm(n)
s = susie_trendfilter(y,0,L=10)
plot(y,pch=".")
lines(mu,col=1,lwd=3)
lines(predict(s),col=2,lwd=2)
```

Compare with the genlasso (lasso-based) solution (blue). (This will
require installation of the `genlasso` package, which is not available
on CRAN, but can be installed from GitHub.)

```{r, eval=FALSE}
# install.packages("remotes")
# remotes::install_github("glmgen/genlasso")
y.tf = trendfilter(y,ord=0)
y.tf.cv = cv.trendfilter(y.tf)
plot(y,pch=".")
lines(mu,col=1,lwd=3)
lines(predict(s),col=2,lwd=2)
lines(y.tf$fit[,which(y.tf$lambda==y.tf.cv$lambda.min)],col=4,lwd=2)
```

What happens if we have linear trend plus a sudden change.

```{r, eval=FALSE}
set.seed(1)
mu = seq(0,4,length=1000)
mu = mu + c(rep(0,500),rep(4,500))
y = mu + rnorm(n)
s = susie_trendfilter(y,0,L=10)
y.tf = trendfilter(y,ord=0)
y.tf.cv = cv.trendfilter(y.tf)
plot(y,pch=".")
lines(mu,col=1,lwd=3)
lines(predict(s),col=2,lwd=2)
lines(y.tf$fit[,which(y.tf$lambda==y.tf.cv$lambda.min)],col=4,lwd=2)
```

The two fits seem similar in accuracy. We can check this numerically:
```{r, eval=FALSE}
sqrt(mean((mu-y.tf$fit[,which(y.tf$lambda==y.tf.cv$lambda.min)])^2))
sqrt(mean((mu-predict(s))^2))
```

---
title: "Fine-mapping example"
author: "Gao Wang"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Fine-mapping example}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(collapse = TRUE,comment = "#",fig.width = 5,
                      fig.height = 3,fig.align = "center",
                      fig.cap = "&nbsp;",dpi = 120)
```

This vignette demonstrates `susieR` in the context of genetic
fine-mapping.  We use simulated data of expression level of a gene
($y$) in $N \approx 600$ individuals.  We want to identify with the
genotype matrix $X_{N\times P}$ ($P=1001$) the genetic variables that
causes changes in expression level.

The simulated data set is simulated to have exactly 3 non-zero
effects.

```{r}
library(susieR)
set.seed(1)
```

## The data-set

```{r}
data(N3finemapping)
attach(N3finemapping)
```

The loaded dataset contains regression data $X$ and $y$, along with some other
relevant properties in the context of genetic studies. It also
contains the "true" regression coefficent the data is simulated from.

Notice that we've simulated 2 sets of $Y$ as 2 simulation
replicates. Here we'll focus on the first data-set.

```{r}
dim(Y)
```

Here are the 3 "true" signals in the first data-set:

```{r}
b <- true_coef[,1]
plot(b, pch=16, ylab='effect size')
```

```{r}
which(b != 0)
```

So the underlying causal variables are 403, 653 and 773.

## Simple regression summary statistics

`univariate_regression` function can be used to compute 
summary statistics by fitting univariate simple regression variable by variable.
The results are $\hat{\beta}$ and $SE(\hat{\beta})$ from which z-scores can be
derived. Again we focus only on results from the first data-set:

```{r}
sumstats <- univariate_regression(X, Y[,1])
z_scores <- sumstats$betahat / sumstats$sebetahat
susie_plot(z_scores, y = "z", b=b)
```

## Fine-mapping with `susieR`

For starters, we assume there are at most 10 causal variables, i.e.,
set `L = 10`, although SuSiE is robust to the choice of `L`.

The `susieR` function call is:

```{r}
fitted <- susie(X, Y[,1],
                L = 10,
		verbose = TRUE)
```

### Credible sets

By default, we output 95% credible set:

```{r}
print(fitted$sets)
```

The 3 causal signals have been captured by the 3 CS reported here. The
3rd CS contains many variables, including the true causal variable
`403`. The minimum absolute correlation is 0.86.

If we use the default 90% coverage for credible sets, we still
capture the 3 signals, but "purity" of the 3rd CS is now 0.91 and size
of the CS is also a bit smaller.

```{r}
sets <- susie_get_cs(fitted,
                     X = X,
	  	     coverage = 0.9,
                     min_abs_corr = 0.1)
```

```{r}
print(sets)
```

### Posterior inclusion probabilities

Previously we've determined that summing over 3 single effect
regression models is approperate for our application. Here we
summarize the variable selection results by posterior inclusion
probability (PIP):

```{r}
susie_plot(fitted, y="PIP", b=b)
```

The true causal variables are colored red. The 95% CS identified are
circled in different colors. Of interest is the cluster around
position 400. The true signal is 403 but apparently it does not have
the highest PIP. To compare ranking of PIP and original z-score in
that CS:

```{r}
i  <- fitted$sets$cs[[3]]
z3 <- cbind(i,z_scores[i],fitted$pip[i])
colnames(z3) <- c('position', 'z-score', 'PIP')
z3[order(z3[,2], decreasing = TRUE),]
```

### Choice of priors

Notice that by default SuSiE estimates prior effect size from data. For
fine-mapping applications, however, we sometimes have knowledge of SuSiE prior effect
size since it is parameterized as percentage of variance explained (PVE) by a non-zero effect, 
which, in the context of fine-mapping, is related to per-SNP heritability. It is possible
to use `scaled_prior_variance` to specify this PVE and explicitly set `estimate_prior_variance=FALSE`
to fix the prior effect to given value.

In this data-set, SuSiE is robust to choice of priors. Here we
set PVE to 0.2,
and compare with previous results:

```{r}
fitted = susie(X, Y[,1],
               L = 10,
               estimate_residual_variance = TRUE, 
               estimate_prior_variance = FALSE, 
               scaled_prior_variance = 0.2)
susie_plot(fitted, y='PIP', b=b)
```

which largely remains unchanged. 

### A note on covariate adjustment

To include covariate `Z` in SuSiE, one approach is to regress it out from both `y`
and `X`, and then run SuSiE on the residuals. The code below illustrates the procedure:

```{r, eval=FALSE}
remove.covariate.effects <- function (X, Z, y) {
  # include the intercept term
  if (any(Z[,1]!=1)) Z = cbind(1, Z)
  A   <- forceSymmetric(crossprod(Z))
  SZy <- as.vector(solve(A,c(y %*% Z)))
  SZX <- as.matrix(solve(A,t(Z) %*% X))
  y <- y - c(Z %*% SZy)
  X <- X - Z %*% SZX
  return(list(X = X,y = y,SZy = SZy,SZX = SZX))
}

out = remove.covariate.effects(X, Z, Y[,1])
fitted_adjusted = susie(out$X, out$y,
               L = 10) ,
```

Note that the covariates `Z` should have a column of ones as the first column. If not, the above function `remove.covariate.effects`
will add such a column to `Z` before regressing it out. 
Data will be centered as a result. Also the scale of data is changed after regressing out `Z`. 
This introduces some subtleties in terms of interpreting the results.
For this reason, we provide covariate adjustment procedure as a tip in the documentation
and not part of `susieR::susie()` function. Cautions should be taken when applying this procedure and interpreting the result from it.

## Session information

Here are some details about the computing environment, including the
versions of R, and the R packages, used to generate these results.

```{r}
sessionInfo()
```

[N3finemapping]: https://github.com/stephenslab/susieR/blob/master/inst/datafiles/N3finemapping.rds

---
title: "SuSiE with L0Learn initialization example"
author: "Kaiqian Zhang"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{L0Learn initialization demo}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(collapse = TRUE,comment = "#")
```

`susieR` allows for a customized initialization. In this vignette we
deomonstrate how to use
[`L0Learn`](https://github.com/hazimehh/L0Learn) fit to initialize
susieR.

```{r, warning=FALSE}
library(susieR)
library(L0Learn)
```

## Simulate data

First, we simulate data from the minimal example.

```{r}
set.seed(1)
n = 1000
p = 1000
beta = rep(0,p)
beta[c(1,2,300,400)] = 1
X = matrix(rnorm(n*p),nrow=n,ncol=p)
y = X %*% beta + rnorm(n)
```

## Fit L0Learn 

We start with fitting a L0-regularized model to the simulated data. 

```{r}
set.seed(1)
L0fit = L0Learn.cvfit(X, y, penalty = "L0")
```

Let's choose the penalty strength parameter that minimizes the
cross-validation error.

```{r}
lambdaIndex = which.min(L0fit$cvMeans[[1]]) 
L0coef = as.numeric(coef(L0fit$fit, lambda = L0fit$fit$lambda[[1]][lambdaIndex]))
effect.beta = L0coef[which(L0coef!=0)][-1]
effect.index = (which(L0coef!=0)-1)[-1] 
length(effect.beta)
effect.beta[1:10]
effect.index[1:10]
```

The L0Learn model finds `r length(effect.beta)` effects, which will be
used to initialize susie.

## Build an initialization object

We create an initialization from `l0learn` coefficients using
`susie_init_coef` function,

```{r}
set.seed(1)
s.init = susie_init_coef(effect.index, effect.beta, p)
```

## Run susieR with initialization

Now, we use `effect.index` and `effect.beta` obtained from L0Learn fit
to create an `s.init` object. We then run susie with this
initialization.

```{r}
susieL0.fit = susie(X,y,s_init=s.init)
susieL0.fit$sets$cs
```

## References

1. Hussein Hazimeh and Rahul Mazumder. (2018). Fast Best Subset
Selection: Coordinate Descent and Local Combinatorial Optimization
Algorithms.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/compute_ss.R
\name{compute_ss}
\alias{compute_ss}
\title{Compute sufficient statistics for input to \code{susie_suff_stat}}
\usage{
compute_ss(X, y, standardize = FALSE)
}
\arguments{
\item{X}{An n by p matrix of covariates.}

\item{y}{An n vector.}

\item{standardize}{Logical flag indicating whether to standardize
columns of X to unit variance prior to computing summary data.}
}
\value{
A list of sufficient statistics (X'X, X'y, y'y and n)
}
\description{
This is a synonym for \code{compute_suff_stat}
  included for historical reasons (deprecated).
}
\examples{
data(N2finemapping)
ss = compute_ss(N2finemapping$X, N2finemapping$Y[,1])

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{N3finemapping}
\alias{N3finemapping}
\title{Simulated Fine-mapping Data with Three Effect Variables.}
\format{
\code{N3finemapping} is a list with the following elements:

\describe{

  \item{X}{N by P variable matrix of centered and scaled genotype
data.}

  \item{chrom}{Chromomsome of the original data, in hg38 coordinate.}

  \item{pos}{Chromomosomal positoin of the original data, in hg38
coordinate. The information can be used to compare impact of using
other genotype references of the same variables in susie_rss
application.}

  \item{true_coef}{The simulated effect sizes.}

  \item{residual_variance}{The simulated residual covariance matrix.}

  \item{Y}{The simulated response variables.}

  \item{allele_freq}{Allele frequency of the original genotype data.}

  \item{V}{Prior covariance matrix for effect size of the three
non-zero effect variables.}  }
}
\description{
The data-set contains a matrix of 574
individuals and 1,001 variables. These variables are real-world
genotypes centered and scaled, and therefore retains the
correlation structure of variables in the original genotype data. 3
out of the variables have non-zero effects.  The response data is
generated under a multivariate linear regression model. See Wang
\emph{et al} (2020) for more details.
}
\examples{
data(N3finemapping)
}
\references{
G. Wang, A. Sarkar, P. Carbonetto and M. Stephens (2020). A simple
  new approach to variable selection in regression, with application
  to genetic fine-mapping. \emph{Journal of the Royal Statistical
  Society, Series B} \doi{10.1101/501114}.
}
\keyword{data}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/susie_utils.R
\name{kriging_rss}
\alias{kriging_rss}
\title{Compute Distribution of z-scores of Variant j Given Other z-scores, and Detect Possible Allele Switch Issue}
\usage{
kriging_rss(
  z,
  R,
  n,
  r_tol = 1e-08,
  s = estimate_s_rss(z, R, n, r_tol, method = "null-mle")
)
}
\arguments{
\item{z}{A p-vector of z scores.}

\item{R}{A p by p symmetric, positive semidefinite correlation
matrix.}

\item{n}{The sample size. (Optional, but highly recommended.)}

\item{r_tol}{Tolerance level for eigenvalue check of positive
semidefinite matrix of R.}

\item{s}{an estimated s from \code{estimate_s_rss}}
}
\value{
a list containing a ggplot2 plot object and a table. The plot
  compares observed z score vs the expected value. The possible allele
  switched variants are labeled as red points (log LR > 2 and abs(z) > 2).
  The table summarizes the conditional distribution for each variant
  and the likelihood ratio test. The table has the following columns:
  the observed z scores, the conditional expectation, the conditional
  variance, the standardized differences between the observed z score
  and expected value, the log likelihood ratio statistics.
}
\description{
Under the null, the rss model with regularized LD
  matrix is \eqn{z|R,s ~ N(0, (1-s)R + s I))}. We use a mixture of
  normals to model the conditional distribution of z_j given other z
  scores, \eqn{z_j | z_{-j}, R, s ~ \sum_{k=1}^{K} \pi_k
  N(-\Omega_{j,-j} z_{-j}/\Omega_{jj}, \sigma_{k}^2/\Omega_{jj})},
  \eqn{\Omega = ((1-s)R + sI)^{-1}}, \eqn{\sigma_1, ..., \sigma_k}
  is a grid of fixed positive numbers. We estimate the mixture
  weights \eqn{\pi}  We detect the possible allele switch issue
  using likelihood ratio for each variant.
}
\examples{
# See also the vignette, "Diagnostic for fine-mapping with summary
# statistics."
set.seed(1)
n = 500
p = 1000
beta = rep(0,p)
beta[1:4] = 0.01
X = matrix(rnorm(n*p),nrow = n,ncol = p)
X = scale(X,center = TRUE,scale = TRUE)
y = drop(X \%*\% beta + rnorm(n))
ss = univariate_regression(X,y)
R = cor(X)
attr(R,"eigen") = eigen(R,symmetric = TRUE)
zhat = with(ss,betahat/sebetahat)
cond_dist = kriging_rss(zhat,R,n = n)
cond_dist$plot

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/susie_zzz_auto.R
\name{susie_auto}
\alias{susie_auto}
\title{Attempt at Automating SuSiE for Hard Problems}
\usage{
susie_auto(
  X,
  y,
  L_init = 1,
  L_max = 512,
  verbose = FALSE,
  init_tol = 1,
  standardize = TRUE,
  intercept = TRUE,
  max_iter = 100,
  tol = 0.01,
  ...
)
}
\arguments{
\item{X}{An n by p matrix of covariates.}

\item{y}{The observed responses, a vector of length n.}

\item{L_init}{The initial value of L.}

\item{L_max}{The largest value of L to consider.}

\item{verbose}{If \code{verbose = TRUE}, the algorithm's progress,
and a summary of the optimization settings, are printed to the
console.}

\item{init_tol}{The tolerance to passed to \code{susie} during
early runs (set large to shorten the initial runs).}

\item{standardize}{If \code{standardize = TRUE}, standardize the
columns of X to unit variance prior to fitting. Note that
\code{scaled_prior_variance} specifies the prior on the
coefficients of X \emph{after} standardization (if it is
performed). If you do not standardize, you may need to think more
carefully about specifying \code{scaled_prior_variance}. Whatever
your choice, the coefficients returned by \code{coef} are given for
\code{X} on the original input scale. Any column of \code{X} that
has zero variance is not standardized.}

\item{intercept}{If \code{intercept = TRUE}, the intercept is
fitted; it \code{intercept = FALSE}, the intercept is set to
zero. Setting \code{intercept = FALSE} is generally not
recommended.}

\item{max_iter}{Maximum number of IBSS iterations to perform.}

\item{tol}{A small, non-negative number specifying the convergence
tolerance for the IBSS fitting procedure. The fitting procedure
will halt when the difference in the variational lower bound, or
\dQuote{ELBO} (the objective function to be maximized), is
less than \code{tol}.}

\item{\dots}{Additional arguments passed to \code{\link{susie}}.}
}
\value{
See \code{\link{susie}} for a description of return values.
}
\description{
\code{susie_auto} is an attempt to automate reliable
  running of susie even on hard problems. It implements a three-stage
  strategy for each L: first, fit susie with very small residual
  error; next, estimate residual error; finally, estimate the prior
  variance. If the last step estimates some prior variances to be
  zero, stop. Otherwise, double L, and repeat. Initial runs are
  performed with relaxed tolerance; the final run is performed using
  the default susie tolerance.
}
\examples{
set.seed(1)
n = 1000
p = 1000
beta = rep(0,p)
beta[1:4] = 1
X = matrix(rnorm(n*p),nrow = n,ncol = p)
X = scale(X,center = TRUE,scale = TRUE)
y = drop(X \%*\% beta + rnorm(n))
res = susie_auto(X,y)
plot(beta,coef(res)[-1])
abline(a = 0,b = 1,col = "skyblue",lty = "dashed")
plot(y,predict(res))
abline(a = 0,b = 1,col = "skyblue",lty = "dashed")

}
\seealso{
\code{\link{susie}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{FinemappingConvergence}
\alias{FinemappingConvergence}
\title{Simulated Fine-mapping Data with Convergence Problem.}
\format{
\code{FinemappingConvergence} is a list with the following
elements:

\describe{

  \item{XtX}{Summary statistics computed using the centered and
    scaled genotype matrix.}

  \item{Xty}{Summary statistics computed using the centered and
    scaled genotype data, and the centered simulated response.}

  \item{yty}{yty is computed using the centered simulated response.}

  \item{n}{The sample size (50,000).}

  \item{true_coef}{The coefficients used to simulate the responses.}

  \item{z}{z-scores from a simple (single-SNP) linear regression.}}
}
\description{
Data simulated using real genotypes from 50,000
  individuals and 200 SNPs. Two of the SNPs have non-zero effects
  on the multivariate response. The response data are generated under
  a linear regression model. The simulated response and the columns
  of the genotype matrix are centered.
}
\examples{
data(FinemappingConvergence)
}
\seealso{
A similar data set with more SNPs is used in the
  \dQuote{Refine SuSiE model} vignette.
}
\keyword{data}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/predict.susie.R
\name{coef.susie}
\alias{coef.susie}
\title{Extract regression coefficients from susie fit}
\usage{
\method{coef}{susie}(object, ...)
}
\arguments{
\item{object}{A susie fit.}

\item{\dots}{Additional arguments passed to the generic \code{coef}
method.}
}
\value{
A p+1 vector, the first element being an intercept, and the
  remaining p elements being estimated regression coefficients.
}
\description{
Extract regression coefficients from susie fit
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/univariate_regression.R
\name{univariate_regression}
\alias{univariate_regression}
\title{Perform Univariate Linear Regression Separately for Columns of X}
\usage{
univariate_regression(
  X,
  y,
  Z = NULL,
  center = TRUE,
  scale = FALSE,
  return_residuals = FALSE
)
}
\arguments{
\item{X}{n by p matrix of regressors.}

\item{y}{n-vector of response variables.}

\item{Z}{Optional n by k matrix of covariates to be included in all
regresions. If Z is not \code{NULL}, the linear effects of
covariates are removed from y first, and the resulting residuals
are used in place of y.}

\item{center}{If \code{center = TRUE}, center X, y and Z.}

\item{scale}{If \code{scale = TRUE}, scale X, y and Z.}

\item{return_residuals}{Whether or not to output the residuals if Z
is not \code{NULL}.}
}
\value{
A list with two vectors containing the least-squares
  estimates of the coefficients (\code{betahat}) and their standard
  errors (\code{sebetahat}). Optionally, and only when a matrix of
  covariates \code{Z} is provided, a third vector \code{residuals}
  containing the residuals is returned.
}
\description{
This function performs the univariate linear
  regression y ~ x separately for each column x of X. Each regression
  is implemented using \code{.lm.fit()}. The estimated effect size
  and stardard error for each variable are outputted.
}
\examples{
set.seed(1)
n = 1000
p = 1000
beta = rep(0,p)
beta[1:4] = 1
X = matrix(rnorm(n*p),nrow = n,ncol = p)
X = scale(X,center = TRUE,scale = TRUE)
y = drop(X \%*\% beta + rnorm(n))
res = univariate_regression(X,y)
plot(res$betahat/res$sebetahat)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/compute_ss.R
\name{compute_suff_stat}
\alias{compute_suff_stat}
\title{Compute sufficient statistics for input to \code{susie_suff_stat}}
\usage{
compute_suff_stat(X, y, standardize = FALSE)
}
\arguments{
\item{X}{An n by p matrix of covariates.}

\item{y}{An n vector.}

\item{standardize}{Logical flag indicating whether to standardize
columns of X to unit variance prior to computing summary data}
}
\value{
A list of sufficient statistics (\code{XtX, Xty, yty, n})
  and \code{X_colmeans}, \code{y_mean}.
}
\description{
Computes the sufficient statistics \eqn{X'X, X'y, y'y}
  and \eqn{n} after centering (and possibly standardizing) the
  columns of \eqn{X} and centering \eqn{y} to have mean zero. We also
  store the column means of \eqn{X} and mean of \eqn{y}.
}
\examples{
data(N2finemapping)
ss = compute_suff_stat(N2finemapping$X, N2finemapping$Y[,1])

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/susie_utils.R
\name{estimate_s_rss}
\alias{estimate_s_rss}
\title{Estimate s in \code{susie_rss} Model Using Regularized LD}
\usage{
estimate_s_rss(z, R, n, r_tol = 1e-08, method = "null-mle")
}
\arguments{
\item{z}{A p-vector of z scores.}

\item{R}{A p by p symmetric, positive semidefinite correlation
matrix.}

\item{n}{The sample size. (Optional, but highly recommended.)}

\item{r_tol}{Tolerance level for eigenvalue check of positive
semidefinite matrix of R.}

\item{method}{a string specifies the method to estimate \eqn{s}.}
}
\value{
A number between 0 and 1.
}
\description{
The estimated s gives information about the
  consistency between the z scores and LD matrix. A larger \eqn{s}
  means there is a strong inconsistency between z scores and LD
  matrix. The \dQuote{null-mle} method obtains mle of \eqn{s} under
  \eqn{z | R ~ N(0,(1-s)R + s I)}, \eqn{0 < s < 1}. The
  \dQuote{null-partialmle} method obtains mle of \eqn{s} under
  \eqn{U^T z | R ~ N(0,s I)}, in which \eqn{U} is a matrix containing
  the of eigenvectors that span the null space of R; that is, the
  eigenvectors corresponding to zero eigenvalues of R. The estimated
  \eqn{s} from \dQuote{null-partialmle} could be greater than 1. The
  \dQuote{null-pseudomle} method obtains mle of \eqn{s} under
  pseudolikelihood \eqn{L(s) = \prod_{j=1}^{p} p(z_j | z_{-j}, s,
  R)}, \eqn{0 < s < 1}.
}
\examples{
set.seed(1)
n = 500
p = 1000
beta = rep(0,p)
beta[1:4] = 0.01
X = matrix(rnorm(n*p),nrow = n,ncol = p)
X = scale(X,center = TRUE,scale = TRUE)
y = drop(X \%*\% beta + rnorm(n))
input_ss = compute_suff_stat(X,y,standardize = TRUE)
ss = univariate_regression(X,y)
R = cor(X)
attr(R,"eigen") = eigen(R,symmetric = TRUE)
zhat = with(ss,betahat/sebetahat)

# Estimate s using the unadjusted z-scores.
s0 = estimate_s_rss(zhat,R)

# Estimate s using the adjusted z-scores.
s1 = estimate_s_rss(zhat,R,n)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary.susie.R
\name{summary.susie}
\alias{summary.susie}
\alias{print.summary.susie}
\title{Summarize Susie Fit.}
\usage{
\method{summary}{susie}(object, ...)

\method{print}{summary.susie}(x, ...)
}
\arguments{
\item{object}{A susie fit.}

\item{\dots}{Additional arguments passed to the generic \code{summary}
or \code{print.summary} method.}

\item{x}{A susie summary.}
}
\value{
\code{summary.susie} returns a list containing a data frame
  of variables and a data frame of credible sets.
}
\description{
\code{summary} method for the \dQuote{susie} class.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/susie.R, R/susie_ss.R
\name{susie}
\alias{susie}
\alias{susie_suff_stat}
\title{Sum of Single Effects (SuSiE) Regression}
\usage{
susie(
  X,
  y,
  L = min(10, ncol(X)),
  scaled_prior_variance = 0.2,
  residual_variance = NULL,
  prior_weights = NULL,
  null_weight = 0,
  standardize = TRUE,
  intercept = TRUE,
  estimate_residual_variance = TRUE,
  estimate_prior_variance = TRUE,
  estimate_prior_method = c("optim", "EM", "simple"),
  check_null_threshold = 0,
  prior_tol = 1e-09,
  residual_variance_upperbound = Inf,
  s_init = NULL,
  coverage = 0.95,
  min_abs_corr = 0.5,
  compute_univariate_zscore = FALSE,
  na.rm = FALSE,
  max_iter = 100,
  tol = 0.001,
  verbose = FALSE,
  track_fit = FALSE,
  residual_variance_lowerbound = var(drop(y))/10000,
  refine = FALSE,
  n_purity = 100
)

susie_suff_stat(
  XtX,
  Xty,
  yty,
  n,
  X_colmeans = NA,
  y_mean = NA,
  maf = NULL,
  maf_thresh = 0,
  L = 10,
  scaled_prior_variance = 0.2,
  residual_variance = NULL,
  estimate_residual_variance = TRUE,
  estimate_prior_variance = TRUE,
  estimate_prior_method = c("optim", "EM", "simple"),
  check_null_threshold = 0,
  prior_tol = 1e-09,
  r_tol = 1e-08,
  prior_weights = NULL,
  null_weight = 0,
  standardize = TRUE,
  max_iter = 100,
  s_init = NULL,
  coverage = 0.95,
  min_abs_corr = 0.5,
  tol = 0.001,
  verbose = FALSE,
  track_fit = FALSE,
  check_input = FALSE,
  refine = FALSE,
  check_prior = FALSE,
  n_purity = 100,
  ...
)
}
\arguments{
\item{X}{An n by p matrix of covariates.}

\item{y}{The observed responses, a vector of length n.}

\item{L}{Maximum number of non-zero effects in the susie
regression model. If L is larger than the number of covariates, p,
L is set to p.}

\item{scaled_prior_variance}{The prior variance, divided by
\code{var(y)} (or by \code{(1/(n-1))yty} for
\code{susie_suff_stat}); that is, the prior variance of each
non-zero element of b is \code{var(y) * scaled_prior_variance}. The
value provided should be either a scalar or a vector of length
\code{L}. If \code{estimate_prior_variance = TRUE}, this provides
initial estimates of the prior variances.}

\item{residual_variance}{Variance of the residual. If
\code{estimate_residual_variance = TRUE}, this value provides the
initial estimate of the residual variance. By default, it is set to
\code{var(y)} in \code{susie} and \code{(1/(n-1))yty} in
\code{susie_suff_stat}.}

\item{prior_weights}{A vector of length p, in which each entry
gives the prior probability that corresponding column of X has a
nonzero effect on the outcome, y.}

\item{null_weight}{Prior probability of no effect (a number between
0 and 1, and cannot be exactly 1).}

\item{standardize}{If \code{standardize = TRUE}, standardize the
columns of X to unit variance prior to fitting (or equivalently
standardize XtX and Xty to have the same effect). Note that
\code{scaled_prior_variance} specifies the prior on the
coefficients of X \emph{after} standardization (if it is
performed). If you do not standardize, you may need to think more
carefully about specifying \code{scaled_prior_variance}. Whatever
your choice, the coefficients returned by \code{coef} are given for
\code{X} on the original input scale. Any column of \code{X} that
has zero variance is not standardized.}

\item{intercept}{If \code{intercept = TRUE}, the intercept is
fitted; it \code{intercept = FALSE}, the intercept is set to
zero. Setting \code{intercept = FALSE} is generally not
recommended.}

\item{estimate_residual_variance}{If
\code{estimate_residual_variance = TRUE}, the residual variance is
estimated, using \code{residual_variance} as an initial value. If
\code{estimate_residual_variance = FALSE}, the residual variance is
fixed to the value supplied by \code{residual_variance}.}

\item{estimate_prior_variance}{If \code{estimate_prior_variance =
TRUE}, the prior variance is estimated (this is a separate
parameter for each of the L effects). If provided,
\code{scaled_prior_variance} is then used as an initial value for
the optimization. When \code{estimate_prior_variance = FALSE}, the
prior variance for each of the L effects is determined by the
value supplied to \code{scaled_prior_variance}.}

\item{estimate_prior_method}{The method used for estimating prior
variance. When \code{estimate_prior_method = "simple"} is used, the
likelihood at the specified prior variance is compared to the
likelihood at a variance of zero, and the setting with the larger
likelihood is retained.}

\item{check_null_threshold}{When the prior variance is estimated,
compare the estimate with the null, and set the prior variance to
zero unless the log-likelihood using the estimate is larger by this
threshold amount. For example, if you set
\code{check_null_threshold = 0.1}, this will "nudge" the estimate
towards zero when the difference in log-likelihoods is small. A
note of caution that setting this to a value greater than zero may
lead the IBSS fitting procedure to occasionally decrease the ELBO.}

\item{prior_tol}{When the prior variance is estimated, compare the
estimated value to \code{prior_tol} at the end of the computation,
and exclude a single effect from PIP computation if the estimated
prior variance is smaller than this tolerance value.}

\item{residual_variance_upperbound}{Upper limit on the estimated
residual variance. It is only relevant when
\code{estimate_residual_variance = TRUE}.}

\item{s_init}{A previous susie fit with which to initialize.}

\item{coverage}{A number between 0 and 1 specifying the
\dQuote{coverage} of the estimated confidence sets.}

\item{min_abs_corr}{Minimum absolute correlation allowed in a
credible set. The default, 0.5, corresponds to a squared
correlation of 0.25, which is a commonly used threshold for
genotype data in genetic studies.}

\item{compute_univariate_zscore}{If \code{compute_univariate_zscore
= TRUE}, the univariate regression z-scores are outputted for each
variable.}

\item{na.rm}{Drop any missing values in y from both X and y.}

\item{max_iter}{Maximum number of IBSS iterations to perform.}

\item{tol}{A small, non-negative number specifying the convergence
tolerance for the IBSS fitting procedure. The fitting procedure
will halt when the difference in the variational lower bound, or
\dQuote{ELBO} (the objective function to be maximized), is
less than \code{tol}.}

\item{verbose}{If \code{verbose = TRUE}, the algorithm's progress,
and a summary of the optimization settings, are printed to the
console.}

\item{track_fit}{If \code{track_fit = TRUE}, \code{trace}
is also returned containing detailed information about the
estimates at each iteration of the IBSS fitting procedure.}

\item{residual_variance_lowerbound}{Lower limit on the estimated
residual variance. It is only relevant when
\code{estimate_residual_variance = TRUE}.}

\item{refine}{If \code{refine = TRUE}, then an additional
iterative refinement procedure is used, after the IBSS algorithm,
to check and escape from local optima (see details).}

\item{n_purity}{Passed as argument \code{n_purity} to
\code{\link{susie_get_cs}}.}

\item{XtX}{A p by p matrix \eqn{X'X} in which the columns of X
are centered to have mean zero.}

\item{Xty}{A p-vector \eqn{X'y} in which y and the columns of X are
centered to have mean zero.}

\item{yty}{A scalar \eqn{y'y} in which y is centered to have mean
zero.}

\item{n}{The sample size.}

\item{X_colmeans}{A p-vector of column means of \code{X}. If both
\code{X_colmeans} and \code{y_mean} are provided, the intercept
is estimated; otherwise, the intercept is NA.}

\item{y_mean}{A scalar containing the mean of \code{y}. If both
\code{X_colmeans} and \code{y_mean} are provided, the intercept
is estimated; otherwise, the intercept is NA.}

\item{maf}{Minor allele frequency; to be used along with
\code{maf_thresh} to filter input summary statistics.}

\item{maf_thresh}{Variants having a minor allele frequency smaller
than this threshold are not used.}

\item{r_tol}{Tolerance level for eigenvalue check of positive
semidefinite matrix of R.}

\item{check_input}{If \code{check_input = TRUE},
\code{susie_suff_stat} performs additional checks on \code{XtX} and
\code{Xty}. The checks are: (1) check that \code{XtX} is positive
semidefinite; (2) check that \code{Xty} is in the space spanned by
the non-zero eigenvectors of \code{XtX}.}

\item{check_prior}{If \code{check_prior = TRUE}, it checks if the
estimated prior variance becomes unreasonably large (comparing with
10 * max(abs(z))^2).}

\item{...}{Additional arguments to provide backward compatibility
with earlier versions of \code{susie_suff_stat}.}
}
\value{
A \code{"susie"} object with some or all of the following
  elements:

\item{alpha}{An L by p matrix of posterior inclusion probabilites.}

\item{mu}{An L by p matrix of posterior means, conditional on
  inclusion.}

\item{mu2}{An L by p matrix of posterior second moments,
  conditional on inclusion.}

\item{Xr}{A vector of length n, equal to \code{X \%*\% colSums(alpha
  * mu)}.}

\item{lbf}{log-Bayes Factor for each single effect.}

\item{lbf_variable}{log-Bayes Factor for each variable and single effect.}

\item{intercept}{Intercept (fixed or estimated).}

\item{sigma2}{Residual variance (fixed or estimated).}

\item{V}{Prior variance of the non-zero elements of b, equal to
  \code{scaled_prior_variance * var(y)}.}

\item{elbo}{The value of the variational lower bound, or
  \dQuote{ELBO} (objective function to be maximized), achieved at
  each iteration of the IBSS fitting procedure.}

\item{fitted}{Vector of length n containing the fitted values of
  the outcome.}

\item{sets}{Credible sets estimated from model fit; see
  \code{\link{susie_get_cs}} for details.}

\item{pip}{A vector of length p giving the (marginal) posterior
  inclusion probabilities for all p covariates.}

\item{z}{A vector of univariate z-scores.}

\item{niter}{Number of IBSS iterations that were performed.}

\item{converged}{\code{TRUE} or \code{FALSE} indicating whether
  the IBSS converged to a solution within the chosen tolerance
  level.}

\code{susie_suff_stat} returns also outputs:

\item{XtXr}{A p-vector of \code{t(X)} times the fitted values,
  \code{X \%*\% colSums(alpha*mu)}.}
}
\description{
Performs a sparse Bayesian multiple linear regression
  of y on X, using the "Sum of Single Effects" model from Wang et al
  (2020). In brief, this function fits the regression model \eqn{y =
  \mu + X b + e}, where elements of \eqn{e} are \emph{i.i.d.} normal
  with zero mean and variance \code{residual_variance}, \eqn{\mu} is
  an intercept term and \eqn{b} is a vector of length p representing
  the effects to be estimated. The \dQuote{susie assumption} is that
  \eqn{b = \sum_{l=1}^L b_l} where each \eqn{b_l} is a vector of
  length p with exactly one non-zero element. The prior on the
  non-zero element is normal with zero mean and variance \code{var(y)
  * scaled_prior_variance}. The value of \code{L} is fixed, and
  should be chosen to provide a reasonable upper bound on the number
  of non-zero effects to be detected. Typically, the hyperparameters
  \code{residual_variance} and \code{scaled_prior_variance} will be
  estimated during model fitting, although they can also be fixed as
  specified by the user. See functions \code{\link{susie_get_cs}} and
  other functions of form \code{susie_get_*} to extract the most
  commonly-used results from a susie fit.
}
\details{
The function \code{susie} implements the IBSS algorithm
from Wang et al (2020). The option \code{refine = TRUE} implements
an additional step to help reduce problems caused by convergence of
the IBSS algorithm to poor local optima (which is rare in our
experience, but can provide misleading results when it occurs). The
refinement step incurs additional computational expense that
increases with the number of CSs found in the initial run.

The function \code{susie_suff_stat} implements essentially the same
algorithms, but using sufficient statistics. (The statistics are
sufficient for the regression coefficients \eqn{b}, but not for the
intercept \eqn{\mu}; see below for how the intercept is treated.)
If the sufficient statistics are computed correctly then the
results from \code{susie_suff_stat} should be the same as (or very
similar to) \code{susie}, although runtimes will differ as
discussed below. The sufficient statistics are the sample
size \code{n}, and then the p by p matrix \eqn{X'X}, the p-vector
\eqn{X'y}, and the sum of squared y values \eqn{y'y}, all computed
after centering the columns of \eqn{X} and the vector \eqn{y} to
have mean 0; these can be computed using \code{compute_suff_stat}.

The handling of the intercept term in \code{susie_suff_stat} needs
some additional explanation. Computing the summary data after
centering \code{X} and \code{y} effectively ensures that the
resulting posterior quantities for \eqn{b} allow for an intercept
in the model; however, the actual value of the intercept cannot be
estimated from these centered data. To estimate the intercept term
the user must also provide the column means of \eqn{X} and the mean
of \eqn{y} (\code{X_colmeans} and \code{y_mean}). If these are not
provided, they are treated as \code{NA}, which results in the
intercept being \code{NA}. If for some reason you prefer to have
the intercept be 0 instead of \code{NA} then set
\code{X_colmeans = 0,y_mean = 0}.

For completeness, we note that if \code{susie_suff_stat} is run on
\eqn{X'X, X'y, y'y} computed \emph{without} centering \eqn{X} and
\eqn{y}, and with \code{X_colmeans = 0,y_mean = 0}, this is
equivalent to \code{susie} applied to \eqn{X, y} with
\code{intercept = FALSE} (although results may differ due to
different initializations of \code{residual_variance} and
\code{scaled_prior_variance}). However, this usage is not
recommended for for most situations.

The computational complexity of \code{susie} is \eqn{O(npL)} per
iteration, whereas \code{susie_suff_stat} is \eqn{O(p^2L)} per
iteration (not including the cost of computing the sufficient
statistics, which is dominated by the \eqn{O(np^2)} cost of
computing \eqn{X'X}). Because of the cost of computing \eqn{X'X},
\code{susie} will usually be faster. However, if \eqn{n >> p},
and/or if \eqn{X'X} is already computed, then
\code{susie_suff_stat} may be faster.
}
\examples{
# susie example
set.seed(1)
n = 1000
p = 1000
beta = rep(0,p)
beta[1:4] = 1
X = matrix(rnorm(n*p),nrow = n,ncol = p)
X = scale(X,center = TRUE,scale = TRUE)
y = drop(X \%*\% beta + rnorm(n))
res1 = susie(X,y,L = 10)
susie_get_cs(res1) # extract credible sets from fit
plot(beta,coef(res1)[-1])
abline(a = 0,b = 1,col = "skyblue",lty = "dashed")
plot(y,predict(res1))
abline(a = 0,b = 1,col = "skyblue",lty = "dashed")

# susie_suff_stat example
input_ss = compute_suff_stat(X,y)
res2 = with(input_ss,
            susie_suff_stat(XtX = XtX,Xty = Xty,yty = yty,n = n,
                            X_colmeans = X_colmeans,y_mean = y_mean,L = 10))
plot(coef(res1),coef(res2))
abline(a = 0,b = 1,col = "skyblue",lty = "dashed")

}
\references{
G. Wang, A. Sarkar, P. Carbonetto and M. Stephens (2020). A simple
new approach to variable selection in regression, with application
to genetic fine-mapping. \emph{Journal of the Royal Statistical
Society, Series B} \bold{82}, 1273-1300 \doi{10.1101/501114}.

Y. Zou, P. Carbonetto, G. Wang, G and M. Stephens
(2022). Fine-mapping from summary data with the \dQuote{Sum of
Single Effects} model. \emph{PLoS Genetics} \bold{18},
e1010299. \doi{10.1371/journal.pgen.1010299}.
}
\seealso{
\code{\link{susie_get_cs}} and other \code{susie_get_*}
functions for extracting results; \code{\link{susie_trendfilter}} for
applying the SuSiE model to non-parametric regression, particularly
changepoint problems, and \code{\link{susie_rss}} for applying the
SuSiE model when one only has access to limited summary statistics
related to \eqn{X} and \eqn{y} (typically in genetic applications).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/single_effect_regression.R,
%   R/single_effect_regression_rss.R, R/single_effect_regression_ss.R
\name{single_effect_regression}
\alias{single_effect_regression}
\alias{single_effect_regression_rss}
\alias{single_effect_regression_ss}
\title{Bayesian single-effect linear regression}
\usage{
single_effect_regression(
  y,
  X,
  V,
  residual_variance = 1,
  prior_weights = NULL,
  optimize_V = c("none", "optim", "uniroot", "EM", "simple"),
  check_null_threshold = 0
)

single_effect_regression_rss(
  z,
  Sigma,
  V = 1,
  prior_weights = NULL,
  optimize_V = c("none", "optim", "uniroot", "EM", "simple"),
  check_null_threshold = 0
)

single_effect_regression_ss(
  Xty,
  dXtX,
  V = 1,
  residual_variance = 1,
  prior_weights = NULL,
  optimize_V = c("none", "optim", "uniroot", "EM", "simple"),
  check_null_threshold = 0
)
}
\arguments{
\item{y}{An n-vector.}

\item{X}{An n by p matrix of covariates.}

\item{V}{A scalar giving the (initial) prior variance}

\item{residual_variance}{The residual variance.}

\item{prior_weights}{A p-vector of prior weights.}

\item{optimize_V}{The optimization method to use for fitting the
prior variance.}

\item{check_null_threshold}{Scalar specifying threshold on the
log-scale to compare likelihood between current estimate and zero
the null.}

\item{z}{A p-vector of z scores.}

\item{Sigma}{\code{residual_var*R + lambda*I}}

\item{Xty}{A p-vector.}

\item{dXtX}{A p-vector containing the diagonal elements of
\code{crossprod(X)}.}
}
\value{
A list with the following elements:

\item{alpha}{Vector of posterior inclusion probabilities;
  \code{alpha[i]} is posterior probability that the ith coefficient
  is non-zero.}

\item{mu}{Vector of posterior means (conditional on inclusion).}

\item{mu2}{Vector of posterior second moments (conditional on
  inclusion).}

\item{lbf}{Vector of log-Bayes factors for each variable.}

\item{lbf_model}{Log-Bayes factor for the single effect regression.}

\code{single_effect_regression} and \code{single_effect_regression_ss}
additionally output:

\item{V}{Prior variance (after optimization if \code{optimize_V !=
  "none"}).}

\item{loglik}{The log-likelihood, \eqn{\log p(y | X, V)}.}
}
\description{
These methods fit the regression model \eqn{y = Xb +
  e}, where elements of e are \emph{i.i.d.}  \eqn{N(0,s^2)}, and b is
  a p-vector of effects to be estimated. The assumption is that b has
  exactly one non-zero element, with all elements equally likely to
  be non-zero. The prior on the coefficient of the non-zero element
  is \eqn{N(0,V)}.
}
\details{
\code{single_effect_regression_ss} performs single-effect
linear regression with summary data, in which only the statistcs
\eqn{X^Ty} and diagonal elements of \eqn{X^TX} are provided to the
method.

\code{single_effect_regression_rss} performs single-effect linear
regression with z scores. That is, this function fits the
regression model \eqn{z = R*b + e}, where e is \eqn{N(0,Sigma)},
\eqn{Sigma = residual_var*R + lambda*I}, and the b is a p-vector of
effects to be estimated. The assumption is that b has exactly one
non-zero element, with all elements equally likely to be non-zero.
The prior on the non-zero element is \eqn{N(0,V)}. The required
summary data are the p-vector \code{z} and the p by p matrix
\code{Sigma}. The summary statistics should come from the same
individuals.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/susie_utils.R
\name{susie_get_objective}
\alias{susie_get_objective}
\alias{susie_get_posterior_mean}
\alias{susie_get_posterior_sd}
\alias{susie_get_niter}
\alias{susie_get_prior_variance}
\alias{susie_get_residual_variance}
\alias{susie_get_lfsr}
\alias{susie_get_posterior_samples}
\alias{susie_get_cs}
\alias{susie_get_pip}
\title{Inferences From Fitted SuSiE Model}
\usage{
susie_get_objective(res, last_only = TRUE, warning_tol = 1e-06)

susie_get_posterior_mean(res, prior_tol = 1e-09)

susie_get_posterior_sd(res, prior_tol = 1e-09)

susie_get_niter(res)

susie_get_prior_variance(res)

susie_get_residual_variance(res)

susie_get_lfsr(res)

susie_get_posterior_samples(susie_fit, num_samples)

susie_get_cs(
  res,
  X = NULL,
  Xcorr = NULL,
  coverage = 0.95,
  min_abs_corr = 0.5,
  dedup = TRUE,
  squared = FALSE,
  check_symmetric = TRUE,
  n_purity = 100,
  use_rfast
)

susie_get_pip(res, prune_by_cs = FALSE, prior_tol = 1e-09)
}
\arguments{
\item{res}{A susie fit, typically an output from
\code{\link{susie}} or one of its variants. For
\code{susie_get_pip} and \code{susie_get_cs}, this may instead be
the posterior inclusion probability matrix, \code{alpha}.}

\item{last_only}{If \code{last_only = FALSE}, return the ELBO from
all iterations; otherwise return the ELBO from the last iteration
only.}

\item{warning_tol}{Warn if ELBO is decreasing by this
tolerance level.}

\item{prior_tol}{Filter out effects having estimated prior variance
smaller than this threshold.}

\item{susie_fit}{A susie fit, an output from \code{\link{susie}}.}

\item{num_samples}{The number of draws from the posterior
distribution.}

\item{X}{n by p matrix of values of the p variables (covariates) in
n samples. When provided, correlation between variables will be
computed and used to remove CSs whose minimum correlation among
variables is smaller than \code{min_abs_corr}.}

\item{Xcorr}{p by p matrix of correlations between variables
(covariates). When provided, it will be used to remove CSs whose
minimum correlation among variables is smaller than
\code{min_abs_corr}.}

\item{coverage}{A number between 0 and 1 specifying desired
coverage of each CS.}

\item{min_abs_corr}{A "purity" threshold for the CS. Any CS that
contains a pair of variables with correlation less than this
threshold will be filtered out and not reported.}

\item{dedup}{If \code{dedup = TRUE}, remove duplicate CSs.}

\item{squared}{If \code{squared = TRUE}, report min, mean and
median of squared correlation instead of the absolute correlation.}

\item{check_symmetric}{If \code{check_symmetric = TRUE}, perform a
check for symmetry of matrix \code{Xcorr} when \code{Xcorr} is
provided (not \code{NULL}).}

\item{n_purity}{The maximum number of credible set (CS) variables
used in calculating the correlation (\dQuote{purity})
statistics. When the number of variables included in the CS is
greater than this number, the CS variables are randomly subsampled.}

\item{use_rfast}{Use the Rfast package for the purity calculations.
By default \code{use_rfast = TRUE} if the Rfast package is
installed.}

\item{prune_by_cs}{Whether or not to ignore single effects not in
a reported CS when calculating PIP.}
}
\value{
\code{susie_get_objective} returns the evidence lower bound
(ELBO) achieved by the fitted susie model and, optionally, at each
iteration of the IBSS fitting procedure.

\code{susie_get_residual_variance} returns the (estimated or
fixed) residual variance parameter.

\code{susie_get_prior_variance} returns the (estimated or fixed)
prior variance parameters.

\code{susie_get_posterior_mean} returns the posterior mean for the
regression coefficients of the fitted susie model.

\code{susie_get_posterior_sd} returns the posterior standard
deviation for coefficients of the fitted susie model.

\code{susie_get_niter} returns the number of model fitting
iterations performed.

\code{susie_get_pip} returns a vector containing the posterior
inclusion probabilities (PIPs) for all variables.

\code{susie_get_lfsr} returns a vector containing the average lfsr
across variables for each single-effect, weighted by the posterior
inclusion probability (alpha).

\code{susie_get_posterior_samples} returns a list containing the
effect sizes samples and causal status with two components: \code{b},
an \code{num_variables} x \code{num_samples} matrix of effect
sizes; \code{gamma}, an \code{num_variables} x \code{num_samples}
matrix of causal status random draws.

\code{susie_get_cs} returns credible sets (CSs) from a susie fit,
as well as summaries of correlation among the variables included in
each CS. If desired, one can filter out CSs that do not meet a
specified \dQuote{purity} threshold; to do this, either \code{X} or
\code{Xcorr} must be supplied. It returns a list with the following
elements:

\item{cs}{A list in which each list element is a vector containing
  the indices of the variables in the CS.}

\item{coverage}{The nominal coverage specified for each CS.}

\item{purity}{If \code{X} or \code{Xcorr} iis provided), the
  purity of each CS.}

\item{cs_index}{If \code{X} or \code{Xcorr} is provided) the index
  (number between 1 and L) of each reported CS in the supplied susie
  fit.}
}
\description{
These functions access basic properties or draw
  inferences from a fitted susie model.
}
\examples{
set.seed(1)
n = 1000
p = 1000
beta = rep(0,p)
beta[1:4] = 1
X = matrix(rnorm(n*p),nrow = n,ncol = p)
X = scale(X,center = TRUE,scale = TRUE)
y = drop(X \%*\% beta + rnorm(n))
s = susie(X,y,L = 10)
susie_get_objective(s)
susie_get_objective(s, last_only=FALSE)
susie_get_residual_variance(s)
susie_get_prior_variance(s)
susie_get_posterior_mean(s)
susie_get_posterior_sd(s)
susie_get_niter(s)
susie_get_pip(s)
susie_get_lfsr(s)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/susie_utils.R
\name{get_cs_correlation}
\alias{get_cs_correlation}
\title{Get Correlations Between CSs, using Variable with Maximum PIP From Each CS}
\usage{
get_cs_correlation(model, X = NULL, Xcorr = NULL, max = FALSE)
}
\arguments{
\item{model}{A SuSiE fit, typically an output from
\code{\link{susie}} or one of its variants.}

\item{X}{n by p matrix of values of the p variables (covariates) in
n samples. When provided, correlation between variables will be
computed and used to remove CSs whose minimum correlation among
variables is smaller than \code{min_abs_corr}.}

\item{Xcorr}{p by p matrix of correlations between variables
(covariates). When provided, it will be used to remove CSs whose
minimum correlation among variables is smaller than
\code{min_abs_corr}.}

\item{max}{When \code{max = FAFLSE}, return a matrix of CS
correlations. When \code{max = TRUE}, return only the maximum
absolute correlation among all pairs of correlations.}
}
\value{
A matrix of correlations between CSs, or the maximum
  absolute correlation when \code{max = TRUE}.
}
\description{
This function evaluates the correlation between single effect
  CSs. It is not part of the SuSiE inference. Rather, it is designed as
  a diagnostic tool to assess how correlated the reported CS are.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/predict.susie.R
\name{predict.susie}
\alias{predict.susie}
\title{Predict outcomes or extract coefficients from susie fit.}
\usage{
\method{predict}{susie}(object, newx = NULL, type = c("response", "coefficients"), ...)
}
\arguments{
\item{object}{A susie fit.}

\item{newx}{A new value for X at which to do predictions.}

\item{type}{The type of output. For \code{type = "response"},
predicted or fitted outcomes are returned; for \code{type =
"coefficients"}, the estimated coefficients are returned.}

\item{\dots}{Other arguments used by generic predict function. These
extra arguments are not used here.}
}
\value{
For \code{type = "response"}, predicted or fitted outcomes
  are returned; for \code{type = "coefficients"}, the estimated
  coefficients are returned. If the susie fit has intercept =
  \code{NA} (which is common when using \code{susie_suff_stat}) then
  predictions are computed using an intercept of 0, and a warning is
  emitted.
}
\description{
Predict outcomes or extract coefficients from susie fit.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/susie_plot_changepoint.R
\name{susie_plot_changepoint}
\alias{susie_plot_changepoint}
\title{Plot changepoint data and susie fit using ggplot2}
\usage{
susie_plot_changepoint(
  s,
  y,
  line_col = "blue",
  line_size = 1.5,
  cs_col = "red"
)
}
\arguments{
\item{s}{A susie fit generated by
\code{susie_trendfilter(y,order = 0)}.}

\item{y}{An n-vector of observations that are ordered in time or
space (assumed equally-spaced).}

\item{line_col}{Color for the line showing fitted values.}

\item{line_size}{Size of the lines showing fitted values}

\item{cs_col}{Color of the shaded rectangles showing credible
sets.}
}
\value{
A ggplot2 plot object.
}
\description{
Plots original data, y, overlaid with line showing
  susie fitted value and shaded rectangles showing credible sets for
  changepoint locations.
}
\examples{
set.seed(1)
mu = c(rep(0,50),rep(1,50),rep(3,50),rep(-2,50),rep(0,300))
y = mu + rnorm(500)
# Here we use a less sensitive tolerance so that the example takes
# less time; in practice you will likely want to use a more stringent
# setting such as tol = 0.001.
s = susie_trendfilter(y,tol = 0.1)

# Produces ggplot with credible sets for changepoints.
susie_plot_changepoint(s,y) 

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/initialize.R
\name{susie_init_coef}
\alias{susie_init_coef}
\title{Initialize a susie object using regression coefficients}
\usage{
susie_init_coef(coef_index, coef_value, p)
}
\arguments{
\item{coef_index}{An L-vector containing the the indices of the
nonzero coefficients.}

\item{coef_value}{An L-vector containing initial coefficient
estimates.}

\item{p}{A scalar giving the number of variables.}
}
\value{
A list with elements \code{alpha}, \code{mu} and \code{mu2}
  to be used by \code{susie}.
}
\description{
Initialize a susie object using regression coefficients
}
\examples{
set.seed(1)
n = 1000
p = 1000
beta = rep(0,p)
beta[sample(1:1000,4)] = 1
X = matrix(rnorm(n*p),nrow = n,ncol = p)
X = scale(X,center = TRUE,scale = TRUE)
y = drop(X \%*\% beta + rnorm(n))

# Initialize susie to ground-truth coefficients.
s = susie_init_coef(which(beta != 0),beta[beta != 0],length(beta))
res = susie(X,y,L = 10,s_init=s)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/susie_rss.R
\name{susie_rss}
\alias{susie_rss}
\title{Sum of Single Effects (SuSiE) Regression using Summary Statistics}
\usage{
susie_rss(
  z,
  R,
  n,
  bhat,
  shat,
  var_y,
  z_ld_weight = 0,
  estimate_residual_variance = FALSE,
  prior_variance = 50,
  check_prior = TRUE,
  ...
)
}
\arguments{
\item{z}{p-vector of z-scores.}

\item{R}{p x p correlation matrix.}

\item{n}{The sample size.}

\item{bhat}{Alternative summary data giving the estimated effects
(a vector of length p). This, together with \code{shat}, may be
provided instead of \code{z}.}

\item{shat}{Alternative summary data giving the standard errors of
the estimated effects (a vector of length p). This, together with
\code{bhat}, may be provided instead of \code{z}.}

\item{var_y}{The sample variance of y, defined as \eqn{y'y/(n-1)}.
When the sample variance is not provided, the coefficients
(returned from \code{coef}) are computed on the
\dQuote{standardized} X, y scale.}

\item{z_ld_weight}{This parameter is included for backwards
compatibility with previous versions of the function, but it is no
longer recommended to set this to a non-zero value. When
\code{z_ld_weight > 0}, the matrix \code{R} is adjusted to be
\code{cov2cor((1-w)*R + w*tcrossprod(z))}, where \code{w =
z_ld_weight}.}

\item{estimate_residual_variance}{The default is FALSE, the
residual variance is fixed to 1 or variance of y. If the in-sample
LD matrix is provided, we recommend setting
\code{estimate_residual_variance = TRUE}.}

\item{prior_variance}{The prior variance(s) for the non-zero
noncentrality parameterss \eqn{\tilde{b}_l}. It is either a scalar,
or a vector of length L. When the \code{susie_suff_stat} option
\code{estimate_prior_variance} is set to \code{TRUE} (which is
highly recommended) this simply provides an initial value for the
prior variance. The default value of 50 is simply intended to be a
large initial value. Note this setting is only relevant when
\code{n} is unknown. If \code{n} is known, the relevant option is
\code{scaled_prior_variance} in \code{\link{susie_suff_stat}}.}

\item{check_prior}{When \code{check_prior = TRUE}, it checks if the
estimated prior variance becomes unreasonably large (comparing with
100 * max(abs(z))^2).}

\item{\dots}{Other parameters to be passed to
\code{\link{susie_suff_stat}}.}
}
\value{
A \code{"susie"} object with the following
  elements:

\item{alpha}{An L by p matrix of posterior inclusion probabilites.}

\item{mu}{An L by p matrix of posterior means, conditional on
  inclusion.}

\item{mu2}{An L by p matrix of posterior second moments,
  conditional on inclusion.}

\item{lbf}{log-Bayes Factor for each single effect.}

\item{lbf_variable}{log-Bayes Factor for each variable and single effect.}

\item{V}{Prior variance of the non-zero elements of b.}

\item{elbo}{The value of the variational lower bound, or
  \dQuote{ELBO} (objective function to be maximized), achieved at
  each iteration of the IBSS fitting procedure.}

\item{sets}{Credible sets estimated from model fit; see
  \code{\link{susie_get_cs}} for details.}

\item{pip}{A vector of length p giving the (marginal) posterior
  inclusion probabilities for all p covariates.}

\item{niter}{Number of IBSS iterations that were performed.}

\item{converged}{\code{TRUE} or \code{FALSE} indicating whether
  the IBSS converged to a solution within the chosen tolerance
  level.}
}
\description{
\code{susie_rss} performs variable selection under a
  sparse Bayesian multiple linear regression of \eqn{Y} on \eqn{X}
  using the z-scores from standard univariate regression
  of \eqn{Y} on each column of \eqn{X}, an estimate, \eqn{R}, of
  the correlation matrix for the columns of \eqn{X}, and optionally,
  \emph{but strongly recommended}, the sample size n. See
  \dQuote{Details} for other ways to call \code{susie_rss}
}
\details{
In some applications, particularly genetic applications,
it is desired to fit a regression model (\eqn{Y = Xb + E} say,
which we refer to as "the original regression model" or ORM)
without access to the actual values of \eqn{Y} and \eqn{X}, but
given only some summary statistics. \code{susie_rss} assumes
availability of z-scores from standard univariate regression of
\eqn{Y} on each column of \eqn{X}, and an estimate, \eqn{R}, of the
correlation matrix for the columns of \eqn{X} (in genetic
applications \eqn{R} is sometimes called the \dQuote{LD matrix}).

With the inputs \code{z}, \code{R} and sample size \code{n},
\code{susie_rss} computes PVE-adjusted z-scores \code{z_tilde}, and
calls \code{susie_suff_stat} with \code{XtX = (n-1)R}, \code{Xty =
} \eqn{\sqrt{n-1} z_tilde}, \code{yty = n-1}, \code{n = n}. The
output effect estimates are on the scale of \eqn{b} in the ORM with
\emph{standardized} \eqn{X} and \eqn{y}. When the LD matrix
\code{R} and the z-scores \code{z} are computed using the same
matrix \eqn{X}, the results from \code{susie_rss} are same as, or
very similar to, \code{susie} with \emph{standardized} \eqn{X} and
\eqn{y}.

Alternatively, if the user provides \code{n}, \code{bhat} (the
univariate OLS estimates from regressing \eqn{y} on each column of
\eqn{X}), \code{shat} (the standard errors from these OLS
regressions), the in-sample correlation matrix \eqn{R =
cov2cor(crossprod(X))}, and the variance of \eqn{y}, the results
from \code{susie_rss} are same as \code{susie} with \eqn{X} and
\eqn{y}. The effect estimates are on the same scale as the
coefficients \eqn{b} in the ORM with \eqn{X} and \eqn{y}.

In rare cases in which the sample size, \eqn{n}, is unknown,
\code{susie_rss} calls \code{susie_suff_stat} with \code{XtX = R}
and \code{Xty = z}, and with \code{residual_variance = 1}. The
underlying assumption of performing the analysis in this way is
that the sample size is large (\emph{i.e.}, infinity), and/or the
effects are small. More formally, this combines the log-likelihood
for the noncentrality parameters, \eqn{\tilde{b} = \sqrt{n} b},
\deqn{L(\tilde{b}; z, R) = -(\tilde{b}'R\tilde{b} -
2z'\tilde{b})/2,} with the \dQuote{susie prior} on
\eqn{\tilde{b}}; see \code{\link{susie}} and Wang \emph{et al}
(2020) for details. In this case, the effect estimates returned by
\code{susie_rss} are on the noncentrality parameter scale.

The \code{estimate_residual_variance} setting is \code{FALSE} by
default, which is recommended when the LD matrix is estimated from
a reference panel. When the LD matrix \code{R} and the summary
statistics \code{z} (or \code{bhat}, \code{shat}) are computed
using the same matrix \eqn{X}, we recommend setting
\code{estimate_residual_variance = TRUE}.
}
\examples{
set.seed(1)
n = 1000
p = 1000
beta = rep(0,p)
beta[1:4] = 1
X = matrix(rnorm(n*p),nrow = n,ncol = p)
X = scale(X,center = TRUE,scale = TRUE)
y = drop(X \%*\% beta + rnorm(n))

input_ss = compute_suff_stat(X,y,standardize = TRUE)
ss   = univariate_regression(X,y)
R    = with(input_ss,cov2cor(XtX))
zhat = with(ss,betahat/sebetahat)
res  = susie_rss(zhat,R, n=n)

# Toy example illustrating behaviour susie_rss when the z-scores
# are mostly consistent with a non-invertible correlation matrix.
# Here the CS should contain both variables, and two PIPs should
# be nearly the same.
z = c(6,6.01)
R = matrix(1,2,2)
fit = susie_rss(z,R)
print(fit$sets$cs)
print(fit$pip)

# In this second toy example, the only difference is that one
# z-score is much larger than the other. Here we expect that the
# second PIP will be much larger than the first.
z = c(6,7)
R = matrix(1,2,2)
fit = susie_rss(z,R)
print(fit$sets$cs)
print(fit$pip)

}
\references{
G. Wang, A. Sarkar, P. Carbonetto and M. Stephens (2020). A simple
  new approach to variable selection in regression, with application
  to genetic fine-mapping. \emph{Journal of the Royal Statistical
  Society, Series B} \bold{82}, 1273-1300 \doi{10.1101/501114}.

  Y. Zou, P. Carbonetto, G. Wang and M. Stephens (2021).
  Fine-mapping from summary data with the \dQuote{Sum of Single Effects}
  model. \emph{bioRxiv} \doi{10.1101/2021.11.03.467167}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/susie_trendfilter.R
\name{susie_trendfilter}
\alias{susie_trendfilter}
\title{Apply susie to trend filtering (especially changepoint
  problems), a type of non-parametric regression.}
\usage{
susie_trendfilter(y, order = 0, standardize = FALSE, use_mad = TRUE, ...)
}
\arguments{
\item{y}{An n-vector of observations ordered in time or space
(assumed to be equally spaced).}

\item{order}{An integer specifying the order of trend filtering.
The default, \code{order = 0}, corresponds to "changepoint"
problems (\emph{i.e.}, piecewise constant \eqn{mu}). Although
\code{order > 0} is implemented, we do not recommend its use; in
practice, we have found problems with convergence of the algorithm
to poor local optima, producing unreliable inferences.}

\item{standardize}{Logical indicating whether to standardize the X
variables ("basis functions"); \code{standardize = FALSE} is
recommended as these basis functions already have a natural scale.}

\item{use_mad}{Logical indicating whether to use the "median
absolute deviation" (MAD) method to the estimate residual
variance. If \code{use_mad = TRUE}, susie is run twice, first by
fixing the residual variance to the MAD value, then a second time,
initialized to the first fit, but with residual variance estimated
the usual way (by maximizing the ELBO). We have found this strategy
typically improves reliability of the results by reducing a
tendency to converge to poor local optima of the ELBO.}

\item{...}{Other arguments passed to \code{\link{susie}}.}
}
\value{
A "susie" fit; see \code{\link{susie}} for details.
}
\description{
Fits the non-parametric Gaussian regression model
  \eqn{y = mu + e}, where the mean \eqn{mu} is modelled as \eqn{mu =
  Xb}, X is a matrix with columns containing an appropriate basis,
  and b is vector with a (sparse) SuSiE prior. In particular, when
  \code{order = 0}, the jth column of X is a vector with the first j
  elements equal to zero, and the remaining elements equal to 1, so
  that \eqn{b_j} corresponds to the change in the mean of y between
  indices j and j+1. For background on trend filtering, see
  Tibshirani (2014). See also the "Trend filtering" vignette,
  \code{vignette("trend_filtering")}.
}
\details{
This implementation exploits the special structure of X,
  which means that the matrix-vector product \eqn{X^Ty} is fast to
  compute; in particular, the computation time is \eqn{O(n)} rather
  than \eqn{O(n^2)} if \code{X} were formed explicitly. For
  implementation details, see the "Implementation of SuSiE trend
  filtering" vignette by running
  \code{vignette("trendfiltering_derivations")}.
}
\examples{
set.seed(1)
mu = c(rep(0,50),rep(1,50),rep(3,50),rep(-2,50),rep(0,200))
y = mu + rnorm(400)
s = susie_trendfilter(y)
plot(y)
lines(mu,col = 1,lwd = 3)
lines(predict(s),col = 2,lwd = 2)

# Calculate credible sets (indices of y that occur just before
# changepoints).
susie_get_cs(s)

# Plot with credible sets for changepoints.
susie_plot_changepoint(s,y) 

}
\references{
R. J. Tibshirani (2014). Adaptive piecewise polynomial
  estimation via trend filtering. \emph{Annals of Statistics}
  \bold{42}, 285-323.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/susie_plots.R
\name{susie_plot}
\alias{susie_plot}
\alias{susie_plot_iteration}
\title{SuSiE Plots.}
\usage{
susie_plot(
  model,
  y,
  add_bar = FALSE,
  pos = NULL,
  b = NULL,
  max_cs = 400,
  add_legend = NULL,
  ...
)

susie_plot_iteration(model, L, file_prefix, pos = NULL)
}
\arguments{
\item{model}{A SuSiE fit, typically an output from
\code{\link{susie}} or one of its variants. For \code{suse_plot},
the susie fit must have \code{model$z}, \code{model$PIP}, and may
include \code{model$sets}. \code{model} may also be a vector of
z-scores or PIPs.}

\item{y}{A string indicating what to plot: either \code{"z_original"} for
z-scores, \code{"z"} for z-score derived p-values on (base-10) log-scale, 
\code{"PIP"} for posterior inclusion probabilities,
\code{"log10PIP"} for posterior inclusion probabiliities on the
(base-10) log-scale. For any other setting, the data are plotted as
is.}

\item{add_bar}{If \code{add_bar = TRUE}, add horizontal bar to
signals in credible interval.}

\item{pos}{Indices of variables to plot. If \code{pos = NULL} all
variables are plotted.}

\item{b}{For simulated data, set \code{b = TRUE} to highlight
"true" effects (highlights in red).}

\item{max_cs}{The largest credible set to display, either based on
purity (set \code{max_cs} between 0 and 1), or based on size (set
\code{max_cs > 1}).}

\item{add_legend}{If \code{add_legend = TRUE}, add a legend to
annotate the size and purity of each CS discovered. It can also be
specified as location where legends should be added, e.g.,
\code{add_legend = "bottomright"} (default location is
\code{"topright"}).}

\item{\dots}{Additional arguments passed to
\code{\link[graphics]{plot}}.}

\item{L}{An integer specifying the number of credible sets to plot.}

\item{file_prefix}{Prefix to path of output plot file. If not
specified, the plot, or plots, will be saved to a temporary
directory generated using \code{\link{tempdir}}.}
}
\value{
Invisibly returns \code{NULL}.
}
\description{
\code{susie_plot} produces a per-variable summary of
  the SuSiE credible sets. \code{susie_plot_iteration} produces a
  diagnostic plot for the susie model fitting. For
  \code{susie_plot_iteration}, several plots will be created if
  \code{track_fit = TRUE} when calling \code{susie}.
}
\examples{
set.seed(1)
n = 1000
p = 1000
beta = rep(0,p)
beta[sample(1:1000,4)] = 1
X = matrix(rnorm(n*p),nrow = n,ncol = p)
X = scale(X,center = TRUE,scale = TRUE)
y = drop(X \%*\% beta + rnorm(n))
res = susie(X,y,L = 10)
susie_plot(res,"PIP")
susie_plot(res,"PIP",add_bar = TRUE)
susie_plot(res,"PIP",add_legend = TRUE)
susie_plot(res,"PIP", pos=1:500, add_legend = TRUE)
# Plot selected regions with adjusted x-axis position label
res$genomic_position = 1000 + (1:length(res$pip))
susie_plot(res,"PIP",add_legend = TRUE,
           pos = list(attr = "genomic_position",start = 1000,end = 1500))
# True effects are shown in red.
susie_plot(res,"PIP",b = beta,add_legend = TRUE)

set.seed(1)
n = 1000
p = 1000
beta = rep(0,p)
beta[sample(1:1000,4)] = 1
X = matrix(rnorm(n*p),nrow = n,ncol = p)
X = scale(X,center = TRUE,scale = TRUE)
y = drop(X \%*\% beta + rnorm(n))
res = susie(X,y,L = 10)
susie_plot_iteration(res, L=10)

}
\seealso{
\code{\link{susie_plot_changepoint}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{SummaryConsistency}
\alias{SummaryConsistency}
\title{Simulated Fine-mapping Data with LD matrix From Reference Panel.}
\format{
\code{SummaryConsistency} is a list with the following
elements:

\describe{

  \item{z}{z-scores computed by fitting univariate simple regression
    variable-by-variable.}

  \item{ldref}{LD matrix estimated from the reference panel.}

  \item{flip_id}{The index of the SNP with the flipped allele.}

  \item{signal_id}{The index of the SNP with the non-zero effect.}}
}
\description{
Data simulated using real genotypes from 10,000
  individuals and 200 SNPs. One SNP have non-zero effect
  on the multivariate response. The response data are generated under
  a linear regression model. There is also one SNP with flipped allele
  between summary statistics and the reference panel.
}
\examples{
data(SummaryConsistency)
}
\seealso{
A similar data set with more samples is used in the
  \dQuote{Diagnostic for fine-mapping with summary statistics}
  vignette.
}
\keyword{data}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{N2finemapping}
\alias{N2finemapping}
\title{Simulated Fine-mapping Data with Two Effect Variables}
\format{
\code{N2finemapping} is a list with the following elements:

\describe{

  \item{X}{Centered and scaled genotype data.}

  \item{chrom}{Chromomsome of the original data, in hg38 coordinates.}

  \item{pos}{Chromomosomal position of the original data, in hg38
    coordinates. The information can be used to compare impact of using
    other genotype references of the same variables in \code{susie_rss}
    application.}

  \item{true_coef}{Simulated effect sizes.}

  \item{residual_variance}{Simulated residual covariance matrix.}

  \item{Y}{Simulated multivariate response.}

  \item{allele_freq}{Allele frequencies based on the original
    genotype data.}

  \item{V}{Suggested prior covariance matrix for effect sizes of
     the two non-zero effect variables.}
}
}
\description{
This data set contains a genotype matrix for 574
  individuals and 1,002 variables. The variables are genotypes after
  centering and scaling, and therefore retain the correlation
  structure of the original genotype data. Two of the variables have
  non-zero effects on the multivariate response. The response data
  are generated under a multivariate linear regression model. See
  Wang \emph{et al} (2020) for details.
}
\examples{
data(N2finemapping)
}
\references{
G. Wang, A. Sarkar, P. Carbonetto and M. Stephens (2020). A simple
  new approach to variable selection in regression, with application
  to genetic fine-mapping. \emph{Journal of the Royal Statistical
  Society, Series B} \doi{10.1101/501114}.
}
\keyword{data}
